@@ -1,4 +1,4 @@
-/* $Id: callable_objects.cpp 34673 2009-04-10 11:13:24Z dragonking $ */
+/* $Id: callable_objects.cpp 40329 2009-12-22 13:52:50Z silene $ */
 /*
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -24,8 +24,10 @@ variant convert_map( const std::map<T, K>& input_map ) {
 	return variant( &tmp );
 }
 
+
 template <typename T>
-variant convert_vector( const std::vector<T>& input_vector ) {
+variant convert_vector( const std::vector<T>& input_vector )
+{
 	std::vector<variant> tmp;
 
 	for(typename std::vector<T>::const_iterator i = input_vector.begin(); i != input_vector.end(); ++i) {
@@ -35,6 +37,7 @@ variant convert_vector( const std::vector<T>& input_vector ) {
 	return variant( &tmp );
 }
 
+
 variant location_callable::get_value(const std::string& key) const
 {
 	if(key == "x") {
@@ -61,11 +64,7 @@ int location_callable::do_compare(const game_logic::formula_callable* callable)
 	}
 
 	const map_location& other_loc = loc_callable->loc();
-	if(other_loc.x != loc_.x) {
-		return loc_.x - other_loc.x;
-	}
-
-	return loc_.y - other_loc.y;
+	return loc_.do_compare(other_loc);
 }
 
 void location_callable::serialize_to_string(std::string& str) const
@@ -75,31 +74,6 @@ void location_callable::serialize_to_string(std::string& str) const
 	str += s.str();
 }
 
-variant move_map_callable::get_value(const std::string& key) const
-{
-	using namespace game_logic;
-	if(key == "moves") {
-		std::vector<variant> vars;
-		for(move_map::const_iterator i = srcdst_.begin(); i != srcdst_.end(); ++i) {
-                        if( i->first == i->second || units_.count(i->second) == 0) {
-                            move_callable* item = new move_callable(i->first, i->second);
-                            vars.push_back(variant(item));
-                        }
-		}
-
-		return variant(&vars);
-	} else if(key == "has_moves") {
-		return variant(!srcdst_.empty());
-	} else {
-		return variant();
-	}
-}
-
-void move_map_callable::get_inputs(std::vector<game_logic::formula_input>* inputs) const
-{
-	using game_logic::FORMULA_READ_ONLY;
-	inputs->push_back(game_logic::formula_input("moves", FORMULA_READ_ONLY));
-}
 
 variant attack_type_callable::get_value(const std::string& key) const
 {
@@ -140,27 +114,63 @@ void attack_type_callable::get_inputs(std::vector<game_logic::formula_input>* in
 	inputs->push_back(game_logic::formula_input("special", FORMULA_READ_ONLY));
 }
 
+int attack_type_callable::do_compare(const formula_callable* callable) const
+{
+	const attack_type_callable* att_callable = dynamic_cast<const attack_type_callable*>(callable);
+	if(att_callable == NULL) {
+		return formula_callable::do_compare(callable);
+	}
+
+	if (att_.damage() != att_callable->att_.damage() )
+		return att_.damage() - att_callable->att_.damage();
+
+	if (att_.num_attacks() != att_callable->att_.num_attacks() )
+		return att_.num_attacks() - att_callable->att_.num_attacks();
+
+	if ( att_.id() != att_callable->att_.id() )
+		return att_.id().compare(att_callable->att_.id());
+
+	if ( att_.type() != att_callable->att_.type() )
+		return att_.type().compare(att_callable->att_.type());
+
+	if ( att_.range() != att_callable->att_.range() )
+		return att_.range().compare(att_callable->att_.range());
+
+	return att_.weapon_specials().compare(att_callable->att_.weapon_specials());
+}
+
 variant unit_callable::get_value(const std::string& key) const
 {
 	if(key == "x") {
-		return variant(loc_.x+1);
+		if (loc_==map_location::null_location) {
+			return variant();
+		} else {
+			return variant(loc_.x+1);
+		}
 	} else if(key == "y") {
-		return variant(loc_.y+1);
+		if (loc_==map_location::null_location) {
+			return variant();
+		} else {
+			return variant(loc_.y+1);
+		}
 	} else if(key == "loc") {
-		return variant(new location_callable(loc_));
+		if (loc_==map_location::null_location) {
+			return variant();
+		} else {
+			return variant(new location_callable(loc_));
+		}
 	} else if(key == "id") {
 		return variant(u_.id());
 	} else if(key == "type") {
 		return variant(u_.type_id());
 	} else if(key == "name") {
 		return variant(u_.name());
+	} else if(key == "usage") {
+		return variant(u_.usage());
 	} else if(key == "leader") {
 		return variant(u_.can_recruit());
 	} else if(key == "undead") {
-		if ( utils::string_bool(u_.get_state("not_living"))  )
-                    return variant( 1 );
-
-		return variant( 0 );
+		return variant(u_.get_state("not_living") ? 1 : 0);
 	} else if(key == "attacks") {
 		const std::vector<attack_type>& att = u_.attacks();
 		std::vector<variant> res;
@@ -194,6 +204,8 @@ variant unit_callable::get_value(const std::string& key) const
 		return variant(u_.total_movement());
 	} else if(key == "movement_left") {
 		return variant(u_.movement_left());
+	} else if(key == "attacks_left") {
+		return variant(u_.attacks_left());
 	} else if(key == "traits") {
 		const std::vector<std::string> traits = u_.get_traits_list();
 		std::vector<variant> res;
@@ -246,12 +258,24 @@ void unit_callable::get_inputs(std::vector<game_logic::formula_input>* inputs) c
 	inputs->push_back(game_logic::formula_input("level", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("total_movement", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("movement_left", FORMULA_READ_ONLY));
+	inputs->push_back(game_logic::formula_input("attacks_left", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("side", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("states", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("cost", FORMULA_READ_ONLY));
+	inputs->push_back(game_logic::formula_input("usage", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("vars", FORMULA_READ_ONLY));
 }
 
+int unit_callable::do_compare(const formula_callable* callable) const
+{
+	const unit_callable* u_callable = dynamic_cast<const unit_callable*>(callable);
+	if(u_callable == NULL) {
+		return formula_callable::do_compare(callable);
+	}
+
+	return u_.underlying_id() - u_callable->u_.underlying_id();
+}
+
 variant unit_type_callable::get_value(const std::string& key) const
 {
 	if(key == "id") {
@@ -291,6 +315,8 @@ variant unit_type_callable::get_value(const std::string& key) const
 		return variant(u_.not_living());
 	} else if(key == "cost") {
 		return variant(u_.cost());
+	} else if(key == "usage") {
+		return variant(u_.usage());
 	} else {
 		return variant();
 	}
@@ -310,6 +336,17 @@ void unit_type_callable::get_inputs(std::vector<game_logic::formula_input>* inpu
 	inputs->push_back(game_logic::formula_input("total_movement", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("undead", FORMULA_READ_ONLY));
 	inputs->push_back(game_logic::formula_input("cost", FORMULA_READ_ONLY));
+	inputs->push_back(game_logic::formula_input("usage", FORMULA_READ_ONLY));
+}
+
+int unit_type_callable::do_compare(const formula_callable* callable) const
+{
+	const unit_type_callable* u_callable = dynamic_cast<const unit_type_callable*>(callable);
+	if(u_callable == NULL) {
+		return formula_callable::do_compare(callable);
+	}
+
+	return u_.id().compare(u_callable->u_.id());
 }
 
 variant terrain_callable::get_value(const std::string& key) const
@@ -335,3 +372,17 @@ void terrain_callable::get_inputs(std::vector<game_logic::formula_input>* inputs
 	inputs->push_back(game_logic::formula_input("id", FORMULA_READ_ONLY));
 }
 
+int terrain_callable::do_compare(const formula_callable* callable) const
+{
+	const terrain_callable* terr_callable = dynamic_cast<const terrain_callable*>(callable);
+	if(terr_callable == NULL) {
+		return formula_callable::do_compare(callable);
+	}
+
+	const map_location& other_loc = terr_callable->loc_;
+
+	return loc_.do_compare(other_loc);
+}
+
+
+
