@@ -1,6 +1,6 @@
-/* $Id: color_range.cpp 40501 2010-01-01 17:11:57Z mordante $ */
+/* $Id: color_range.cpp 49342 2011-05-02 18:52:36Z crab $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include <cassert>
 #include <set>
+#include <iomanip>
 
 std::map<Uint32, Uint32> recolor_range(const color_range& new_range, const std::vector<Uint32>& old_rgb){
 	std::map<Uint32, Uint32> map_rgb;
@@ -161,6 +162,16 @@ std::string rgb2highlight(Uint32 rgb)
 	return h.str();
 }
 
+std::string rgb2highlight_pango(Uint32 rgb)
+{
+	std::ostringstream h;
+	// Must match what the pango expects
+	h << "#" << std::hex << std::setfill('0') << std::setw(2) << ((rgb & 0xFF0000) >> 16)
+	  << std::hex << std::setfill('0') << std::setw(2) <<((rgb & 0x00FF00) >> 8)
+	  << std::hex << std::setfill('0') << std::setw(2) <<(rgb & 0x0000FF);
+	return h.str();
+}
+
 int color_range::index() const
 {
 	for(int i = 1; i <= gamemap::MAX_PLAYERS; ++i) {
