@@ -1,6 +1,6 @@
-/* $Id: controller_base.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: controller_base.cpp 49411 2011-05-07 15:11:50Z soliton $ */
 /*
-   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -98,12 +98,6 @@ void controller_base::handle_event(const SDL_Event& event)
 				int x, y;
 				Uint8 mouse_flags = SDL_GetMouseState(&x, &y);
 				if ((mouse_flags & SDL_BUTTON_LEFT) == 0) {
-					SDL_Event e;
-					e.type = SDL_MOUSEBUTTONUP;
-					e.button.state = SDL_RELEASED;
-					e.button.button = SDL_BUTTON_LEFT;
-					e.button.x = x;
-					e.button.y = y;
 					get_mouse_handler_base().mouse_press(event.button, browse_);
 					post_mouse_press(event);
 				}
