@@ -1,6 +1,6 @@
-/* $Id: cursor.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: cursor.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,23 +12,19 @@
    See the COPYING file for more details.
 */
 
-//! @file cursor.cpp
-//! Support for different cursors-shapes.
+/**
+ * @file cursor.cpp
+ * Support for different cursors-shapes.
+ */
 
 #include "global.hpp"
 
 #include "cursor.hpp"
-#include "game_config.hpp"
 #include "image.hpp"
 #include "preferences_display.hpp"
-#include "scoped_resource.hpp"
-#include "sdl_utils.hpp"
 #include "video.hpp"
 
-#include "SDL.h"
-
 #include <iostream>
-#include <vector>
 
 static bool use_colour_cursors()
 {
@@ -61,25 +57,20 @@ static SDL_Cursor* create_cursor(surface surf)
 	const Uint32* const pixels = reinterpret_cast<Uint32*>(lock.pixels());
 	for(int y = 0; y != nsurf->h; ++y) {
 		for(int x = 0; x != nsurf->w; ++x) {
-			Uint8 r,g,b,a;
-			Uint8 trans = 0;
-			Uint8 black = 0;
-
-			const size_t index = y*cursor_width + x;
 
 			if (static_cast<size_t>(x) < cursor_width) {
+				Uint8 r,g,b,a;
 				SDL_GetRGBA(pixels[y*nsurf->w + x],nsurf->format,&r,&g,&b,&a);
 
+				const size_t index = y*cursor_width + x;
 				const size_t shift = 7 - (index % 8);
 
-				trans = (a < 128 ? 0 : 1) << shift;
-				black = (trans == 0 || (r+g + b) / 3 > 128 ? 0 : 1) << shift;
+				const Uint8 trans = (a < 128 ? 0 : 1) << shift;
+				const Uint8 black = (trans == 0 || (r+g + b) / 3 > 128 ? 0 : 1) << shift;
 
 				data[index/8] |= black;
 				mask[index/8] |= trans;
 			}
-
-
 		}
 	}
 
@@ -153,12 +144,6 @@ manager::~manager()
 	SDL_ShowCursor(SDL_ENABLE);
 }
 
-void temporary_use_bw()
-{
-	colour_ready = false;
-	set();
-}
-
 void set(CURSOR_TYPE type)
 {
 	// Change only if it's a valid cursor
@@ -178,6 +163,7 @@ void set(CURSOR_TYPE type)
 	//if (cursor_image != NULL && cursor_image != SDL_GetCursor())
 		SDL_SetCursor(cursor_image);
 
+    SDL_ShowCursor(SDL_ENABLE);
 }
 
 void set_dragging(bool drag)
@@ -247,7 +233,7 @@ void draw(surface screen)
 		set();
 	}
 
-	//! @todo FIXME: don't parse the file path every time
+	/** @todo FIXME: don't parse the file path every time */
 	const surface surf(image::get_image("cursors/" + colour_images[current_cursor]));
 	if(surf == NULL) {
 		// Fall back to b&w cursors
