@@ -1,6 +1,6 @@
-/* $Id: cursor.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: cursor.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -31,12 +31,6 @@ struct manager
 
 enum CURSOR_TYPE { NORMAL, WAIT, MOVE, ATTACK, HYPERLINK, MOVE_DRAG, ATTACK_DRAG, NO_CURSOR, NUM_CURSORS };
 
-/**
- * This function uses temporary the b&w cursors while the display is busy.
- * If used, the colour-cursors will be reused when the display is ready again.
- */
-void temporary_use_bw();
-
 /**
  * Use the default parameter to reset cursors.
  * e.g. after a change in color cursor preferences
