@@ -1,6 +1,6 @@
-/* $Id: display.cpp 42541 2010-05-10 22:52:59Z ai0867 $ */
+/* $Id: display.cpp 48934 2011-03-19 19:29:54Z boucman $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -52,6 +52,12 @@
 #include <iostream>
 #include <sstream>
 
+// Includes for bug #17573
+#if defined(__GLIBC__)
+#include <gnu/libc-version.h> 
+#include <cstdio>
+#endif
+
 static lg::log_domain log_display("display");
 #define ERR_DP LOG_STREAM(err, log_display)
 #define LOG_DP LOG_STREAM(info, log_display)
@@ -101,6 +107,7 @@ display::display(CVideo& video, const gamemap* map, const config& theme_cfg, con
 	fog_image_("terrain/" + get_map().get_terrain_info(t_translation::FOGGED).minimap_image() + ".png"),
 	tod_(time_of_day()),
 	scroll_event_("scrolled"),
+	complete_redraw_event_("completely_redrawn"),
 	nextDraw_(0),
 	report_(),
 	buttons_(),
@@ -138,6 +145,20 @@ display::display(CVideo& video, const gamemap* map, const config& theme_cfg, con
 	std::fill(reportRects_,reportRects_+reports::NUM_REPORTS,empty_rect);
 
 	image::set_zoom(zoom_);
+
+#if defined(__GLIBC__)
+	// Runtime checks for bug #17573
+	// Get glibc runtime version information
+	int glibc, glibc_minor; 
+	sscanf(gnu_get_libc_version(), "%d.%d", &glibc, &glibc_minor);
+
+	// Get SDL runtime version information
+	const SDL_version* v = SDL_Linked_Version();
+
+	do_reverse_memcpy_workaround_ = (glibc > 2 || (glibc == 2 && glibc_minor >= 13)) &&
+		(v->major < 1 || (v->major == 1 && v->minor < 2) || 
+			(v->major == 1 && v->minor == 2 && v->patch < 15) );
+#endif
 }
 
 display::~display()
@@ -1432,8 +1453,20 @@ bool display::scroll(int xmove, int ymove)
 	SDL_Rect srcrect = dstrect;
 	srcrect.x -= dx;
 	srcrect.y -= dy;
-	if (!screen_.update_locked())
+	if (!screen_.update_locked()) {
+
+// Hack to workaround bug #17573
+#if defined(__GLIBC__)
+		if (do_reverse_memcpy_workaround_) {
+			surface screen_copy = make_neutral_surface(screen);
+			SDL_BlitSurface(screen_copy,&srcrect,screen,&dstrect);
+		} else {
+			SDL_BlitSurface(screen,&srcrect,screen,&dstrect);
+		}
+#else 
 		SDL_BlitSurface(screen,&srcrect,screen,&dstrect);
+#endif
+	}
 
 //This is necessary to avoid a crash in some SDL versions on some systems
 //see http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=462794
@@ -1864,6 +1897,8 @@ void display::redraw_everything()
 	foreach (boost::function<void(display&)> f, redraw_observers_) {
 		f(*this);
 	}
+	
+	complete_redraw_event_.notify_observers();
 }
 
 void display::add_redraw_observer(boost::function<void(display&)> f)
