@@ -1,6 +1,6 @@
-/* $Id: display.hpp 35509 2009-05-09 16:25:14Z silene $ */
+/* $Id: display.hpp 48934 2011-03-19 19:29:54Z boucman $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -34,10 +34,6 @@
 #define DISPLAY_H_INCLUDED
 
 class config;
-class gamestatus;
-class team;
-class unit;
-class unit_map;
 class terrain_builder;
 class map_labels;
 
@@ -47,6 +43,7 @@ class map_labels;
 #include "key.hpp"
 #include "map_location.hpp"
 #include "reports.hpp"
+#include "team.hpp"
 #include "time_of_day.hpp"
 #include "theme.hpp"
 #include "video.hpp"
@@ -61,12 +58,13 @@ class map_labels;
 #include <boost/function.hpp>
 #include <boost/scoped_ptr.hpp>
 
+class gamemap;
 
 class display
 {
 public:
 	display(CVideo& video, const gamemap* map, const config& theme_cfg,
-			const config& cfg, const config& level);
+			const config& level);
 	virtual ~display();
 
 	/**
@@ -79,6 +77,14 @@ public:
 
 	static Uint32 rgb(Uint8 red, Uint8 green, Uint8 blue)
 		{ return 0xFF000000 | (red << 16) | (green << 8) | blue; }
+	static Uint8 red(Uint32 color)
+		{ return (color & 0x00FF0000) >> 16;}
+	static Uint8 green(Uint32 color)
+		{ return (color & 0x0000FF00) >> 8;}
+	static Uint8 blue(Uint32 color)
+		{ return (color & 0x000000FF) ;}
+	static Uint32 max_rgb(Uint32 first,Uint32 second)
+		{ return rgb(std::max(red(first),red(second)),std::max(green(first),green(second)),std::max(blue(first),blue(second))) ; }
 
 	/** Gets the underlying screen object. */
 	CVideo& video() { return screen_; }
@@ -141,7 +147,7 @@ public:
 	int hex_size() const { return zoom_; }
 
 	/** Returns the current zoom factor. */
-	double get_zoom_factor() { return double(zoom_)/double(image::tile_size); }
+	double get_zoom_factor() const { return double(zoom_)/double(image::tile_size); }
 
 	/**
 	 * given x,y co-ordinates of an onscreen pixel, will return the
@@ -164,8 +170,8 @@ public:
 	 */
 	map_location minimap_location_on(int x, int y);
 
-	const map_location& selected_hex() { return selectedHex_; }
-	const map_location& mouseover_hex() { return mouseoverHex_; }
+	const map_location& selected_hex() const { return selectedHex_; }
+	const map_location& mouseover_hex() const { return mouseoverHex_; }
 
 	virtual void select_hex(map_location hex);
 	virtual void highlight_hex(map_location hex);
@@ -189,21 +195,29 @@ public:
 
 		/**  very simple iterator to walk into the rect_of_hexes */
 		struct iterator {
-			iterator(map_location loc, rect_of_hexes& rect)
+			iterator(const map_location &loc, const rect_of_hexes &rect)
 				: loc_(loc), rect_(rect){};
 
 			/** increment y first, then when reaching bottom, increment x */
 			void operator++();
-			bool operator!=(const iterator &that) const {return that.loc_ != loc_;};
+			bool operator==(const iterator &that) const { return that.loc_ == loc_; }
+			bool operator!=(const iterator &that) const { return that.loc_ != loc_; }
 			const map_location& operator*() const {return loc_;};
 
+			typedef std::forward_iterator_tag iterator_category;
+			typedef map_location value_type;
+			typedef int difference_type;
+			typedef const map_location *pointer;
+			typedef const map_location &reference;
+
 			private:
 				map_location loc_;
-				rect_of_hexes& rect_;
+				const rect_of_hexes &rect_;
 		};
+		typedef iterator const_iterator;
 
-		iterator begin();
-		iterator end();
+		iterator begin() const;
+		iterator end() const;
 	};
 
 	/** Return the rectangular area of hexes overlapped by r (r is in screen coordinates) */
@@ -228,12 +242,12 @@ public:
 	void set_grid(const bool grid) { grid_ = grid; }
 
 	/** Getter for the x,y debug overlay on tiles */
-	bool get_draw_coordinates() { return draw_coordinates_; }
+	bool get_draw_coordinates() const { return draw_coordinates_; }
 	/** Setter for the x,y debug overlay on tiles */
 	void set_draw_coordinates(bool value) { draw_coordinates_ = value; }
 
 	/** Getter for the terrain code debug overlay on tiles */
-	bool get_draw_terrain_codes() { return draw_terrain_codes_; }
+	bool get_draw_terrain_codes() const { return draw_terrain_codes_; }
 	/** Setter for the terrain code debug overlay on tiles */
 	void set_draw_terrain_codes(bool value) { draw_terrain_codes_ = value; }
 
@@ -255,8 +269,7 @@ public:
 	void create_buttons();
 	void invalidate_theme() { panelsDrawn_ = false; }
 
-	void refresh_report(reports::TYPE report_num, reports::report report,
-		      bool brightened = false);
+	void refresh_report(reports::TYPE report_num, reports::report report);
 
 	// Will be overridden in the display subclass
 	virtual void draw_minimap_units() {};
@@ -269,16 +282,16 @@ public:
 
 	bool invalidate(const std::set<map_location>& locs);
 
+	/**
+	 * If this set is partially invalidated, invalidate all its hexes.
+	 * Returns if any new invalidation was needed
+	 */
+	bool propagate_invalidation(const std::set<map_location>& locs);
+
 	/** invalidate all hexes under the rectangle rect (in screen coordinates) */
 	bool invalidate_locations_in_rect(const SDL_Rect& rect);
 	bool invalidate_visible_locations_in_rect(const SDL_Rect& rect);
 
-	/** check if an hexes under the rectangle is invalidated */
-	bool rectangle_need_update(const SDL_Rect& rect) const;
-
-	/** check if an hex is invalidated */
-	bool hex_need_update(const map_location& loc) const;
-
 	/**
 	 * Function to invalidate animated terrains which may have changed.
 	 */
@@ -290,6 +303,14 @@ public:
 	 */
 	virtual void invalidate_animations_location(const map_location& /*loc*/) {}
 
+	/**
+	 * What hex are currently invalidated (read only)
+	 * used for some fine grained invalidation algorithm which need recurstion
+	 */
+	const std::set<map_location> & get_invalidated() { return invalidated_; }
+
+
+
 	const gamemap& get_map() const { return *map_; }
 
 	/**
@@ -302,11 +323,6 @@ public:
 	 * These functions require a prerendered surface.
 	 * Since they are drawn at the top, they are not influenced by TOD, shroud etc.
 	 */
-	void set_hex_overlay(const map_location& loc, const surface& image)
-		{ hex_overlay_[loc] = image; }
-
-	void clear_hex_overlay(const map_location& loc);
-
 	void set_selected_hex_overlay(const surface& image)
 		{ selected_hex_overlay_ = image; }
 
@@ -368,27 +384,15 @@ public:
 	void set_idle_anim_rate(int rate);
 	double idle_anim_rate() const { return idle_anim_rate_; }
 
-	/**
-	 * Add a location to highlight.
-	 *
-	 * Note that this has nothing to do with selecting hexes,
-	 * it is pure highlighting. These hexes will be highlighted
-	 * slightly darker than the currently selected hex.
-	 */
-	void add_highlighted_loc(const map_location &hex);
-
-	void clear_highlighted_locs();
-
-	void remove_highlighted_loc(const map_location &hex);
-
 	void bounds_check_position();
 	void bounds_check_position(int& xpos, int& ypos);
 
 	/**
 	 * Scrolls the display by xmov,ymov pixels.
 	 * Invalidation and redrawing will be scheduled.
+	 * @return true if the map actually moved.
 	 */
-	void scroll(int xmov, int ymov);
+	bool scroll(int xmov, int ymov);
 
 	/**
 	 * Zooms the display by the specified amount.
@@ -430,12 +434,14 @@ public:
 
 	/** Expose the event, so observers can be notified about map scrolling. */
 	events::generic_event &scroll_event() const { return scroll_event_; }
+	
+	events::generic_event& complete_redraw_event() { return complete_redraw_event_; }
 
-	/** Check if a tile is fully on screen. */
-	bool tile_on_screen(const map_location& loc);
+	/** Check if a tile is fully visible on screen. */
+	bool tile_fully_on_screen(const map_location& loc);
 
 	/** Checks if location @a loc or one of the adjacent tiles is visible on screen. */
-	bool tile_nearly_on_screen(const map_location &loc);
+	bool tile_nearly_on_screen(const map_location &loc) const;
 
 	/**
 	 * Draws invalidated items.
@@ -450,7 +456,7 @@ public:
 	const map_labels& labels() const;
 
 	/** Announce a message prominently. */
-	void announce(const std::string msg,
+	void announce(const std::string& msg,
 		       const SDL_Color& colour = font::GOOD_COLOUR);
 
 	/**
@@ -546,7 +552,7 @@ protected:
 	enum ADJACENT_TERRAIN_TYPE { ADJACENT_BACKGROUND, ADJACENT_FOREGROUND, ADJACENT_FOGSHROUD };
 
 	std::vector<surface> get_terrain_images(const map_location &loc,
-					const std::string timeid,
+					const std::string& timeid,
 					image::TYPE type,
 					ADJACENT_TERRAIN_TYPE terrain_type);
 
@@ -558,7 +564,7 @@ protected:
 
 	CVideo& screen_;
 	const gamemap* map_;
-	const viewpoint *viewpoint_;
+	const team *viewpoint_;
 	int xpos_, ypos_;
 	theme theme_;
 	int zoom_;
@@ -582,6 +588,12 @@ protected:
 
 	/** Event raised when the map is being scrolled */
 	mutable events::generic_event scroll_event_;
+	
+	/**
+	 * notify observers that the screen has been redrawn completely
+	 * atm this is used for replay_controller to add replay controls to the standard theme
+	 */
+	events::generic_event complete_redraw_event_;
 
 	/**
 	 * Holds the tick count for when the next drawing event is scheduled.
@@ -597,12 +609,10 @@ protected:
 	std::vector<gui::button> buttons_;
 	std::set<map_location> invalidated_;
 	std::set<map_location> previous_invalidated_;
-	std::map<map_location, surface> hex_overlay_;
 	surface selected_hex_overlay_;
 	surface mouseover_hex_overlay_;
 	map_location selectedHex_;
 	map_location mouseoverHex_;
-	std::set<map_location> highlighted_locations_;
 	CKey keys_;
 
 public:
@@ -692,7 +702,7 @@ public:
 	 * x,y: pixel location on screen to draw the unit
 	 * image: the image of the unit
 	 * reverse: if the unit should be flipped across the x axis
-	 * greyscale: used when the unit is stoned
+	 * greyscale: used when the unit is petrified
 	 * alpha: the merging to use with the background
 	 * blendto: blend to this colour using blend_ratio
 	 * submerged: the amount of the unit out of 1.0 that is submerged
@@ -813,6 +823,9 @@ protected:
 private:
 	/** Handle for the label which displays frames per second. */
 	int fps_handle_;
+	/** Count work done for the debug info displayed under fps */
+	int invalidated_hexes_;
+	int drawn_hexes_;
 
 	bool idle_anim_;
 	double idle_anim_rate_;
@@ -825,6 +838,11 @@ private:
 	bool draw_coordinates_;
 	/** Debug flag - overlay terrain codes on tiles */
 	bool draw_terrain_codes_;
+
+#if defined(__GLIBC__)
+	/** Flag for bug #17573 - this is set in the constructor **/
+	bool do_reverse_memcpy_workaround_;
+#endif
 };
 
 #endif
