@@ -1,6 +1,6 @@
-/* $Id: game_preferences.cpp 42333 2010-04-30 23:22:12Z loonycyborg $ */
+/* $Id: game_preferences.cpp 48505 2011-02-13 07:44:00Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -281,15 +281,12 @@ void _set_lobby_joins(int show)
 
 bool new_lobby()
 {
-	std::cerr << get("new_lobby") << "\n";
 	return utils::string_bool(get("new_lobby"), false);
 }
 
 void set_new_lobby(bool value)
 {
-	std::cerr << get("new_lobby") << value;
 	preferences::set("new_lobby", (value ? "yes" : "no"));
-	std::cerr << get("new_lobby") << "\n";
 }
 
 
