@@ -1,6 +1,6 @@
-/* $Id: gettext.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: gettext.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,23 +17,31 @@
 
 /**
  * How to use gettext for wesnoth source files:
- * 1) include this header file in the .cpp file
- * 2) add this include to set the correct textdomain, in this example
- *    wesnoth-lib (not required for the domain 'wesnoth', required for all
- *    other textdomains):
- *    #define GETTEXT_DOMAIN "wesnoth-lib"
- * 3) make sure, that the source file is listed in the respective POTFILES.in
+ * -# include this header file in the .cpp file
+ * -# make sure, that the source file is listed in the respective POTFILES.in
  *    for the textdomain, in the case of wesnoth-lib it is this file:
  *    po/wesnoth-lib/POTFILES.in
+ * -# add the following include to set the correct textdomain, in this example
+ *    wesnoth-lib (not required for the domain 'wesnoth', required for all
+ *    other textdomains).
+ *    @code
+ *    #define GETTEXT_DOMAIN "wesnoth-lib"
+ *    @endcode
+ *
  * This should be all that is required to have your strings that are marked
- * translateable in the po files and translated ingame. So you at least have
- * to mark the strings translateable, too. ;)
+ * translatable in the po files and translated ingame. So you at least have
+ * to mark the strings translatable, too. ;)
  */
 
 // gettext-related declarations
 
 #include <libintl.h>
 
+#ifdef setlocale
+// Someone in libintl world decided it was a good idea to define a "setlocale" macro.
+#undef setlocale
+#endif
+
 const char* egettext(const char*);
 const char* sgettext(const char*);
 const char* dsgettext(const char * domainname, const char *msgid);
