@@ -1,18 +1,17 @@
-/* $Id: listbox.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: listbox.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
-
 #ifndef GUI_WIDGETS_LISTBOX_HPP_INCLUDED
 #define GUI_WIDGETS_LISTBOX_HPP_INCLUDED
 
@@ -21,15 +20,18 @@
 
 namespace gui2 {
 
+namespace implementation {
+	struct tbuilder_listbox;
+	struct tbuilder_horizontal_listbox;
+}
+
 /** The listbox class. */
 class tlistbox
 		: public tscrollbar_container
 {
-	friend struct tbuilder_listbox;
+	friend struct implementation::tbuilder_listbox;
+	friend struct implementation::tbuilder_horizontal_listbox;
 	friend class tdebug_layout_graph;
-
-	/** @todo Remove this item, part of the markup hack. */
-	friend class twml_message_;
 public:
 	/**
 	 * Constructor.
@@ -50,8 +52,13 @@ public:
 	 * When an item in the list is selected by the user we need to
 	 * update the state. We installed a callback handler which
 	 * calls us.
+	 *
+	 * @param item                The data send to the set_members of the
+	 *                            widgets.
+	 * @param index               The item before which to add the new item,
+	 *                            0 == begin, -1 == end.
 	 */
-	void add_row(const string_map& item);
+	void add_row(const string_map& item, const int index = -1);
 
 	/**
 	 * Adds single row to the grid.
@@ -67,9 +74,25 @@ public:
 	 *                            empty string, it is send to all members.
 	 *                            Having both empty and non-empty id's gives
 	 *                            undefined behaviour.
+	 * @param index               The item before which to add the new item,
+	 *                            0 == begin, -1 == end.
 	 */
-	void add_row(const std::map<std::string /* widget id */,
-			string_map>& data);
+	void add_row(
+			  const std::map<std::string /* widget id */, string_map>& data
+			, const int index = -1);
+
+	/**
+	 * Removes a row in the listbox.
+	 *
+	 * @param row                 The row to remove, when not in
+	 *                            range the function is ignored.
+	 * @param count               The number of rows to remove, 0 means all
+	 *                            rows (starting from row).
+	 */
+	void remove_row(const unsigned row, unsigned count = 1);
+
+	/** Removes all the rows in the listbox, clearing it. */
+	void clear();
 
 	/** Returns the number of items in the listbox. */
 	unsigned get_item_count() const;
@@ -80,10 +103,32 @@ public:
 	 * NOTE this doesn't change the select status of the row.
 	 *
 	 * @param row                 The row to (de)activate.
-	 * @param select              true activate, false deactivate.
+	 * @param active              true activate, false deactivate.
 	 */
 	void set_row_active(const unsigned row, const bool active);
 
+	/**
+	 * Makes a row visible or invisible.
+	 *
+	 * @param row                 The row to show or hide.
+	 * @param shown               true visible, false invisible.
+	 */
+	void set_row_shown(const unsigned row, const bool shown);
+
+	/**
+	 * Makes a row visible or invisible.
+	 *
+	 * Use this version if you want to show hide multiple items since it's
+	 * optimized for that purpose, for one it calls the selection changed
+	 * callback only once instead of serveral times.
+	 *
+	 * @param shown               A vector with the show hide status for every
+	 *                            row. The number of items in the vector must
+	 *                            be equal to the number of items in the
+	 *                            listbox.
+	 */
+	void set_row_shown(const std::vector<bool>& shown);
+
 	/**
 	 * Returns the grid of the wanted row.
 	 *
@@ -98,6 +143,15 @@ public:
 	 */
 	const tgrid* get_row_grid(const unsigned row) const;
 
+	/**
+	 * The possibly-giving-problems nonconst version of get_row_grid
+	 *
+	 * @param row                 The row to get the grid from, the caller has
+	 *                            to make sure the row is a valid row.
+	 * @returns                   The grid of the wanted row.
+	 */
+	tgrid* get_row_grid(const unsigned row);
+
 	/**
 	 * Selectes a row.
 	 *
@@ -121,8 +175,31 @@ public:
 	void set_self_active(const bool /*active*/)  {}
 //		{ state_ = active ? ENABLED : DISABLED; }
 //
+
+	/**
+	 * Request to update the size of the content after changing the content.
+	 *
+	 * When a resize is required the container first can try to handle it
+	 * itself. If it can't honour the request the function will call @ref
+	 * twindow::invalidate_layout().
+	 *
+	 * @note Calling this function on a widget with size == (0, 0) results
+	 * false but doesn't call invalidate_layout, the engine expects to be in
+	 * build up phase with the layout already invalidated.
+	 *
+	 * @returns                      True if the resizing succeeded, false
+	 *                               otherwise.
+	 */
+	bool update_content_size();
+
 	/***** ***** ***** ***** inherited ***** ***** ****** *****/
 
+	/** Inherited from tscrollbar_container. */
+	void place(const tpoint& origin, const tpoint& size);
+
+	/** Inherited from tscrollbar_container. */
+	void layout_children();
+
 	/** Inherited from tscrollbar_container. */
 	void child_populate_dirty_list(twindow& caller,
 			const std::vector<twidget*>& call_stack);
@@ -167,34 +244,6 @@ private:
 	 * For now it's always fixed width depending on the first row.
 	 */
 
-	/**
-	 * Initializes the linked size list.
-	 *
-	 * The routine goes from begin to end through the widgets and if the widget
-	 * has an id it's used to initialize the linked size list of the parent
-	 * window. If the widget is a container all it's children are also
-	 * initialized.
-	 *
-	 * @param window              The parent window.
-	 * @param begin               Begin iterator.
-	 * @param end                 End iterator.
-	 */
-	void init_linked_size_widets(twindow& window,
-			const tgrid::iterator& begin, const tgrid::iterator& end);
-
-	/**
-	 * Adds widgets to the linked size list.
-	 *
-	 * The routine goes from begin to end through the widgets and if the widget
-	 * has an id it's added to the linked size list of the parent window. If
-	 * the widget is a container all it's children are also added.
-	 *
-	 * @param window              The parent window.
-	 * @param begin               Begin iterator.
-	 * @param end                 End iterator.
-	 */
-	void add_linked_size_widgets(twindow& window,
-			const tgrid::iterator& begin, const tgrid::iterator& end);
 
 	/**
 	 * Finishes the building initialization of the widget.
@@ -210,7 +259,9 @@ private:
 	/**
 	 * Contains a pointer to the generator.
 	 *
-	 * The pointer is not owned by this variable.
+	 * The pointer is not owned by this class, it's stored in the content_grid_
+	 * of the tscrollbar_container super class and freed when it's grid is
+	 * freed.
 	 */
 	tgenerator_* generator_;
 
@@ -226,6 +277,33 @@ private:
 	 */
 	void (*callback_value_changed_) (twidget*);
 
+	bool need_layout_;
+
+	/**
+	 * Resizes the content.
+	 *
+	 * The resize either happens due to resizing the content or invalidate the
+	 * layout of the window.
+	 *
+	 * @param width_modification  The wanted modification to the width:
+	 *                            * negative values reduce width.
+	 *                            * zero leave width as is.
+	 *                            * positive values increase width.
+	 * @param height_modification The wanted modification to the height:
+	 *                            * negative values reduce height.
+	 *                            * zero leave height as is.
+	 *                            * positive values increase height.
+	 */
+	void resize_content(
+			  const int width_modification
+			, const int height_modification);
+
+	/** Layouts the children if needed. */
+	void layout_children(const bool force);
+
+	/** Inherited from tscrollbar_container. */
+	virtual void set_content_size(const tpoint& origin, const tpoint& size);
+
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
 };
