@@ -1,15 +1,15 @@
-/* $Id: scroll_label.hpp 37939 2009-08-18 19:47:08Z mordante $ */
+/* $Id: scroll_label.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2008 - 2011 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_SCROLL_LABEL_HPP_INCLUDED
@@ -22,6 +22,10 @@ namespace gui2 {
 class tlabel;
 class tspacer;
 
+namespace implementation {
+	struct tbuilder_scroll_label;
+}
+
 /**
  * Label showing a text.
  *
@@ -31,7 +35,7 @@ class tspacer;
  */
 class tscroll_label : public tscrollbar_container
 {
-	friend struct tbuilder_scroll_label;
+	friend struct implementation::tbuilder_scroll_label;
 public:
 
 	tscroll_label();
@@ -40,7 +44,7 @@ public:
 	void set_label(const t_string& label);
 
 	/** Inherited from tcontrol. */
-	void set_markup_mode(const tmarkup_mode markup_mode);
+	void set_use_markup(bool use_markup);
 
 	/** Inherited from tcontainer_. */
 	void set_self_active(const bool active)
@@ -78,6 +82,9 @@ private:
 	/** Inherited from tcontrol. */
 	const std::string& get_control_type() const;
 
+	/***** ***** ***** signal handlers ***** ****** *****/
+
+	void signal_handler_left_button_down(const event::tevent event);
 };
 
 } // namespace gui2
