@@ -1,20 +1,21 @@
-/* $Id: widget.hpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: widget.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   copyright (C) 2007 - 2009 by mark de wever <koraq@xs4all.nl>
-   part of the battle for wesnoth project http://www.wesnoth.org/
+   Copyright (C) 2007 - 2011 by Mark de Wever <koraq@xs4all.nl>
+   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
-   this program is free software; you can redistribute it and/or modify
-   it under the terms of the gnu general public license version 2
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2
    or at your option any later version.
-   this program is distributed in the hope that it will be useful,
-   but without any warranty.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY.
 
-   see the copying file for more details.
+   See the COPYING file for more details.
 */
 
 #ifndef GUI_WIDGETS_WIDGET_HPP_INCLUDED
 #define GUI_WIDGETS_WIDGET_HPP_INCLUDED
 
+#include "gui/auxiliary/event/dispatcher.hpp"
 #include "gui/widgets/event_executor.hpp"
 #include "gui/widgets/helper.hpp"
 #include "sdl_utils.hpp"
@@ -40,30 +41,15 @@ class twindow;
 class twidget
 	: private boost::noncopyable
 	, public virtual tevent_executor
+	, public virtual event::tdispatcher
 {
 	friend class tdebug_layout_graph;
 	friend class twindow; // needed for modifying the layout_size.
 
 public:
-	twidget()
-		: id_("")
-		, definition_("default")
-		, parent_(0)
-		, x_(-1)
-		, y_(-1)
-		, w_(0)
-		, h_(0)
-		, dirty_(true)
-		, visible_(VISIBLE)
-		, drawing_action_(DRAWN)
-		, clip_rect_()
-		, layout_size_(tpoint(0,0))
-#ifdef DEBUG_WINDOW_LAYOUT_GRAPHS
-		, last_best_size_(tpoint(0,0))
-#endif
-		{}
+	twidget();
 
-	virtual ~twidget() {}
+	virtual ~twidget();
 
 	/**
 	 * Loads the configuration of the widget.
@@ -82,8 +68,7 @@ public:
 		/**
 		 * The user set the widget visible, that means:
 		 * * widget is visible.
-		 * * find_widget always 'sees' the widget (the active flag is tested
-		 *   later).
+		 * * find_at always 'sees' the widget (the active flag is tested later).
 		 * * widget (if active) handles events (and sends events to children)
 		 * * widget is drawn (and sends populate_dirty_list to children)
 		 */
@@ -91,7 +76,7 @@ public:
 		/**
 		 * The user set the widget hidden, that means:
 		 * * item is invisible but keeps its size.
-		 * * find_widget 'sees' the widget if active is false.
+		 * * find_at 'sees' the widget if active is false.
 		 * * item doesn't handle events (and doesn't send events to children).
 		 * * item doesn't populate_dirty_list (nor does it send the request
 		 *   to its children).
@@ -100,7 +85,7 @@ public:
 		/**
 		 * The user set the widget invisible, that means:
 		 * * item is invisible and gridcell has size 0,0.
-		 * * find_widget never 'sees' the widget.
+		 * * find_at never 'sees' the widget.
 		 * * item doesn't handle events (and doesn't send events to children).
 		 * * item doesn't populate_dirty_list (nor does it send the request
 		 *   to its children).
@@ -143,19 +128,72 @@ public:
 	 * that after the layout phase get_best_size() still returns this value.
 	 */
 
-	/** Initializes the layout phase. */
-	virtual void layout_init() { layout_size_ = tpoint(0,0); }
-
 	/**
 	 * Initializes the layout phase.
 	 *
-	 * @param full_initialization Reset the widget to its initial state. This
-	 *                            flag is used to change the status in the
-	 *                            first layout run, but keep the status when a
-	 *                            relayout phase happens.
+	 * Clears the initial best size for the widgets.
+	 *
+	 * @see @ref layout_algorihm for more information.
+	 *
+	 * @param full_initialization For widgets with scrollbars it hides them
+	 *                            unless the mode is
+	 *                            tscrollbar_mode::always_visible. For other
+	 *                            widgets this flag is a NOP.
 	 */
-	virtual void layout_init2(const bool /*full_initialization*/)
-		{ layout_size_ = tpoint(0,0); }
+	virtual void layout_init(const bool full_initialization);
+
+	/**
+	 * Tries to reduce the width of a widget.
+	 *
+	 * This function tries to do it 'friendly' and only use scrollbars or
+	 * wrapping of the widget.
+	 *
+	 * @see @ref layout_algorihm for more information.
+	 *
+	 * @param maximum_width       The wanted maximum width.
+	 */
+	virtual void request_reduce_width(const unsigned maximum_width) = 0;
+
+	/**
+	 * Tries to reduce the width of a widget.
+	 *
+	 * This function does it more agressive and should only be used when
+	 * using scrollbars and wrapping failed.
+	 *
+	 * @todo Make pure virtual.
+	 *
+	 * @see @ref layout_algorihm for more information.
+	 *
+	 * @param maximum_width       The wanted maximum width.
+	 */
+	virtual void demand_reduce_width(const unsigned /*maximum_width*/) {}
+
+	/**
+	 * Tries to reduce the height of a widget.
+	 *
+	 * This function tries to do it 'friendly' and only use scrollbars.
+	 *
+	 * @todo Make pure virtual.
+	 *
+	 * @see @ref layout_algorihm for more information.
+	 *
+	 * @param maximum_height      The wanted maximum height.
+	 */
+	virtual void request_reduce_height(const unsigned /*maximum_height*/) {}
+
+	/**
+	 * Tries to reduce the height of a widget.
+	 *
+	 * This function does it more agressive and should only be used when
+	 * using scrollbars failed.
+	 *
+	 * @todo Make pure virtual.
+	 *
+	 * @see @ref layout_algorihm for more information.
+	 *
+	 * @param maximum_height      The wanted maximum height.
+	 */
+	virtual void demand_reduce_height(const unsigned /*maximum_height*/) {}
 
 	/**
 	 * Gets the best size for the widget.
@@ -195,116 +233,26 @@ public:
 	virtual bool can_wrap() const { return false; }
 
 	/**
-	 * Wraps the contents of the widget.
+	 * Places the widget.
 	 *
-	 * @todo implement this function properly.
-	 *
-	 * @param maximum_width       The wanted maximum width of the widget.
-	 *
-	 * @pre                       can_wrap() == true.
-	 */
-	virtual void layout_wrap(const unsigned /*maximum_width*/)
-		{ assert(can_wrap()); }
-
-	/**
-	 * Does the widget have a horizontal scrollbar.
-	 *
-	 * See has_vertical_scrollbar for more info.
-	 * @returns                   Whether or not the widget has a horizontal
-	 *                            scrollbar.
-	 */
-	virtual bool has_horizontal_scrollbar() const { return false; }
-
-	/**
-	 * Tries to use a horizontal scrollbar with the widget.
-	 *
-	 * @param maximum_width       The wanted maximum width of the widget.
-	 *
-	 * @pre                       has_horizontal_scrollbar() == true.
-	 */
-	virtual void layout_use_horizontal_scrollbar(const unsigned /*maximum_width*/)
-		{ assert(has_horizontal_scrollbar()); }
-
-	/** Can the widget reduce its width. */
-	virtual bool can_shrink_width() const { return false; }
-
-	/**
-	 * Shrinks the contents of the widget so it can fit the wanted width
-	 *
-	 * @todo implement this function properly at the moment it kind of works
-	 * for a label but should also work for a button and maybe some other
-	 * classes.
-	 *
-	 * @param maximum_width       The wanted maximum width of the widget.
-	 *
-	 * @pre                       can_shrink_width() == true.
-	 */
-	virtual void layout_shrink_width(const unsigned /*maximum_width*/)
-		{ assert(can_shrink_width()); }
-
-	/** Flags to decide how to try to fit the widget in the layout. */
-	enum tfit_flags {
-		  WRAP      = 1 /**< Try to wrap the widget. */
-		, SCROLLBAR = 2 /**< Try to use scrollbars. */
-		, SHRINK    = 4 /**< Try to shrink the widget. */
-		, FORCE     = 7 /**< Try all of the above. */
-	};
-
-	/**
-	 * Fits the layout in the wanted width.
-	 *
-	 * @param maximum_width       The maximum width of the window.
-	 * @param flags               The flags for the fitting.
-	 */
-	virtual void layout_fit_width(const unsigned /*maximum_width*/,
-			const tfit_flags /*flags*/)
-	{
-		//FIXME should be pure abstract but gives linker errors.
-	}
-
-	/**
-	 * Does the widget have a vertical scrollbar.
-	 *
-	 * We want to use the best size for a widget, when the best size for all
-	 * widgets doesn't fit inside the window some widgets need to be reduced in
-	 * size. Widgets that have a scrollbar in the wanted direction are resized
-	 * first. The next option is to look at the minimum size.
-	 *
-	 * @returns                   Whether or not the widget has a vertical
-	 *                            scrollbar.
-	 */
-	virtual bool has_vertical_scrollbar() const { return false; }
-
-	/**
-	 * Tries to use a vertical scrollbar with the widget.
-	 *
-	 * @todo implement this function properly.
-	 *
-	 * @param maximum_height      The wanted maximum height of the widget.
-	 *
-	 * @pre                       has_vertical_scrollbar() == true.
-	 */
-	virtual void layout_use_vertical_scrollbar(const unsigned /*maximum_height*/)
-		{ assert(has_vertical_scrollbar()); }
-
-	/**
-	 * Tries to shrink a widget so it fits in the wanted height.
-	 *
-	 * @todo implement this function properly.
-	 *
-	 * @param maximum_height      The wanted maximum height of the widget.
-	 *
-	 */
-	virtual void layout_shrink_height(const unsigned /*maximum_height*/) { }
-
-	/**
-	 * Sets the size of the widget.
+	 * This function is normally called by a layout function to do the
+	 * placement of a widget.
 	 *
 	 * @param origin              The position of top left of the widget.
 	 * @param size                The size of the widget.
 	 */
-	virtual void set_size(const tpoint& origin, const tpoint& size);
+	virtual void place(const tpoint& origin, const tpoint& size);
 
+	/**
+	 * Sets the size of the widget.
+	 *
+	 * This version is meant to resize a widget, since the origin isn't
+	 * modified. This can be used if a widget needs to change its size and the
+	 * layout will be fixed later.
+	 *
+	 * @param size                The size of the widget.
+	 */
+	virtual void set_size(const tpoint& size);
 
 	/***** ***** ***** ***** query ***** ***** ***** *****/
 
@@ -321,11 +269,11 @@ public:
 	 * @retval 0                  No widget at the wanted coordinate found (or
 	 *                            not active if must_be_active was set).
 	 */
-	virtual twidget* find_widget(const tpoint& coordinate,
+	virtual twidget* find_at(const tpoint& coordinate,
 			const bool must_be_active);
 
-	/** The const version of find_widget. */
-	virtual const twidget* find_widget(const tpoint& coordinate,
+	/** The const version of find_at. */
+	virtual const twidget* find_at(const tpoint& coordinate,
 			const bool must_be_active) const;
 
 	/**
@@ -340,85 +288,15 @@ public:
 	 * @retval 0                  No widget with the id found (or not active if
 	 *                            must_be_active was set).
 	 */
-	virtual twidget* find_widget(const std::string& id,
+	virtual twidget* find(const std::string& id,
 			const bool /*must_be_active*/)
 		{ return id_ == id ? this : 0; }
 
-	/** The const version of find_widget. */
-	virtual const twidget* find_widget(const std::string& id,
+	/** The const version of find. */
+	virtual const twidget* find(const std::string& id,
 			const bool /*must_be_active*/) const
 		{ return id_ == id ? this : 0; }
 
-	/**
-	 * Gets a widget with the wanted id.
-	 *
-	 * This template function doesn't return a pointer to a generic widget but
-	 * returns the wanted type and tests for its existence if required.
-	 *
-	 * @param id                  The id of the widget to find.
-	 * @param must_be_active      The widget should be active, not all widgets
-	 *                            have an active flag, those who don't ignore
-	 *                            flag.
-	 * @param must_exist          The widget should be exist, the function will
-	 *                            fail if the widget doesn't exist or is
-	 *                            inactive and must be active. Upon failure a
-	 *                            wml_error is thrown.
-	 *
-	 * @returns                   The widget with the id.
-	 */
-	template<class T>
-	T* find_widget(const std::string& id,
-			const bool must_be_active, const bool must_exist)
-	{
-		T* widget =
-			dynamic_cast<T*>(find_widget(id, must_be_active));
-		VALIDATE(!must_exist || widget, missing_widget(id));
-
-		return widget;
-	}
-
-	/** The const version of find_widget. */
-	template<class T>
-	const T* find_widget(const std::string& id,
-			const bool must_be_active, const bool must_exist) const
-	{
-		const T* widget =
-			dynamic_cast<const T*>(find_widget(id, must_be_active));
-		VALIDATE(!must_exist || widget, missing_widget(id));
-
-		return widget;
-	}
-
-	/**
-	 * Gets a widget with the wanted id.
-	 *
-	 * This template function doesn't return a pointer to a generic widget but
-	 * returns the wanted type and tests for its existence.
-	 *
-	 * @param id                  The id of the widget to find.
-	 * @param must_be_active      The widget should be active, not all widgets
-	 *                            have an active flag, those who don't ignore
-	 *                            flag.
-	 *
-	 * @returns                   The widget with the id.
-	 */
-	template <class T>
-	T& get_widget(const std::string& id, const bool must_be_active)
-	{
-		T* result = dynamic_cast<T*>(find_widget(id, must_be_active));
-		VALIDATE(result, missing_widget(id));
-		return *result;
-	}
-
-	/** The const version of get_widget. */
-	template <class T>
-	const T& get_widget(const std::string& id, const bool must_be_active) const
-	{
-		const T* result = dynamic_cast<const T*>(find_widget(id, must_be_active));
-		VALIDATE(result, missing_widget(id));
-		return *result;
-	}
-
 	/**
 	 * Does the widget contain the widget.
 	 *
@@ -457,13 +335,18 @@ public:
 	 */
 	tdialog* dialog();
 
+	/***** ***** ***** Misc. ***** ****** *****/
+
+	/** Does the widget disable easy close? */
+	virtual bool disable_click_dismiss() const = 0;
+
 	/***** ***** ***** setters / getters for members ***** ****** *****/
 
 	twidget* parent() { return parent_; }
 	void set_parent(twidget* parent) { parent_ = parent; }
 
 	const std::string& id() const { return id_; }
-	void set_id(const std::string& id) { id_ = id; }
+	void set_id(const std::string& id);
 
 	const std::string& definition() const { return definition_; }
 
@@ -523,13 +406,7 @@ public:
 	void set_visible(const tvisible visible);
 	tvisible get_visible() const { return visible_; }
 
-	/** Returns true if the widget is visible. */
-	bool is_visible() const { return visible_ == VISIBLE; }
-
-	/** Returns true if the widget is invisible. */
-	bool is_invisible() const { return visible_ == INVISIBLE; }
-
-	tdrawing_action get_drawing_action() const { return drawing_action_; }
+	tdrawing_action get_drawing_action() const;
 
 	/**
 	 * Sets the visible area for a widget.
@@ -553,6 +430,18 @@ public:
 	/** Returns the dirty state for a widget, final function. */
 	bool get_dirty() const { return dirty_; }
 
+#ifndef LOW_MEM
+	void set_debug_border_mode(const unsigned debug_border_mode)
+	{
+		debug_border_mode_ = debug_border_mode;
+	}
+
+	void set_debug_border_colour(const unsigned debug_border_colour)
+	{
+		debug_border_colour_ = debug_border_colour;
+	}
+#endif
+
 	/**
 	 * Draws the background of a widget.
 	 *
@@ -588,6 +477,16 @@ public:
 	 */
 	void draw_foreground(surface& frame_buffer);
 
+	/**
+	 * Allows a widget to update its children.
+	 *
+	 * Before the window is populating the dirty list the widgets can update
+	 * their content, which allows delayed initialization. This delayed
+	 * initialization is only allowed if the widget resizes itself, not when
+	 * being placed.
+	 */
+	virtual void layout_children() {}
+
 	/**
 	 * Adds a widget to the dirty list if it is dirty.
 	 *
@@ -604,9 +503,14 @@ public:
 	void populate_dirty_list(twindow& caller,
 			std::vector<twidget*>& call_stack);
 
+private:
+
 	/**
 	 * Tries to add all children of a container to the dirty list.
 	 *
+	 * @note The function is private since everybody should call
+	 * populate_dirty_list instead.
+	 *
 	 * @param caller              The parent window, if dirty it should
 	 *                            register itself to this window.
 	 * @param call_stack          The callstack of widgets traversed to reach
@@ -620,7 +524,15 @@ protected:
 	void set_layout_size(const tpoint& size) { layout_size_ = size; }
 	const tpoint& layout_size() const { return layout_size_; }
 
+public:
+
+	void set_linked_group(const std::string& linked_group)
+	{
+		linked_group_ = linked_group;
+	}
+
 private:
+
 	/**
 	 * The id is the unique name of the widget in a certain context. This is
 	 * needed for certain widgets so the engine knows which widget is which.
@@ -683,6 +595,40 @@ private:
 	 */
 	tpoint layout_size_;
 
+	/**
+	 * The linked group the widget belongs to.
+	 *
+	 * @todo For now the linked group is initialized when the layout of the
+	 * widget is initialized. The best time to set it would be upon adding the
+	 * widget in the window. Need to look whether it's possible in a clean way.
+	 * Maybe a signal just prior to showing a window where the widget can do
+	 * some of it's on things, would also be nice for widgets that need a
+	 * finalizer function.
+	 */
+	std::string linked_group_;
+
+#ifndef LOW_MEM
+	/**
+	 * Mode for drawing the debug border.
+	 *
+	 * The debug border is a helper border to determine where a widget is
+	 * placed. It's only intended for debugging purposes.
+	 *
+	 * Possible values:
+	 * - 0 no border
+	 * - 1 single pixel border
+	 * - 2 floodfilled rectangle
+	 */
+	unsigned debug_border_mode_;
+
+	/** The color for the debug border. */
+	unsigned debug_border_colour_;
+
+	void draw_debug_border(surface& frame_buffer);
+#else
+	void draw_debug_border(surface&) {}
+#endif
+
 #ifdef DEBUG_WINDOW_LAYOUT_GRAPHS
 	/**
 	 * Debug helper to store last value of get_best_size().
@@ -701,13 +647,34 @@ private:
 
 	/** See draw_foreground. */
 	virtual void impl_draw_foreground(surface& /*frame_buffer*/) {}
+
+	/** (Will be) inherited from event::tdispatcher. */
+	virtual bool is_at(const tpoint& coordinate) const
+	{
+		return is_at(coordinate, true);
+	}
+
+	/**
+	 * Is the coordinate inside our area.
+	 *
+	 * Helper for find_at so also looks at our visibility.
+	 *
+	 * @param coordinate          The coordinate which should be inside the
+	 *                            widget.
+	 * @param must_be_active      The widget should be active, not all widgets
+	 *                            have an active flag, those who don't ignore
+	 *                            flag.
+	 *
+	 * @returns                   Status.
+	 */
+	bool is_at(const tpoint& coordinate, const bool must_be_active) const;
 };
 
 /**
  * Returns the first parent of a widget with a certain type.
  *
  * @param widget                  The widget to get the parent from,
- * @param T                       The class of the widget to return.
+ * @tparam T                      The class of the widget to return.
  *
  * @returns                       The parent widget.
  */
@@ -724,6 +691,62 @@ template<class T> T* get_parent(twidget* widget)
 	return result;
 }
 
+/**
+ * Gets a widget with the wanted id.
+ *
+ * This template function doesn't return a pointer to a generic widget but
+ * returns the wanted type and tests for its existence if required.
+ *
+ * @param widget              The widget test or find a child with the wanted
+ *                            id.
+ * @param id                  The id of the widget to find.
+ * @param must_be_active      The widget should be active, not all widgets
+ *                            have an active flag, those who don't ignore
+ *                            flag.
+ * @param must_exist          The widget should be exist, the function will
+ *                            fail if the widget doesn't exist or is
+ *                            inactive and must be active. Upon failure a
+ *                            wml_error is thrown.
+ *
+ * @returns                   The widget with the id.
+ */
+template<class T>
+T* find_widget(typename tconst_duplicator<T, twidget>::type* widget
+		, const std::string& id
+		, const bool must_be_active
+		, const bool must_exist)
+{
+	T* result =
+		dynamic_cast<T*>(widget->find(id, must_be_active));
+	VALIDATE(!must_exist || result, missing_widget(id));
+
+	return result;
+}
+
+/**
+ * Gets a widget with the wanted id.
+ *
+ * This template function doesn't return a reference to a generic widget but
+ * returns a reference to the wanted type
+ *
+ * @param widget              The widget test or find a child with the wanted
+ *                            id.
+ * @param id                  The id of the widget to find.
+ * @param must_be_active      The widget should be active, not all widgets
+ *                            have an active flag, those who don't ignore
+ *                            flag.
+ *
+ * @returns                   The widget with the id.
+ */
+template<class T>
+T& find_widget(typename tconst_duplicator<T, twidget>::type* widget
+		, const std::string& id
+		, const bool must_be_active)
+{
+	return *find_widget<T>(widget, id, must_be_active, true);
+}
+
 } // namespace gui2
 
 #endif
+
