@@ -1,6 +1,6 @@
-/* $Id: image.cpp 35477 2009-05-08 00:06:52Z alink $ */
+/* $Id: image.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 
 #include "global.hpp"
 
+#include "color_range.hpp"
 #include "config.hpp"
 #include "filesystem.hpp"
 #include "foreach.hpp"
@@ -28,19 +29,22 @@
 #include "image.hpp"
 #include "image_function.hpp"
 #include "log.hpp"
+#include "gettext.hpp"
+#include "serialization/string_utils.hpp"
 
 #include "SDL_image.h"
 
+#include <boost/functional/hash.hpp>
 
-#define ERR_DP LOG_STREAM(err, display)
-#define INFO_DP LOG_STREAM(info, display)
-#define DBG_CACHE LOG_STREAM(debug, cache)
+#include <set>
+
+static lg::log_domain log_display("display");
+#define ERR_DP LOG_STREAM(err, log_display)
+#define LOG_DP LOG_STREAM(info, log_display)
 
 namespace {
 
-typedef std::map<image::locator::value, int> locator_finder_t;
-typedef std::pair<image::locator::value, int> locator_finder_pair;
-locator_finder_t locator_finder;
+image::locator::locator_finder_t locator_finder;
 
 /** Definition of all image maps */
 image::image_cache images_,hexed_images_,scaled_to_hex_images_,scaled_to_zoom_,unmasked_images_;
@@ -107,13 +111,12 @@ int locator::last_index_ = 0;
 
 void locator::init_index()
 {
-	locator_finder_t::iterator i = locator_finder.find(val_);
+	std::map<value, int>& finder = locator_finder[hash_value(val_)];
+	std::map<value, int>::iterator i = finder.find(val_);
 
-	if(i == locator_finder.end()) {
+	if(i == finder.end()) {
 		index_ = last_index_++;
-		locator_finder.insert(locator_finder_pair(val_, index_));
-
-
+		finder.insert(std::make_pair(val_, index_));
 	} else {
 		index_ = i->second;
 	}
@@ -168,13 +171,6 @@ locator::locator(const std::string &filename) :
 	init_index();
 }
 
-locator::locator(const char *filename, const std::string& modifications) :
-	index_(-1),
-	val_(filename, modifications)
-{
-	init_index();
-}
-
 locator::locator(const std::string &filename, const std::string& modifications) :
 	index_(-1),
 	val_(filename, modifications)
@@ -182,14 +178,6 @@ locator::locator(const std::string &filename, const std::string& modifications)
 	init_index();
 }
 
-locator::locator(const std::string &filename,
-		const map_location &loc, const std::string& modifications) :
-	index_(-1),
-	val_(filename, loc, modifications)
-{
-	init_index();
-}
-
 locator::locator(const std::string &filename, const map_location &loc,
 		int center_x, int center_y, const std::string& modifications) :
 	index_(-1),
@@ -226,14 +214,6 @@ locator::value::value(const char *filename) :
 {
 }
 
-
-locator::value::value(const char *filename, const std::string& modifications) :
-  type_(SUB_FILE), filename_(filename), loc_(), modifications_(modifications),
-  center_x_(0), center_y_(0)
-
-{
-}
-
 locator::value::value(const std::string& filename) :
   type_(FILE), filename_(filename),  loc_(), modifications_(),
   center_x_(0), center_y_(0)
@@ -248,13 +228,6 @@ locator::value::value(const std::string& filename, const std::string& modificati
 {
 }
 
-locator::value::value(const std::string& filename, const map_location& loc, const std::string& modifications) :
-  type_(SUB_FILE), filename_(filename), loc_(loc), modifications_(modifications),
-  center_x_(0), center_y_(0)
-
-{
-}
-
 locator::value::value(const std::string& filename, const map_location& loc, int center_x, int center_y, const std::string& modifications) :
   type_(SUB_FILE), filename_(filename), loc_(loc), modifications_(modifications), center_x_(center_x), center_y_(center_y)
 {
@@ -296,6 +269,108 @@ bool locator::value::operator<(const value& a) const
 	}
 }
 
+size_t hash_value(const locator::value& val) {
+	using boost::hash_value;
+	using boost::hash_combine;
+
+	size_t hash = hash_value(val.type_);
+	if (val.type_ == locator::FILE || val.type_ == locator::SUB_FILE) {
+		hash_combine(hash, val.filename_);
+	}
+	if (val.type_ == locator::SUB_FILE) {
+		hash_combine(hash, val.loc_.x);
+		hash_combine(hash, val.loc_.y);
+		hash_combine(hash, val.center_x_);
+		hash_combine(hash, val.center_y_);
+		hash_combine(hash, val.modifications_);
+	}
+
+	return hash;
+}
+
+// Check if localized file is uptodate according to l10n track index.
+// Make sure only that the image is not explicitly recorded as fuzzy,
+// in order to be able to use non-tracked images (e.g. from UMC).
+static std::set<std::string> fuzzy_localized_files;
+static bool localized_file_uptodate (const std::string& loc_file)
+{
+	if (fuzzy_localized_files.size() == 0) {
+		// First call, parse track index to collect fuzzy files by path.
+		std::string fsep = "\xC2\xA6"; // UTF-8 for "broken bar"
+		std::string trackpath = get_binary_file_location("", "l10n-track");
+		std::string contents = read_file(trackpath);
+		std::vector<std::string> lines = utils::split(contents, '\n');
+		foreach (const std::string &line, lines) {
+			size_t p1 = line.find(fsep);
+			if (p1 == std::string::npos)
+				continue;
+			std::string state = line.substr(0, p1);
+			utils::strip(state);
+			if (state == "fuzzy") {
+				size_t p2 = line.find(fsep, p1 + fsep.length());
+				if (p2 == std::string::npos)
+					continue;
+				std::string relpath = line.substr(p1 + fsep.length(), p2 - p1 - fsep.length());
+				fuzzy_localized_files.insert(game_config::path + '/' + relpath);
+			}
+		}
+		fuzzy_localized_files.insert(""); // make sure not empty any more
+	}
+	return fuzzy_localized_files.count(loc_file) == 0;
+}
+
+// Return path to localized counterpart of the given file, if any, or empty string.
+// Localized counterpart may also be requested to have a suffix to base name.
+static std::string get_localized_path (const std::string& file, const std::string& suff = "")
+{
+	std::string dir = directory_name(file);
+	std::string base = file_name(file);
+	const size_t pos_ext = base.rfind(".");
+	std::string loc_base;
+	if (pos_ext != std::string::npos) {
+		loc_base = base.substr(0, pos_ext) + suff + base.substr(pos_ext);
+	} else {
+		loc_base = base + suff;
+	}
+	// TRANSLATORS: This is the language code which will be used
+	// to store and fetch localized non-textual resources, such as images,
+	// when they exist. Normally it is just the code of the PO file itself,
+	// e.g. "de" of de.po for German. But it can also be a comma-separated
+	// list of language codes by priority, when the localized resource
+	// found for first of those languages will be used. This is useful when
+	// two languages share sufficient commonality, that they can use each
+	// other's resources rather than duplicating them. For example,
+	// Swedish (sv) and Danish (da) are such, so Swedish translator could
+	// translate this message as "sv,da", while Danish as "da,sv".
+	std::vector<std::string> langs = utils::split(_("language code for localized resources^en_US"));
+	// In case even the original image is split into base and overlay,
+	// add en_US with lowest priority, since the message above will
+	// not have it when translated.
+	langs.push_back("en_US");
+	foreach (const std::string &lang, langs) {
+		std::string loc_file = dir + "l10n" + "/" + lang + "/" + loc_base;
+		if (file_exists(loc_file) && localized_file_uptodate(loc_file)) {
+			return loc_file;
+		}
+	}
+	return "";
+}
+
+// Load overlay image and compose it with the original surface.
+static void add_localized_overlay (const std::string& ovr_file, const surface &orig_surf)
+{
+	surface ovr_surf = IMG_Load(ovr_file.c_str());
+	if (ovr_surf.null()) {
+		return;
+	}
+	SDL_Rect area;
+	area.x = 0;
+	area.y = 0;
+	area.w = ovr_surf->w;
+	area.h = ovr_surf->h;
+	SDL_BlitSurface(ovr_surf, 0, orig_surf, &area);
+}
+
 surface locator::load_image_file() const
 {
 	surface res;
@@ -305,7 +380,19 @@ surface locator::load_image_file() const
 
 	{
 		if (!location.empty()) {
+			// Check if there is a localized image.
+			const std::string loc_location = get_localized_path(location);
+			if (!loc_location.empty()) {
+				location = loc_location;
+			}
 			res = IMG_Load(location.c_str());
+			// If there was no standalone localized image, check if there is an overlay.
+			if (!res.null() && loc_location.empty()) {
+				const std::string ovr_location = get_localized_path(location, "--overlay");
+				if (!ovr_location.empty()) {
+					add_localized_overlay(ovr_location, res);
+				}
+			}
 		}
 	}
 
@@ -472,35 +559,39 @@ surface locator::load_image_sub_file() const
 						}
 					}
 					else {
-						//
-						// try to recolor source palette to target palette (palette switch)
-						//
-						const std::vector<std::string> remap_params = utils::split(field,'=');
-						if(remap_params.size() > 1) {
-							std::map<Uint32, Uint32> tmp_map;
-							try {
-								std::vector<Uint32> const& old_palette =
-									game_config::tc_info(remap_params[0]);
-								std::vector<Uint32> const& new_palette =
-									game_config::tc_info(remap_params[1]);
+						// Deprecated 1.6 palette switch syntax
+						if(field.find('=') != std::string::npos) {
+							lg::wml_error << "the ~RC() image function cannot be used for palette switch (A=B) in 1.7.x; use ~PAL(A>B) instead\n";
+						}
+					}
+				}
+				// Palette switch (PAL)
+				else if("PAL" == function) {
+					const std::vector<std::string> remap_params = utils::split(field,'>');
+					if(remap_params.size() > 1) {
+						std::map<Uint32, Uint32> tmp_map;
+						try {
+							std::vector<Uint32> const& old_palette =
+								game_config::tc_info(remap_params[0]);
+							std::vector<Uint32> const& new_palette =
+								game_config::tc_info(remap_params[1]);
 
-								for(size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
-									tmp_map[old_palette[i]] = new_palette[i];
-								}
-							}
-							catch(config::error& e) {
-								ERR_DP
-									<< "caught config::error while processing palette switch RC: "
-									<< e.message
-									<< '\n';
-								ERR_DP
-									<< "bailing out from RC\n";
-								tmp_map.clear();
+							for(size_t i = 0; i < old_palette.size() && i < new_palette.size(); ++i) {
+								tmp_map[old_palette[i]] = new_palette[i];
 							}
+						}
+						catch(config::error& e) {
+							ERR_DP
+								<< "caught config::error while processing PAL function: "
+								<< e.message
+								<< '\n';
+							ERR_DP
+								<< "bailing out from PAL\n";
+							tmp_map.clear();
+						}
 
-							foreach(const rc_entry_type& rc_entry, tmp_map) {
-								rc.map()[rc_entry.first] = rc_entry.second;
-							}
+						foreach(const rc_entry_type& rc_entry, tmp_map) {
+							rc.map()[rc_entry.first] = rc_entry.second;
 						}
 					}
 				}
@@ -535,7 +626,7 @@ surface locator::load_image_sub_file() const
 						functor_queue.push_back(new cs_function(r,g,b));
 					}
 					else {
-						INFO_DP << "no arguments passed to the ~CS() function\n";
+						LOG_DP << "no arguments passed to the ~CS() function\n";
 					}
 				}
 				// Crop/slice (CROP)
@@ -618,6 +709,19 @@ surface locator::load_image_sub_file() const
 					const int b = lexical_cast_default<int>(field);
 					functor_queue.push_back(new cs_function(0,0,b));
 				}
+				else if("NOP" == function) {
+				}
+				// Fake image function used by GUI2 portraits until
+				// Mordante gets rid of it. *tsk* *tsk*
+				else if("RIGHT" == function) {
+				}
+				// Add a bright overlay.
+				else if (function == "BRIGHTEN") {
+					functor_queue.push_back(new brighten_function());
+				}
+				else {
+					ERR_DP << "unknown image function in path: " << function << '\n';
+				}
 			}
 		}
 
@@ -630,10 +734,6 @@ surface locator::load_image_sub_file() const
 		}
 
 		foreach(function_base* f, functor_queue) {
-			if(f == NULL) {
-				ERR_DP << "somebody set up us the bomb\n";
-				continue;
-			}
 			surf = (*f)(surf);
 			delete f;
 		}
@@ -695,7 +795,7 @@ void set_pixel_format(SDL_PixelFormat* format)
 		f.Rshift != l.Rshift || f.Gshift != l.Gshift || f.Bshift != l.Bshift ||
 		f.colorkey != l.colorkey || f.alpha != l.alpha)
 	{
-		INFO_DP << "detected a new display format\n";
+		LOG_DP << "detected a new display format\n";
 		flush_cache();
 	}
 	last_pixel_format = *format;
@@ -714,11 +814,6 @@ void set_colour_adjustment(int r, int g, int b)
 	}
 }
 
-boost::tuple<int, int, int> get_colour_adjustment()
-{
-	return boost::make_tuple(red_adjust, green_adjust, blue_adjust);
-}
-
 colour_adjustment_resetter::colour_adjustment_resetter()
 : r_(red_adjust), g_(green_adjust), b_(blue_adjust)
 {
@@ -758,7 +853,7 @@ void set_zoom(int amount)
 	}
 }
 
-static surface get_hexed(const locator i_locator)
+static surface get_hexed(const locator& i_locator)
 {
 	surface image(get_image(i_locator, UNSCALED));
 	// Re-cut scaled tiles according to a mask.
@@ -767,7 +862,7 @@ static surface get_hexed(const locator i_locator)
 	return mask_surface(image, hex);
 }
 
-static surface get_unmasked(const locator i_locator)
+static surface get_unmasked(const locator& i_locator)
 {
 	// If no scaling needed at this zoom level,
 	// we just use the hexed image.
@@ -778,7 +873,7 @@ static surface get_unmasked(const locator i_locator)
 		return image;
 }
 
-static surface get_scaled_to_hex(const locator i_locator)
+static surface get_scaled_to_hex(const locator& i_locator)
 {
 	surface res(get_image(i_locator, UNMASKED));
 
@@ -792,7 +887,7 @@ static surface get_scaled_to_hex(const locator i_locator)
 	return res;
 }
 
-static surface get_scaled_to_zoom(const locator i_locator)
+static surface get_scaled_to_zoom(const locator& i_locator)
 {
 	assert(zoom != tile_size);
 	assert(tile_size != 0);
@@ -806,13 +901,13 @@ static surface get_scaled_to_zoom(const locator i_locator)
 	}
 }
 
-static surface get_brightened(const locator i_locator)
+static surface get_brightened(const locator& i_locator)
 {
 	surface image(get_image(i_locator, SCALED_TO_HEX));
 	return surface(brighten_image(image, ftofxp(1.5)));
 }
 
-static surface get_semi_brightened(const locator i_locator)
+static surface get_semi_brightened(const locator& i_locator)
 {
 	surface image(get_image(i_locator, SCALED_TO_HEX));
 	return surface(brighten_image(image, ftofxp(1.25)));
@@ -932,6 +1027,7 @@ bool is_in_hex(const locator& i_locator)
 	}
 }
 
+
 surface reverse_image(const surface& surf)
 {
 	if(surf == NULL) {
