@@ -1,6 +1,6 @@
-/* $Id: map.cpp 36510 2009-07-02 10:16:13Z soliton $ */
+/* $Id: map.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -27,13 +27,14 @@
 #include "gettext.hpp"
 #include "log.hpp"
 #include "map_exception.hpp"
-#include "pathfind.hpp"
 #include "serialization/parser.hpp"
+#include "util.hpp"
 #include "wml_exception.hpp"
 
-#define ERR_CF LOG_STREAM(err, config)
-#define LOG_G LOG_STREAM(info, general)
-#define DBG_G LOG_STREAM(debug, general)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+#define LOG_G LOG_STREAM(info, lg::general)
+#define DBG_G LOG_STREAM(debug, lg::general)
 
 const std::string gamemap::default_map_header = "usage=map\nborder_size=1\n\n";
 const gamemap::tborder gamemap::default_border = gamemap::SINGLE_TILE_BORDER;
@@ -83,7 +84,7 @@ const t_translation::t_list& gamemap::underlying_union_terrain(t_translation::t_
 std::string gamemap::get_terrain_string(const t_translation::t_terrain& terrain) const
 {
 	std::stringstream ss;
-	const std::string& name = get_terrain_info(terrain).name();
+	const std::string& name = get_terrain_info(terrain).description();
 	const t_translation::t_list& underlying = underlying_union_terrain(terrain);
 	assert(!underlying.empty());
 	ss << name;
@@ -119,8 +120,8 @@ gamemap::gamemap(const config& cfg, const std::string& data):
 		usage_(IS_MAP)
 {
 	DBG_G << "loading map: '" << data << "'\n";
-	const config::child_list& terrains = cfg.get_children("terrain");
-	create_terrain_maps(terrains,terrainList_,tcodeToTerrain_);
+	const config::const_child_itors &terrains = cfg.child_range("terrain_type");
+	create_terrain_maps(terrains, terrainList_, tcodeToTerrain_);
 
 	read(data);
 }
@@ -277,7 +278,7 @@ std::string gamemap::write() const
 
 void gamemap::overlay(const gamemap& m, const config& rules_cfg, int xpos, int ypos, bool border)
 {
-	const config::child_list& rules = rules_cfg.get_children("rule");
+	const config::const_child_itors &rules = rules_cfg.child_range("rule");
 	int actual_border = (m.border_size() == border_size()) && border ? border_size() : 0;
 
 	const int xstart = std::max<int>(-actual_border, -xpos - actual_border);
@@ -298,11 +299,12 @@ void gamemap::overlay(const gamemap& m, const config& rules_cfg, int xpos, int y
 			}
 
 			// See if there is a matching rule
-			config::child_list::const_iterator rule = rules.begin();
-			for( ; rule != rules.end(); ++rule) {
+			config::const_child_iterator rule = rules.first;
+			for( ; rule != rules.second; ++rule)
+			{
 				static const std::string src_key = "old", src_not_key = "old_not",
 				                         dst_key = "new", dst_not_key = "new_not";
-				const config& cfg = **rule;
+				const config &cfg = *rule;
 				const t_translation::t_list& src = t_translation::read_list(cfg[src_key]);
 
 				if(!src.empty() && t_translation::terrain_matches(current, src) == false) {
@@ -331,8 +333,9 @@ void gamemap::overlay(const gamemap& m, const config& rules_cfg, int xpos, int y
 			}
 
 
-			if(rule != rules.end()) {
-				const config& cfg = **rule;
+			if (rule != rules.second)
+			{
+				const config &cfg = *rule;
 				const t_translation::t_list& terrain = t_translation::read_list(cfg["terrain"]);
 
 				tmerge_mode mode = BOTH;
