@@ -1,6 +1,6 @@
-/* $Id: map_label.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: map_label.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,53 +15,50 @@
 #ifndef MAP_LABEL_HPP_INCLUDED
 #define MAP_LABEL_HPP_INCLUDED
 
-class config;
-#include "map.hpp"
+#include "map_location.hpp"
 #include "font.hpp"
 
-#include "serialization/string_utils.hpp"
-
 #include <map>
 #include <string>
 
+class config;
 class display;
+class team;
 class terrain_label;
-class game_state;
-
 
 
 class map_labels
 {
 public:
-	typedef std::map<gamemap::location,const terrain_label*> label_map;
+	typedef std::map<map_location, terrain_label *> label_map;
 	typedef std::map<std::string,label_map> team_label_map;
 
-	map_labels(const display& disp, const gamemap& map, const team*);
-	map_labels(const display& disp, const config& cfg, const gamemap& map, const team*, const variable_set *variables);
+	map_labels(const display& disp, const team*);
 	~map_labels();
 
 	void write(config& res) const;
-	void read(const config& cfg, const variable_set *variables);
+	void read(const config &cfg);
 
 	static size_t get_max_chars();
 
-	const terrain_label* get_label(const gamemap::location& loc, const std::string& team_name);
+	const terrain_label* get_label(const map_location& loc, const std::string& team_name);
 	// search a team-only label, if fails then try public labels
-	const terrain_label* get_label(const gamemap::location& loc);
-	const terrain_label* set_label(const gamemap::location& loc,
+	const terrain_label* get_label(const map_location& loc);
+	const terrain_label* set_label(const map_location& loc,
 							   const std::string& text,
-							   const std::string team = "",
-							   const SDL_Color colour = font::NORMAL_COLOUR);
+							   const std::string& team = "",
+							   const SDL_Color colour = font::NORMAL_COLOUR,
+							   const bool visible_in_fog = true,
+							   const bool visible_in_shroud = false);
 
-	void add_label(const gamemap::location&,
-				   const terrain_label*);
+	void add_label(const map_location &, terrain_label *);
 
 	void clear(const std::string&);
 
 	void scroll(double xmove, double ymove);
 
 	void recalculate_labels();
-	bool visible_global_label(const gamemap::location&) const;
+	bool visible_global_label(const map_location&) const;
 
 	void recalculate_shroud();
 
@@ -71,18 +68,16 @@ public:
 
 	void set_team(const team*);
 
-private:
-	void clear_map(const label_map&);
 	void clear_all();
+private:
+	void clear_map(label_map &);
 	map_labels(const map_labels&);
 	void operator=(const map_labels&);
 
 	const display& disp_;
 	const team* team_;
-	const gamemap& map_;
 
 	team_label_map labels_;
-	label_map label_cache_;
 };
 
 /// To store label data
@@ -92,24 +87,24 @@ class terrain_label
 public:
 	terrain_label(const std::string&,
 				  const std::string&,
-				  const gamemap::location&,
+				  const map_location&,
 				  const map_labels&,
-				  const SDL_Color colour = font::NORMAL_COLOUR);
+				  const SDL_Color colour = font::NORMAL_COLOUR,
+				  const bool visible_in_fog = true,
+				  const bool visible_in_shroud = false);
 
-	terrain_label(const map_labels&,
-		      const config&,
-		      const variable_set *variables);
-
-	terrain_label(const map_labels&);
+	terrain_label(const map_labels &, const config &);
 
 	~terrain_label();
 
 	void write(config& res) const;
-	void read(const config& cfg, const variable_set *variables);
+	void read(const config &cfg);
 
 	const std::string& text() const;
 	const std::string& team_name() const;
-	const gamemap::location& location() const;
+	bool visible_in_fog() const;
+	bool visible_in_shroud() const;
+	const map_location& location() const;
 	const SDL_Colour& colour() const;
 
 	void set_text(const std::string&);
@@ -130,16 +125,19 @@ private:
 	void draw();
 	bool visible() const;
 	void check_text_length();
-	const std::string cfg_colour() const;
+	std::string cfg_colour() const;
 
 	int handle_;
 
 	std::string text_;
 	std::string team_name_;
+	bool visible_in_fog_;
+	bool visible_in_shroud_;
+
 	SDL_Color	colour_;
 
 	const map_labels* parent_;
-	gamemap::location loc_;
+	map_location loc_;
 
 };
 
