@@ -1,6 +1,6 @@
-/* $Id: mapgen_dialog.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: mapgen_dialog.cpp 49411 2011-05-07 15:11:50Z soliton $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -245,7 +245,7 @@ void default_map_generator::user_config(display& disp)
 	link_castles.set_check(link_castles_);
 	link_castles.set_location(link_rect);
 
-	for(bool draw = true;; draw = false) {
+	while(true) {
 		nplayers_ = players_slider.value();
 		width_ = width_slider.value();
 		height_ = height_slider.value();
