@@ -1,6 +1,6 @@
-/* $Id: multiplayer.cpp 33768 2009-03-17 19:22:14Z baufo $ */
+/* $Id: multiplayer.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2007 - 2009
+   Copyright (C) 2007 - 2011
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -16,21 +16,29 @@
 
 #include "dialogs.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
+#include "gui/dialogs/lobby_main.hpp"
+#include "gui/dialogs/message.hpp"
 #include "gui/dialogs/mp_connect.hpp"
 #include "gui/dialogs/mp_create_game.hpp"
+#include "gui/widgets/settings.hpp"
 #include "gui/widgets/window.hpp"
-#include "md5.hpp"
+#include "hash.hpp"
 #include "multiplayer.hpp"
 #include "multiplayer_connect.hpp"
+#include "multiplayer_create.hpp"
 #include "multiplayer_error_codes.hpp"
 #include "multiplayer_wait.hpp"
 #include "multiplayer_lobby.hpp"
 #include "playmp_controller.hpp"
 #include "playcampaign.hpp"
-#include "upload_log.hpp"
 #include "formula_string_utils.hpp"
+#include "sound.hpp"
 
-#define LOG_NW LOG_STREAM(info, network)
+#include <boost/bind.hpp>
+
+static lg::log_domain log_network("network");
+#define LOG_NW LOG_STREAM(info, log_network)
 
 namespace {
 
@@ -65,13 +73,15 @@ static void run_lobby_loop(display& disp, mp::ui& ui)
 			ui.set_location(lobby_pos);
 			first = false;
 		}
+		// process network data first so user actions can override the result
+		// or uptodate data can prevent invalid actions
+		// i.e. press cancel while you receive [start_game] or press start game while someone leaves
+		ui.process_network();
 
 		events::pump();
 		events::raise_process_event();
 		events::raise_draw_event();
 
-		ui.process_network();
-
 		disp.flip();
 		disp.delay(20);
 	}
@@ -123,7 +133,6 @@ static server_type open_connection(game_display& disp, const std::string& origin
 	std::set<hostpair> shown_hosts;
 	shown_hosts.insert(hostpair(host, port));
 
-	config::child_list redirects;
 	config data;
 	sock = dialogs::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
 
@@ -150,11 +159,10 @@ static server_type open_connection(game_display& disp, const std::string& origin
 		}
 
 		// Check for "redirect" messages
-		if(data.child("redirect")) {
-			config* redirect = data.child("redirect");
-
-			host = (*redirect)["host"];
-			port = lexical_cast_default<unsigned int>((*redirect)["port"], 15000);
+		if (const config &redirect = data.child("redirect"))
+		{
+			host = redirect["host"];
+			port = lexical_cast_default<unsigned int>(redirect["port"], 15000);
 
 			if(shown_hosts.find(hostpair(host,port)) != shown_hosts.end()) {
 				throw network::error(_("Server-side redirect loop"));
@@ -207,16 +215,16 @@ static server_type open_connection(game_display& disp, const std::string& origin
 					throw network::error(_("Connection timed out"));
 				}
 
-				config* error = data.child("error");
+				config *error = &data.child("error");
 
 				// ... and get us out of here if the server did not complain
-				if(!error) break;
+				if (!*error) break;
 
 				do {
 					std::string password = preferences::password();
 
 					bool fall_through = (*error)["force_confirmation"] == "yes" ?
-						gui::dialog(disp,_("Confirm"),(*error)["message"],gui::OK_CANCEL).show() :
+						(gui::dialog(disp,_("Confirm"),(*error)["message"],gui::OK_CANCEL).show() != 0) :
 						false;
 
 					const bool is_pw_request = !((*error)["password_request"].empty()) && !(password.empty());
@@ -242,73 +250,14 @@ static server_type open_connection(game_display& disp, const std::string& origin
 								for(std::string::size_type pos = 0; (pos = password.find('>', pos)) != std::string::npos; ++pos )
 									password.replace(pos, 1, "&gt;");
 
-								// start the hashing
-								std::string result;
+								const std::string salt = (*error)["salt"];
 
-								// Check if we have everything we need
-								if((*error)["salt"].empty() || (*error)["hash_seed"].empty()) {
-									return ABORT_SERVER;
+								if (salt.length() < 12) {
+									throw network::error(_("Bad data received from server"));
 								}
 
-								std::string itoa64("./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
-
-								std::string salt = (*error)["salt"];
-								int hash_seed;
-								try {
-									hash_seed = lexical_cast_default<int>((*error)["hash_seed"]);
-								} catch (bad_lexical_cast) {
-									std::cerr << "Bad lexical cast reading hash_seed\n";
-									return ABORT_SERVER;
-								}
-
-								// Start the MD5 hashing
-								salt.append(password);
-								MD5 md5_worker;
-								md5_worker.update((unsigned char *)salt.c_str(),salt.length());
-								md5_worker.finalize();
-								unsigned char * output = (unsigned char *) malloc (sizeof(unsigned char) * 16);
-								output = md5_worker.raw_digest();
-								std::string temp_hash;
-								do {
-									temp_hash = std::string((char *) output, (char *) output + 16);
-									temp_hash.append(password);
-									MD5 md5_worker;
-									md5_worker.update((unsigned char *)temp_hash.c_str(),temp_hash.length());
-									md5_worker.finalize();
-									output =  md5_worker.raw_digest();
-								} while (--hash_seed);
-								temp_hash = std::string((char *) output, (char *) output + 16);
-
-								// Now encode the resulting mix
-								std::string encoded_hash;
-								unsigned int i = 0, value;
-								do {
-									value = output[i++];
-									encoded_hash.append(itoa64.substr(value & 0x3f,1));
-									if(i < 16)
-										value |= (int)output[i] << 8;
-									encoded_hash.append(itoa64.substr((value >> 6) & 0x3f,1));
-									if(i++ >= 16)
-										break;
-									if(i < 16)
-										value |= (int)output[i] << 16;
-									encoded_hash.append(itoa64.substr((value >> 12) & 0x3f,1));
-									if(i++ >= 16)
-										break;
-									encoded_hash.append(itoa64.substr((value >> 18) & 0x3f,1));
-								} while (i < 16);
-								free (output);
-
-								// Now mix the resulting hash with the random seed
-								result = encoded_hash + (*error)["random_salt"];
-
-								MD5 md5_worker2;
-								md5_worker2.update((unsigned char *)result.c_str(), result.size());
-								md5_worker2.finalize();
-
-								result = std::string(md5_worker2.hex_digest());
-
-								sp["password"] = result;
+								sp["password"] = util::create_hash(util::create_hash(password, util::get_salt(salt),
+										util::get_iteration_count(salt)), salt.substr(12, 8));
 
 							} else {
 								sp["password"] = password;
@@ -325,10 +274,10 @@ static server_type open_connection(game_display& disp, const std::string& origin
 							throw network::error(_("Connection timed out"));
 						}
 
-						error = data.child("error");
+						error = &data.child("error");
 
 						// ... and get us out of here if the server is happy now
-						if(!error) break;
+						if (!*error) break;
 
 
 					}
@@ -358,8 +307,8 @@ static server_type open_connection(game_display& disp, const std::string& origin
 					} else if((*error)["error_code"] == MP_NAME_RESERVED_ERROR) {
 						error_message = vgettext("The nick '$nick' is reserved and cannot be used by players.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_NAME_UNREGISTERED_ERROR) {
-						/** @todo  add translatable message once we are out of the string freeze */
-						error_message = (*error)["message"];
+						error_message = vgettext("The nick '$nick' is not registered on this server.", i18n_symbols)
+								+ _(" This server disallows unregistered nicks.");
 					} else if((*error)["error_code"] == MP_PASSWORD_REQUEST) {
 						error_message = vgettext("The nick '$nick' is registered on this server.", i18n_symbols);
 					} else if((*error)["error_code"] == MP_PASSWORD_REQUEST_FOR_LOGGED_IN_NAME) {
@@ -396,7 +345,7 @@ static server_type open_connection(game_display& disp, const std::string& origin
 				// Somewhat hacky...
 				// If we broke out of the do-while loop above error
 				// is still going to be NULL
-				if(!error) break;
+				if(!*error) break;
 			} // end login loop
 		}
 	} while(!(data.child("join_lobby") || data.child("join_game")));
@@ -430,7 +379,6 @@ static void enter_wait_mode(game_display& disp, const config& game_config, mp::c
 	mp::ui::result res;
 	game_state state;
 	network_game_manager m;
-	upload_log nolog(false);
 
 	gamelist.clear();
 	statistics::fresh_stats();
@@ -459,7 +407,7 @@ static void enter_wait_mode(game_display& disp, const config& game_config, mp::c
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, nolog, IO_CLIENT,
+		play_game(disp, state, game_config, IO_CLIENT,
 			preferences::skip_mp_replay() && observe);
 		recorder.clear();
 
@@ -473,20 +421,19 @@ static void enter_wait_mode(game_display& disp, const config& game_config, mp::c
 static void enter_create_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist, mp::controller default_controller, bool local_players_only = false);
 
 static void enter_connect_mode(game_display& disp, const config& game_config,
-		mp::chat& chat, config& gamelist, const mp::create::parameters& params,
-		mp::controller default_controller, bool local_players_only = false)
+		mp::chat& chat, config& gamelist, const mp_game_settings& params,
+		const int num_turns, mp::controller default_controller, bool local_players_only = false)
 {
 	mp::ui::result res;
 	game_state state;
 	const network::manager net_manager(1,1);
 	network_game_manager m;
-	upload_log nolog(false);
 
 	gamelist.clear();
 	statistics::fresh_stats();
 
 	{
-		mp::connect ui(disp, game_config, chat, gamelist, params, default_controller, local_players_only);
+		mp::connect ui(disp, game_config, chat, gamelist, params, num_turns, default_controller, local_players_only);
 		run_lobby_loop(disp, ui);
 
 		res = ui.get_result();
@@ -501,7 +448,7 @@ static void enter_connect_mode(game_display& disp, const config& game_config,
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, nolog, IO_SERVER);
+		play_game(disp, state, game_config, IO_SERVER);
 		recorder.clear();
 
 		break;
@@ -517,7 +464,7 @@ static void enter_connect_mode(game_display& disp, const config& game_config,
 
 static void enter_create_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist, mp::controller default_controller, bool local_players_only)
 {
-	if(gui2::new_widgets) {
+	if (0 && gui2::new_widgets) {
 
 		gui2::tmp_create_game dlg(game_config);
 
@@ -527,18 +474,20 @@ static void enter_create_mode(game_display& disp, const config& game_config, mp:
 	} else {
 
 		mp::ui::result res;
-		mp::create::parameters params;
+		mp_game_settings params;
+		int num_turns;
 
 		{
 			mp::create ui(disp, game_config, chat, gamelist);
 			run_lobby_loop(disp, ui);
 			res = ui.get_result();
 			params = ui.get_parameters();
+			num_turns = ui.num_turns();
 		}
 
 		switch (res) {
 		case mp::ui::CREATE:
-			enter_connect_mode(disp, game_config, chat, gamelist, params, default_controller, local_players_only);
+			enter_connect_mode(disp, game_config, chat, gamelist, params, num_turns, default_controller, local_players_only);
 			break;
 		case mp::ui::QUIT:
 		default:
@@ -549,12 +498,74 @@ static void enter_create_mode(game_display& disp, const config& game_config, mp:
 	}
 }
 
+static void do_preferences_dialog(game_display& disp, const config& game_config)
+{
+	const preferences::display_manager disp_manager(&disp);
+	preferences::show_preferences_dialog(disp,game_config);
+
+	/**
+	 * The screen size might have changed force an update of the size.
+	 *
+	 * @todo This might no longer be needed when gui2 is done.
+	 */
+	const SDL_Rect rect = screen_area();
+	preferences::set_resolution(disp.video(), rect.w, rect.h);
+
+	gui2::settings::gamemap_width += rect.w - gui2::settings::screen_width ;
+	gui2::settings::gamemap_height += rect.h - gui2::settings::screen_height ;
+	gui2::settings::screen_width = rect.w;
+	gui2::settings::screen_height = rect.h;
+}
+
 static void enter_lobby_mode(game_display& disp, const config& game_config, mp::chat& chat, config& gamelist)
 {
+
+
 	mp::ui::result res;
 
 	while (true) {
-		{
+		const config &cfg = game_config.child("lobby_music");
+		if (cfg) {
+			foreach (const config &i, cfg.child_range("music")) {
+				sound::play_music_config(i);
+			}
+			sound::commit_music_changes();
+		} else {
+			sound::empty_playlist();
+			sound::stop_music();
+		}
+		lobby_info li(game_config);
+
+		// Force a black background
+		const Uint32 colour = SDL_MapRGBA(disp.video().getSurface()->format
+				, 0
+				, 0
+				, 0
+				, 255);
+
+		SDL_FillRect(disp.video().getSurface(), NULL, colour);
+
+		if(preferences::new_lobby()) {
+			gui2::tlobby_main dlg(game_config, li, disp);
+			dlg.set_preferences_callback(
+				boost::bind(do_preferences_dialog,
+					boost::ref(disp), boost::ref(game_config)));
+			dlg.show(disp.video());
+			//ugly kludge for launching other dialogs like the old lobby
+			switch (dlg.get_legacy_result()) {
+				case gui2::tlobby_main::CREATE:
+					res = mp::ui::CREATE;
+					break;
+				case gui2::tlobby_main::JOIN:
+					res = mp::ui::JOIN;
+					break;
+				case gui2::tlobby_main::OBSERVE:
+					res = mp::ui::OBSERVE;
+					break;
+				default:
+					res = mp::ui::QUIT;
+			}
+		} else {
 			mp::lobby ui(disp, game_config, chat, gamelist);
 			run_lobby_loop(disp, ui);
 			res = ui.get_result();
@@ -566,7 +577,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 				enter_wait_mode(disp, game_config, chat, gamelist, false);
 			} catch(config::error& error) {
 				if(!error.message.empty()) {
-					gui::show_error_message(disp, error.message);
+					gui2::show_error_message(disp.video(), error.message);
 				}
 				//update lobby content
 				network::send_data(config("refresh_lobby"), 0, true);
@@ -577,7 +588,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 				enter_wait_mode(disp, game_config, chat, gamelist, true);
 			} catch(config::error& error) {
 				if(!error.message.empty()) {
-					gui::show_error_message(disp, error.message);
+					gui2::show_error_message(disp.video(), error.message);
 				}
 			}
 			// update lobby content unconditionally because we might have left only after the
@@ -589,7 +600,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 				enter_create_mode(disp, game_config, chat, gamelist, mp::CNTR_NETWORK);
 			} catch(config::error& error) {
 				if (!error.message.empty())
-					gui::show_error_message(disp, error.message);
+					gui2::show_error_message(disp.video(), error.message);
 				//update lobby content
 				network::send_data(config("refresh_lobby"), 0, true);
 			}
@@ -598,8 +609,7 @@ static void enter_lobby_mode(game_display& disp, const config& game_config, mp::
 			return;
 		case mp::ui::PREFERENCES:
 			{
-				const preferences::display_manager disp_manager(&disp);
-				preferences::show_preferences_dialog(disp,game_config);
+				do_preferences_dialog(disp, game_config);
 				//update lobby content
 				network::send_data(config("refresh_lobby"), 0, true);
 			}
@@ -624,7 +634,7 @@ void start_local_game(game_display& disp, const config& game_config,
 }
 
 void start_client(game_display& disp, const config& game_config,
-		const std::string host)
+		const std::string& host)
 {
 	const rand_rng::set_random_generator generator_setter(&recorder);
 	const network::manager net_manager(1,1);
