@@ -1,4 +1,4 @@
-/* $Id: multiplayer_lobby.cpp 36683 2009-07-07 13:36:09Z soliton $ */
+/* $Id: multiplayer_lobby.cpp 48533 2011-02-16 22:27:08Z espreon $ */
 /*
    Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -18,6 +18,9 @@
 
 #include "global.hpp"
 
+#include "filesystem.hpp"
+#include "foreach.hpp"
+#include "lobby_preferences.hpp"
 #include "map_exception.hpp"
 #include "marked-up_text.hpp"
 #include "minimap.hpp"
@@ -29,14 +32,15 @@
 #include "wml_exception.hpp"
 #include "formula_string_utils.hpp"
 
-#define ERR_CF LOG_STREAM(err, config)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
 
 namespace {
 std::vector<std::string> empty_string_vector;
 }
 
 namespace mp {
-gamebrowser::gamebrowser(CVideo& video, const config* map_hashes) :
+gamebrowser::gamebrowser(CVideo& video, const config &map_hashes) :
 	menu(video, empty_string_vector, false, -1, -1, NULL, &menu::bluebg_style),
 	gold_icon_locator_("themes/gold.png"),
 	xp_icon_locator_("themes/units.png"),
@@ -331,8 +335,8 @@ void gamebrowser::handle_event(const SDL_Event& event)
 			x = event.button.x;
 			y = event.button.y;
 		} else {
-			x = (long)event.user.data1;
-			y = (long)event.user.data2;
+			x = reinterpret_cast<long>(event.user.data1);
+			y = reinterpret_cast<long>(event.user.data2);
 		}
 		const SDL_Rect& loc = inner_location();
 
@@ -403,23 +407,23 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 	}
 
 	games_.clear();
-	config::child_list games = cfg.get_children("game");
-	config::child_iterator game;
 
-	for(game = games.begin(); game != games.end(); ++game) {
+	foreach (const config &game, cfg.child("gamelist").child_range("game"))
+	{
 		bool verified = true;
 		games_.push_back(game_item());
-		games_.back().password_required = (**game)["password"] == "yes";
-		games_.back().reloaded = (**game)["savegame"] == "yes";
+		games_.back().password_required = game["password"] == "yes";
+		games_.back().reloaded = game["savegame"] == "yes";
 		games_.back().have_era = true;
-		if((**game)["mp_era"] != "") {
-			const config* const era_cfg = game_config.find_child("era", "id", (**game)["mp_era"]);
+		if (!game["mp_era"].empty())
+		{
+			const config &era_cfg = game_config.find_child("era", "id", game["mp_era"]);
 			utils::string_map symbols;
-			symbols["era_id"] = (**game)["mp_era"];
-			if (era_cfg != NULL) {
-				games_.back().map_info = era_cfg->get_attribute("name");
+			symbols["era_id"] = game["mp_era"];
+			if (era_cfg) {
+				games_.back().map_info = era_cfg["name"];
 			} else {
-				if((**game)["require_era"] == "no") {
+				if (game["require_era"] == "no") {
 					games_.back().have_era = true;
 				} else {
 					games_.back().have_era = false;
@@ -431,9 +435,9 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			games_.back().map_info = _("Unknown era");
 			verified = false;
 		}
-		games_.back().map_data = (**game)["map_data"];
+		games_.back().map_data = game["map_data"];
 		if(games_.back().map_data.empty()) {
-			games_.back().map_data = read_map((**game)["map"]);
+			games_.back().map_data = read_map(game["map"]);
 		}
 
 		if(! games_.back().map_data.empty()) {
@@ -441,7 +445,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 				std::vector<minimap_cache_item>::iterator i;
 				bool found = false;
 				for(i = minimap_cache.begin(); i != minimap_cache.end() && !found; ++i) {
- 					if (i->map_data == games_.back().map_data) {
+					if (i->map_data == games_.back().map_data) {
 						found = true;
 						games_.back().map_info_size = i->map_info_size;
 						games_.back().mini_map = i->mini_map;
@@ -466,23 +470,24 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			games_.back().map_info += " - ??x??";
 		}
 		games_.back().map_info += " ";
-		if((**game)["mp_scenario"] != "") {
+		if (!game["mp_scenario"].empty())
+		{
 			// check if it's a multiplayer scenario
-			const config* level_cfg = game_config.find_child("multiplayer", "id", (**game)["mp_scenario"]);
-			if(level_cfg == NULL) {
+			const config *level_cfg = &game_config.find_child("multiplayer", "id", game["mp_scenario"]);
+			if (!*level_cfg) {
 				// check if it's a user map
-				level_cfg = game_config.find_child("generic_multiplayer", "id", (**game)["mp_scenario"]);
+				level_cfg = &game_config.find_child("generic_multiplayer", "id", game["mp_scenario"]);
 			}
-			if(level_cfg) {
+			if (*level_cfg) {
 				games_.back().map_info += level_cfg->get_attribute("name");
 				// reloaded games do not match the original scenario hash,
 				// so it makes no sense to test them, they always would appear
 				// as remote scenarios
 				if (map_hashes_ && !games_.back().reloaded) {
-					const std::string& hash = (**game)["hash"];
+					std::string hash = game["hash"];
 					bool hash_found = false;
-					for(string_map::const_iterator i = map_hashes_->values.begin(); i != map_hashes_->values.end(); ++i) {
-						if(i->first == (**game)["mp_scenario"] && i->second == hash) {
+					foreach (const config::attribute &i, map_hashes_.attribute_range()) {
+						if (i.first == game["mp_scenario"] && i.second == hash) {
 							hash_found = true;
 							break;
 						}
@@ -495,7 +500,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 				}
 			} else {
 				utils::string_map symbols;
-				symbols["scenario_id"] = (**game)["mp_scenario"];
+				symbols["scenario_id"] = game["mp_scenario"];
 				games_.back().map_info += vgettext("Unknown scenario: $scenario_id", symbols);
 				verified = false;
 			}
@@ -508,13 +513,13 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			games_.back().map_info += _("Reloaded game");
 			verified = false;
 		}
-		games_.back().id = (**game)["id"];
-		games_.back().name = (**game)["name"];
-		const std::string& turn = (**game)["turn"];
-		const std::string& slots = (**game)["slots"];
+		games_.back().id = game["id"];
+		games_.back().name = game["name"];
+		std::string turn = game["turn"];
+		std::string slots = game["slots"];
 		games_.back().vacant_slots = lexical_cast_default<size_t>(slots, 0);
 		games_.back().current_turn = 0;
-		if(turn != "") {
+		if (!turn.empty()) {
 			games_.back().started = true;
 			int index = turn.find_first_of('/');
 			if (index > -1){
@@ -533,19 +538,19 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			}
 		}
 
-		games_.back().use_map_settings = ((**game)["mp_use_map_settings"] == "yes");
-		games_.back().gold = (**game)["mp_village_gold"];
-		if((**game)["mp_fog"] == "yes") {
+		games_.back().use_map_settings = game["mp_use_map_settings"] == "yes";
+		games_.back().gold = game["mp_village_gold"];
+		if (game["mp_fog"] == "yes") {
 			games_.back().vision = _("Fog");
 			games_.back().fog = true;
-			if((**game)["mp_shroud"] == "yes") {
+			if (game["mp_shroud"] == "yes") {
 				games_.back().vision += "/";
 				games_.back().vision += _("Shroud");
 				games_.back().shroud = true;
 			} else {
 				games_.back().shroud = false;
 			}
-		} else if((**game)["mp_shroud"] == "yes") {
+		} else if (game["mp_shroud"] == "yes") {
 			games_.back().vision = _("Shroud");
 			games_.back().fog = false;
 			games_.back().shroud = true;
@@ -554,16 +559,19 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			games_.back().fog = false;
 			games_.back().shroud = false;
 		}
-		if((**game)["mp_countdown"] == "yes" ) {
-			games_.back().time_limit =   (**game)["mp_countdown_init_time"] + " / +"
-			                           + (**game)["mp_countdown_turn_bonus"] + " "
-			                           + (**game)["mp_countdown_action_bonus"];
+		if (game["mp_countdown"] == "yes" ) {
+			games_.back().time_limit =   game["mp_countdown_init_time"] + " / +"
+			                           + game["mp_countdown_turn_bonus"] + " "
+			                           + game["mp_countdown_action_bonus"];
 		} else {
 			games_.back().time_limit = "";
 		}
-		games_.back().xp = (**game)["experience_modifier"] + "%";
-		games_.back().observers = (**game)["observer"] != "no" ? true : false;
+		games_.back().xp = game["experience_modifier"] + "%";
+		games_.back().observers = game["observer"] != "no" ? true : false;
 		games_.back().verified = verified;
+
+		// Hack...
+		if(preferences::fi_invert() ? game_matches_filter(games_.back(), cfg) : !game_matches_filter(games_.back(), cfg)) games_.pop_back();
 	}
 	set_full_size(games_.size());
 	set_shown_size(inner_location().h / row_height());
@@ -593,7 +601,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 	set_dirty();
 }
 
-void gamebrowser::select_game(const std::string id) {
+void gamebrowser::select_game(const std::string& id) {
 	if (id.empty()) return;
 
 	for (unsigned int i=0; i < games_.size(); i++) {
@@ -606,6 +614,38 @@ void gamebrowser::select_game(const std::string id) {
 	set_dirty();
 }
 
+bool gamebrowser::game_matches_filter(const game_item& i, const config& cfg) {
+
+    if(!preferences::filter_lobby()) return true;
+
+    if(preferences::fi_vacant_slots() && i.vacant_slots == 0) return false;
+
+    if(preferences::fi_friends_in_game()) {
+        bool found_friend = false;
+        foreach(const config &user, cfg.child_range("user")) {
+            if(preferences::is_friend(user["name"]) && user["game_id"] == i.id) {
+                found_friend = true;
+                break;
+            }
+        }
+        if(!found_friend) return false;
+    }
+
+    if(!preferences::fi_text().empty()) {
+        bool found_match = true;
+        foreach(const std::string& search_string, utils::split(preferences::fi_text(), ' ', utils::STRIP_SPACES)) {
+            if(std::search(i.map_info.begin(), i.map_info.end(), search_string.begin(), search_string.end(), chars_equal_insensitive) == i.map_info.end() &&
+                    std::search(i.name.begin(), i.name.end(), search_string.begin(), search_string.end(), chars_equal_insensitive) == i.name.end()) {
+                found_match = false;
+                break;
+            }
+        }
+        if(!found_match) return false;
+    }
+
+    return true;
+}
+
 lobby::lobby_sorter::lobby_sorter(const config& cfg) : cfg_(cfg)
 {
 	set_alpha_sort(1);
@@ -625,18 +665,22 @@ bool lobby::lobby_sorter::column_sortable(int column) const
 
 bool lobby::lobby_sorter::less(int column, const gui::menu::item& row1, const gui::menu::item& row2) const
 {
-	const config* const list = cfg_.child("gamelist");
-	if(list == NULL) {
+	const config &list = cfg_.child("gamelist");
+	if (!list) {
 		return false;
 	}
 
-	const config::child_list& games = list->get_children("game");
-	if(row1.id >= games.size() || row2.id >= games.size()) {
+	size_t nb = list.child_count("game");
+	if(row1.id >= nb || row2.id >= nb) {
 		return false;
 	}
 
-	const config& game1 = *games[row1.id];
-	const config& game2 = *games[row2.id];
+	config::const_child_iterator gi = list.child_range("game").first, gs = gi;
+	std::advance(gi, row1.id);
+	const config &game1 = *gi;
+	gi = gs;
+	std::advance(gi, row2.id);
+	const config &game2 = *gi;
 
 	if(column == MAP_COLUMN) {
 		size_t mapsize1 = game1["map_data"].size();
@@ -686,12 +730,45 @@ lobby::lobby(game_display& disp, const config& cfg, chat& c, config& gamelist) :
 	game_preferences_(disp.video(), _("Preferences")),
 #endif
 	quit_game_(disp.video(), _("Quit")),
+#ifndef USE_TINY_GUI
+	apply_filter_(disp.video(), _("Apply Filter"), gui::button::TYPE_CHECK),
+	invert_filter_(disp.video(), _("Invert"), gui::button::TYPE_CHECK),
+	vacant_slots_(disp.video(), _("Vacant Slots"), gui::button::TYPE_CHECK),
+	friends_in_game_(disp.video(), _("Friends in Game"), gui::button::TYPE_CHECK),
+	filter_label_(disp.video(), _("Filter:")),
+	filter_text_(disp.video(), 150),
+#endif
 	last_selected_game_(-1), sorter_(gamelist),
 	games_menu_(disp.video(),cfg.child("multiplayer_hashes")),
-	minimaps_()
+	minimaps_(),
+	search_string_(preferences::fi_text())
 {
 	skip_replay_.set_check(preferences::skip_mp_replay());
 	skip_replay_.set_help_string(_("Skip quickly to the active turn when observing"));
+
+#ifndef USE_TINY_GUI
+	apply_filter_.set_check(preferences::filter_lobby());
+	apply_filter_.set_help_string(_("Enable the games filter. If unchecked all games are shown, regardless of any filter."));
+
+	invert_filter_.set_check(preferences::fi_invert());
+	invert_filter_.set_help_string(_("Show all games that do *not* match your filter. Useful for hiding games you are not interested in."));
+	invert_filter_.enable(apply_filter_.checked());
+
+	vacant_slots_.set_check(preferences::fi_vacant_slots());
+	vacant_slots_.set_help_string(_("Only show games that have at least one vacant slot"));
+	vacant_slots_.enable(apply_filter_.checked());
+
+	friends_in_game_.set_check(preferences::fi_friends_in_game());
+	friends_in_game_.set_help_string(_("Only show games that are played or observed by at least one of your friends"));
+	friends_in_game_.enable(apply_filter_.checked());
+	
+	filter_label_.enable(apply_filter_.checked());
+
+	filter_text_.set_text(search_string_);
+	filter_text_.set_help_string(_("Only show games whose title or description contain the entered text"));
+	filter_text_.set_editable(apply_filter_.checked());
+#endif
+
 	game_config::debug = false;
 	gamelist_updated();
 	sound::play_music_repeatedly(game_config::lobby_music);
@@ -710,6 +787,14 @@ void lobby::hide_children(bool hide)
 	game_preferences_.hide(hide);
 #endif
 	quit_game_.hide(hide);
+#ifndef USE_TINY_GUI
+	apply_filter_.hide(hide);
+	invert_filter_.hide(hide);
+	vacant_slots_.hide(hide);
+	friends_in_game_.hide(hide);
+	filter_label_.hide(hide);
+	filter_text_.hide(hide);
+#endif
 }
 
 void lobby::layout_children(const SDL_Rect& rect)
@@ -748,19 +833,33 @@ void lobby::layout_children(const SDL_Rect& rect)
 
 	games_menu_.set_location(client_area().x, client_area().y + title().height());
 	games_menu_.set_measurements(client_area().w, client_area().h
-			- title().height() - gui::ButtonVPadding);
+			- title().height() - gui::ButtonVPadding
+#ifndef USE_TINY_GUI
+			- apply_filter_.location().h
+#endif
+			);
+
+#ifndef USE_TINY_GUI
+    apply_filter_.set_location(client_area().x, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding);
+    invert_filter_.set_location(client_area().x + apply_filter_.location().w + btn_space, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding);
+    vacant_slots_.set_location(client_area().w - apply_filter_.location().w, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding);
+    friends_in_game_.set_location(vacant_slots_.location().x - friends_in_game_.location().w - btn_space, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding);
+    filter_text_.set_location(friends_in_game_.location().x - filter_text_.location().w - btn_space * 4, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding);
+    filter_label_.set_location(filter_text_.location().x - filter_label_.location().w - btn_space, games_menu_.location().y + games_menu_.location().h + gui::ButtonVPadding
+            + (apply_filter_.location().h - filter_label_.location().h) / 2);
+
+#endif
 }
 
 void lobby::gamelist_updated(bool silent)
 {
 	ui::gamelist_updated(silent);
-
-	const config* list = gamelist().child("gamelist");
-	if(list == NULL) {
+	const config &list = gamelist().child("gamelist");
+	if (!list) {
 		// No gamelist yet. Do not update anything.
 		return;
 	}
-	games_menu_.set_game_items(*list, game_config());
+	games_menu_.set_game_items(gamelist(), game_config());
 	join_game_.enable(games_menu_.selection_is_joinable());
 	observe_game_.enable(games_menu_.selection_is_observable());
 }
@@ -852,6 +951,50 @@ void lobby::process_event()
 		set_result(QUIT);
 		return;
 	}
+
+#ifndef USE_TINY_GUI
+	if(apply_filter_.pressed()) {
+		preferences::set_filter_lobby(apply_filter_.checked());
+		invert_filter_.enable(apply_filter_.checked());
+		vacant_slots_.enable(apply_filter_.checked());
+		friends_in_game_.enable(apply_filter_.checked());
+		filter_label_.enable(apply_filter_.checked());
+
+		/** @todo I am aware that the box is not grayed out even though
+                  it definitely should be. This is because the textbox
+                  class currently does not really have an easy way to do
+                  that. I'll have to look into this.
+		*/
+		filter_text_.set_editable(apply_filter_.checked());
+		gamelist_updated();
+		return;
+	}
+
+	if(invert_filter_.pressed()) {
+		preferences::set_fi_invert(invert_filter_.checked());
+		gamelist_updated();
+		return;
+	}
+
+	if(vacant_slots_.pressed()) {
+		preferences::set_fi_vacant_slots(vacant_slots_.checked());
+		gamelist_updated();
+		return;
+	}
+
+	if(friends_in_game_.pressed()) {
+		preferences::set_fi_friends_in_game(friends_in_game_.checked());
+		gamelist_updated();
+		return;
+	}
+
+	if(search_string_ != filter_text_.text()) {
+	    search_string_ = filter_text_.text();
+	    preferences::set_fi_text(search_string_);
+	    gamelist_updated();
+	}
+#endif
+
 }
 
 void lobby::process_network_data(const config& data, const network::connection sock)
