@@ -1,6 +1,6 @@
-/* $Id: play_controller.cpp 42178 2010-04-19 05:30:57Z silene $ */
+/* $Id: play_controller.cpp 49335 2011-04-30 11:28:30Z boucman $ */
 /*
-   Copyright (C) 2006 - 2010 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -482,7 +482,7 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 	team& current_team = teams_[team_index];
 
 	mouse_handler_.set_side(team_index + 1);
-
+	
 	// If we are observers we move to watch next team if it is allowed
 	if (is_observer()
 		&& !current_team.get_disallow_observers()) {
@@ -502,13 +502,13 @@ void play_controller::init_side(const unsigned int team_index, bool is_replay){
 		|| is_replay)
 		return;
 	if (!loading_game_) recorder.init_side();
-	do_init_side(team_index);
+	do_init_side(team_index, is_replay);
 }
 
 /**
  * Called by replay handler or init_side() to do actual work for turn change.
  */
-void play_controller::do_init_side(const unsigned int team_index){
+void play_controller::do_init_side(const unsigned int team_index, bool /*is_replay*/) {
 	log_scope("player turn");
 	team& current_team = teams_[team_index];
 
@@ -867,14 +867,6 @@ void play_controller::slice_before_scroll() {
 	soundsources_manager_->update();
 }
 
-void play_controller::slice_end() {
-	if(!browse_ && teams_[gui_->viewing_team()].objectives_changed()) {
-		check_end_level();
-		dialogs::show_objectives(level_, teams_[gui_->viewing_team()].objectives());
-		teams_[gui_->viewing_team()].reset_objectives_changed();
-	}
-}
-
 events::mouse_handler& play_controller::get_mouse_handler_base() {
 	return mouse_handler_;
 }
