@@ -1,7 +1,7 @@
-/* $Id: playsingle_controller.cpp 33833 2009-03-18 15:52:13Z shadowmaster $ */
+/* $Id: playsingle_controller.cpp 49335 2011-04-30 11:28:30Z boucman $ */
 
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -21,19 +21,33 @@
 
 #include "playsingle_controller.hpp"
 
-#include "ai.hpp"
+#include "actions.hpp"
+#include "ai/manager.hpp"
+#include "ai/game_info.hpp"
+#include "ai/testing.hpp"
+#include "dialogs.hpp"
+#include "foreach.hpp"
 #include "game_end_exceptions.hpp"
+#include "game_events.hpp"
+#include "game_preferences.hpp"
 #include "gettext.hpp"
-#include "intro.hpp"
+#include "gui/dialogs/transient_message.hpp"
 #include "log.hpp"
 #include "map_label.hpp"
 #include "marked-up_text.hpp"
+#include "playturn.hpp"
+#include "resources.hpp"
+#include "savegame.hpp"
 #include "sound.hpp"
-#include "upload_log.hpp"
 #include "formula_string_utils.hpp"
+#include "events.hpp"
+#include "save_blocker.hpp"
+#include "soundsource.hpp"
+#include "storyscreen/interface.hpp"
 
-#define ERR_NG LOG_STREAM(err, engine)
-#define LOG_NG LOG_STREAM(info, engine)
+static lg::log_domain log_engine("engine");
+#define ERR_NG LOG_STREAM(err, log_engine)
+#define LOG_NG LOG_STREAM(info, log_engine)
 
 playsingle_controller::playsingle_controller(const config& level,
 		game_state& state_of_game, const int ticks, const int num_turns,
@@ -48,24 +62,26 @@ playsingle_controller::playsingle_controller(const config& level,
 	replaying_(false),
 	turn_over_(false),
 	skip_next_turn_(false),
-	victory_music_(),
-	defeat_music_()
+	level_result_(NONE)
 {
 	// game may need to start in linger mode
-	if (state_of_game.completion == "victory" || state_of_game.completion == "defeat")
+	if (state_of_game.classification().completion == "victory" || state_of_game.classification().completion == "defeat")
 	{
 		LOG_NG << "Setting linger mode.\n";
 		browse_ = linger_ = true;
 	}
-}
 
+	ai::game_info ai_info(*gui_,map_,units_,teams_, tod_manager_, gamestate_);
+	ai::manager::set_ai_info(ai_info);
+	ai::manager::add_observer(this) ;
+}
 
 playsingle_controller::~playsingle_controller()
 {
-	ai_manager::reap_ais() ;
+	ai::manager::remove_observer(this) ;
+	ai::manager::clear_ais() ;
 }
 
-
 void playsingle_controller::init_gui(){
 	LOG_NG << "Initializing GUI... " << (SDL_GetTicks() - ticks_) << "\n";
 	play_controller::init_gui();
@@ -79,7 +95,7 @@ void playsingle_controller::init_gui(){
 	events::raise_draw_event();
 	gui_->draw();
 	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
-		::clear_shroud(*gui_,map_,units_,teams_,(t-teams_.begin()));
+		::clear_shroud(t - teams_.begin() + 1);
 	}
 }
 
@@ -99,11 +115,11 @@ void playsingle_controller::recall(){
 }
 
 void playsingle_controller::toggle_shroud_updates(){
-	menu_handler_.toggle_shroud_updates(player_number_);
+	menu_handler_.toggle_shroud_updates(gui_->viewing_team()+1);
 }
 
 void playsingle_controller::update_shroud_now(){
-	menu_handler_.update_shroud_now(player_number_);
+	menu_handler_.update_shroud_now(gui_->viewing_team()+1);
 }
 
 void playsingle_controller::end_turn(){
@@ -116,6 +132,25 @@ void playsingle_controller::end_turn(){
 	}
 }
 
+void playsingle_controller::force_end_turn(){
+	skip_next_turn_ = true;
+	end_turn_ = true;
+}
+
+void playsingle_controller::check_end_level()
+{
+	if ((level_result_ == NONE) || linger_)
+	{
+		team &t = teams_[gui_->viewing_team()];
+		if (!browse_ && t.objectives_changed()) {
+			dialogs::show_objectives(level_, t.objectives());
+			t.reset_objectives_changed();
+		}
+		return;
+	}
+	throw end_level_exception(level_result_);
+}
+
 void playsingle_controller::rename_unit(){
 	menu_handler_.rename_unit(mouse_handler_);
 }
@@ -124,8 +159,8 @@ void playsingle_controller::create_unit(){
 	menu_handler_.create_unit(mouse_handler_);
 }
 
-void playsingle_controller::change_unit_side(){
-	menu_handler_.change_unit_side(mouse_handler_);
+void playsingle_controller::change_side(){
+	menu_handler_.change_side(mouse_handler_);
 }
 
 void playsingle_controller::label_terrain(bool team_only){
@@ -136,6 +171,11 @@ void playsingle_controller::continue_move(){
 	menu_handler_.continue_move(mouse_handler_, player_number_);
 }
 
+void playsingle_controller::add_waypoint(){
+	if (!browse_)
+		mouse_handler_.add_waypoint(mouse_handler_.get_last_hex());
+}
+
 void playsingle_controller::unit_hold_position(){
 	if (!browse_)
 		menu_handler_.unit_hold_position(mouse_handler_, player_number_);
@@ -173,12 +213,10 @@ void playsingle_controller::user_command_3(){
 #endif
 
 void playsingle_controller::report_victory(
-		    std::stringstream& report,
-		    end_level_exception& end_level,
-		    int player_gold,
-		    int remaining_gold, int finishing_bonus_per_turn,
-		    int turns_left, int finishing_bonus)
+	std::ostringstream &report, int player_gold, int remaining_gold,
+	int finishing_bonus_per_turn, int turns_left, int finishing_bonus)
 {
+	end_level_data &end_level = get_end_level_data();
 	report << _("Remaining gold: ")
 		   << remaining_gold << "\n";
 	if(end_level.gold_bonus) {
@@ -186,11 +224,11 @@ void playsingle_controller::report_victory(
 			report << _("Early finish bonus: ")
 				   << finishing_bonus_per_turn
 				   << " " << _("per turn") << "\n"
-				   << font::BOLD_TEXT << _("Turns finished early: ")
-				   << turns_left << "\n"
+				   << "<b>" << _("Turns finished early: ")
+				   << turns_left << "</b>\n"
 				   << _("Bonus: ")
 				   << finishing_bonus << "\n";
-	  	}
+		}
 		report << _("Gold: ")
 		       << (remaining_gold + finishing_bonus);
 	}
@@ -198,24 +236,35 @@ void playsingle_controller::report_victory(
 		report << '\n' << _("Carry over percentage: ") << end_level.carryover_percentage;
 	}
 	if(end_level.carryover_add) {
-		report << '\n' << font::BOLD_TEXT << _("Bonus Gold: ") << player_gold;
+		report << "\n<b>" << _("Bonus Gold: ") << player_gold <<"</b>";
 	} else {
-		report << '\n' << font::BOLD_TEXT << _("Retained Gold: ") << player_gold;
+		report << "\n<b>" << _("Retained Gold: ") << player_gold << "</b>";
 	}
 
 	std::string goldmsg;
 	utils::string_map symbols;
+
 	symbols["gold"] = lexical_cast_default<std::string>(player_gold);
+
 	// Note that both strings are the same in english, but some languages will
 	// want to translate them differently.
 	if(end_level.carryover_add) {
-		goldmsg = vngettext(
-			"You will start the next scenario with $gold "
-			"on top of the defined minimum starting gold.",
-			"You will start the next scenario with $gold "
-			"on top of the defined minimum starting gold.",
-			player_gold, symbols);
+		if(player_gold > 0) {
+			goldmsg = vngettext(
+					"You will start the next scenario with $gold "
+					"on top of the defined minimum starting gold.",
+					"You will start the next scenario with $gold "
+					"on top of the defined minimum starting gold.",
+					player_gold, symbols);
 
+		} else {
+			goldmsg = vngettext(
+					"You will start the next scenario with "
+					"the defined minimum starting gold.",
+					"You will start the next scenario with "
+					"the defined minimum starting gold.",
+					player_gold, symbols);
+		}
 	} else {
 		goldmsg = vngettext(
 			"You will start the next scenario with $gold "
@@ -231,58 +280,50 @@ void playsingle_controller::report_victory(
 	report << '\n' << goldmsg;
 }
 
-LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& story, upload_log& log,
-												  bool skip_replay, end_level_exception* end_level_result)
+LEVEL_RESULT playsingle_controller::play_scenario(
+	const config::const_child_itors &story,
+	bool skip_replay)
 {
 	LOG_NG << "in playsingle_controller::play_scenario()...\n";
 
 	// Start music.
-	const config::child_list& m = level_.get_children("music");
-	config::const_child_iterator i;
-	for (i = m.begin(); i != m.end(); i++) {
-		sound::play_music_config(**i);
+	foreach (const config &m, level_.child_range("music")) {
+		sound::play_music_config(m);
 	}
 	sound::commit_music_changes();
 
 	if(!skip_replay) {
-		for(std::vector<config*>::const_iterator story_i = story.begin(); story_i != story.end(); ++story_i) {
-			show_intro(*gui_,vconfig(*story_i, true), level_);
+		foreach (const config &s, story) {
+			show_storyscreen(*gui_, vconfig(s, true), level_["name"]);
 		}
 	}
-	gui_->labels().read(level_, game_events::get_state_of_game());
+	gui_->labels().read(level_);
 
 	// Find a list of 'items' (i.e. overlays) on the level, and add them
-	const config::child_list& overlays = level_.get_children("item");
-	for(config::child_list::const_iterator overlay = overlays.begin(); overlay != overlays.end(); ++overlay) {
-		gui_->add_overlay(map_location(**overlay, game_events::get_state_of_game()), (**overlay)["image"], (**overlay)["halo"], (**overlay)["team_name"], utils::string_bool((**overlay)["visible_in_fog"], true));
+	foreach (const config &overlay, level_.child_range("item"))
+	{
+		gui_->add_overlay(
+			map_location(overlay, resources::state_of_game),
+			overlay["image"], overlay["halo"], overlay["team_name"],
+			utils::string_bool(overlay["visible_in_fog"], true));
 	}
 
 	// Read sound sources
 	assert(soundsources_manager_ != NULL);
-	const config::child_list& snd_sources = level_.get_children("sound_source");
-	for(config::child_list::const_iterator i = snd_sources.begin(); i != snd_sources.end(); ++i) {
-		assert(*i != NULL);
-		soundsource::sourcespec spec(**i);
+	foreach (const config &s, level_.child_range("sound_source")) {
+		soundsource::sourcespec spec(s);
 		soundsources_manager_->add(spec);
 	}
 
-	victory_conditions::set_victory_when_enemies_defeated(
-		utils::string_bool(level_["victory_when_enemies_defeated"], true)
-	);
-	victory_conditions::set_carryover_percentage(
-		lexical_cast_default<int>(level_["carryover_percentage"],
-		game_config::gold_carryover_percentage));
-	victory_conditions::set_carryover_add(utils::string_bool(
-		level_["carryover_add"], game_config::gold_carryover_add));
+	set_victory_when_enemies_defeated(
+		utils::string_bool(level_["victory_when_enemies_defeated"], true));
+	end_level_data &end_level = get_end_level_data();
+	end_level.carryover_percentage = lexical_cast_default<int>(
+		level_["carryover_percentage"], game_config::gold_carryover_percentage);
+	end_level.carryover_add = utils::string_bool(level_["carryover_add"]);
 
 	LOG_NG << "entering try... " << (SDL_GetTicks() - ticks_) << "\n";
 	try {
-		// Log before prestart events: they do weird things.
-		if (first_human_team_ != -1) {
-			log.start(gamestate_, teams_[first_human_team_], first_human_team_ + 1, units_,
-					  loading_game_ ? gamestate_.get_variable("turn_number") : "", status_.number_of_turns());
-		}
-
 		fire_prestart(!loading_game_);
 		init_gui();
 
@@ -298,51 +339,67 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 		// Initialize countdown clock.
 		std::vector<team>::iterator t;
 		for(t = teams_.begin(); t != teams_.end(); ++t) {
-			std::string countd_enabled = level_["mp_countdown"].c_str();
-			if (utils::string_bool(countd_enabled) && !loading_game_ ){
-				t->set_countdown_time(1000 * lexical_cast_default<int>(level_["mp_countdown_init_time"],0));
+			if (gamestate_.mp_settings().mp_countdown && !loading_game_ ){
+				t->set_countdown_time(1000 * gamestate_.mp_settings().mp_countdown_init_time);
 			}
 		}
 
 		// if we loaded a save file in linger mode, skip to it.
-		if (linger_)
+		if (linger_) {
+			//determine the bonus gold handling for this scenario
+			const config end_cfg = level_.child_or_empty("endlevel");
+			end_level.carryover_percentage = lexical_cast_default<int>(
+				level_["carryover_percentage"],
+				game_config::gold_carryover_percentage);
+			end_level.carryover_add = utils::string_bool(level_["carryover_add"]);
+			end_level.gold_bonus = utils::string_bool(end_cfg["bonus"], true);
+			end_level.carryover_report = false;
 			throw end_level_exception(SKIP_TO_LINGER);
+		}
 
 		// Avoid autosaving after loading, but still
 		// allow the first turn to have an autosave.
 		bool save = !loading_game_;
+		ai_testing::log_game_start();
 		for(; ; first_player_ = 1) {
 			play_turn(save);
 			save = true;
 		} //end for loop
 
-	} catch(game::load_game_exception&) {
+	} catch(game::load_game_exception& lge) {
 		// Loading a new game is effectively a quit.
-		log.quit(status_.turn());
-		throw;
-	} catch(end_level_exception& end_level) {
-		*end_level_result = end_level;
-		if(!end_level.custom_endlevel_music.empty()) {
-			switch(end_level.result) {
-			case DEFEAT:
+		//
+		if (lge.game != "") {
+			gamestate_ = game_state();
+		}
+
+		throw lge;
+	} catch (end_level_exception &end_level_exn) {
+		ai_testing::log_game_end();
+		LEVEL_RESULT end_level_result = end_level_exn.result;
+		if (!end_level.custom_endlevel_music.empty()) {
+			if (end_level_result == DEFEAT) {
 				set_defeat_music_list(end_level.custom_endlevel_music);
-				break;
-			default:
+			} else {
 				set_victory_music_list(end_level.custom_endlevel_music);
 			}
 		}
 
-		if(!team_manager_.get_teams().size())
+		if (teams_.empty())
 		{
+			//store persistent teams
+			gamestate_.snapshot = config();
+			store_recalls();
+
 			return VICTORY; // this is probably only a story scenario, i.e. has its endlevel in the prestart event
 		}
-		const bool obs = team_manager_.is_observer();
+		const bool obs = is_observer();
 		if (game_config::exit_at_end) {
 			exit(0);
 		}
-		if (end_level.result == DEFEAT || end_level.result == VICTORY) {
-			gamestate_.completion = (end_level.result == VICTORY) ? "victory" : "defeat";
-			recorder.set_save_info_completion(gamestate_.completion);
+		if (end_level_result == DEFEAT || end_level_result == VICTORY)
+		{
+			gamestate_.classification().completion = (end_level_exn.result == VICTORY) ? "victory" : "defeat";
 			// If we're a player, and the result is victory/defeat, then send
 			// a message to notify the server of the reason for the game ending.
 			if (!obs) {
@@ -350,26 +407,24 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 				config& info = cfg.add_child("info");
 				info["type"] = "termination";
 				info["condition"] = "game over";
-				info["result"] = gamestate_.completion;
+				info["result"] = gamestate_.classification().completion;
 				network::send_data(cfg, 0, true);
 			} else {
-				gui::message_dialog(*gui_,_("Game Over"),
-									_("The game is over.")).show();
+				gui2::show_transient_message(gui_->video(),_("Game Over"),
+									_("The game is over."));
 				return OBSERVER_END;
 			}
 		}
 
-		if(end_level.result == QUIT) {
-			log.quit(status_.turn());
-			return end_level.result;
-		} else if(end_level.result == DEFEAT) {
-			gamestate_.completion = "defeat";
-			log.defeat(status_.turn());
-			try {
-				game_events::fire("defeat");
-			} catch(end_level_exception&) {
-				ERR_NG << "[endlevel] used in 'defeat' event handler\n";
-			}
+		if (end_level_result == QUIT) {
+
+			return QUIT;
+		}
+		else if (end_level_result == DEFEAT)
+		{
+			gamestate_.classification().completion = "defeat";
+
+			game_events::fire("defeat");
 
 			if (!obs) {
 				const std::string& defeat_music = select_defeat_music();
@@ -380,16 +435,13 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			} else {
 				return QUIT;
 			}
-		} else if (end_level.result == VICTORY)
+		}
+		else if (end_level_result == VICTORY)
 		{
-			gamestate_.completion = (!end_level.linger_mode ?
-			                         "running" : "victory");
-			recorder.set_save_info_completion(gamestate_.completion);
-			try {
-				game_events::fire("victory");
-			} catch(end_level_exception&) {
-				ERR_NG << "[endlevel] used in 'victory' event handler\n";
-			}
+			gamestate_.classification().completion =
+				!end_level.linger_mode ? "running" : "victory";
+			game_events::fire("victory");
+
 			//
 			// Play victory music once all victory events
 			// are finished, if we aren't observers.
@@ -399,98 +451,40 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			// a victory, so let them use [music] tags
 			// instead should they want special music.
 			//
-			if(end_level.result == VICTORY && (!obs) && (end_level.linger_mode)) {
+			if (!obs && end_level.linger_mode) {
 				const std::string& victory_music = select_victory_music();
 				if(victory_music.empty() != true)
 					sound::play_music_once(victory_music);
 			}
-			if (first_human_team_ != -1)
-				log.victory(status_.turn(), teams_[first_human_team_].gold());
-
-			const bool has_next_scenario = !gamestate_.next_scenario.empty() &&
-											gamestate_.next_scenario != "null";
-
-			// Save current_player name to reuse it when setting next_scenario side info
-			std::vector<team>::iterator i;
-			for (i = teams_.begin(); i != teams_.end(); ++i) {
-				player_info *player=gamestate_.get_player(i->save_id());
-				if (player)
-					player->name = i->current_player();
-			}
 
 			// Add all the units that survived the scenario.
 			LOG_NG << "Add units that survived the scenario to the recall list.\n";
 			for(unit_map::iterator un = units_.begin(); un != units_.end(); ++un) {
-				player_info *player=gamestate_.get_player(teams_[un->second.side()-1].save_id());
 
-				if(player) {
+				if(teams_[un->second.side()-1].persistent()) {
 					LOG_NG << "Added unit " << un->second.id() << ", " << un->second.name() << "\n";
 					un->second.new_turn();
 					un->second.new_scenario();
-					player->available_units.push_back(un->second);
+					teams_[un->second.side()-1].recall_list().push_back(un->second);
 				}
 			}
-
-			std::stringstream report;
-			std::string title;
-
-			if (obs) {
-				title = _("Scenario Report");
-			} else {
-				title = _("Victory");
-				report << font::BOLD_TEXT << _("You have emerged victorious!") << "\n~\n";
-			}
-			if (gamestate_.players.size() > 0 &&
-					 (has_next_scenario ||
-					 gamestate_.campaign_type == "test")) {
-				const int finishing_bonus_per_turn =
-						 map_.villages().size() * game_config::village_income +
-						 game_config::base_income;
-				const int turns_left = std::max<int>(0,status_.number_of_turns() - status_.turn());
-				const int finishing_bonus = (end_level.gold_bonus && (turns_left > -1)) ?
-						 (finishing_bonus_per_turn * turns_left) : 0;
-
-				for(i=teams_.begin(); i!=teams_.end(); ++i) {
-					player_info *player=gamestate_.get_player(i->save_id());
-
-					if (player) {
-						// Store the gold for all players.
-						player->gold = ((i->gold() + finishing_bonus)
-								* end_level.carryover_percentage) / 100;
-						player->gold_add = end_level.carryover_add;
-
-						// Only show the report for ourselves.
-						if (!i->is_local())
-							continue;
-
-						if(gamestate_.players.size()>1) {
-							if(i!=teams_.begin()) {
-								report << "\n";
-							}
-
-							report << font::BOLD_TEXT << i->current_player() << "\n";
-						}
-
-						report_victory(report, end_level, player->gold, i->gold(), finishing_bonus_per_turn, turns_left, finishing_bonus);
-					}
-				}
-			}
-
-			if(end_level.carryover_report)
-			{
-				gui::message_dialog(*gui_, title, report.str()).show();
-			}
-
+			//store all units that survived (recall list for the next scenario) in snapshot
+			gamestate_.snapshot = config();
+			store_recalls();
+			//store gold and report victory
+			store_gold(obs);
 			return VICTORY;
-		} else if (end_level.result == SKIP_TO_LINGER) {
+		}
+		else if (end_level_result == SKIP_TO_LINGER)
+		{
 			LOG_NG << "resuming from loaded linger state...\n";
+			//as carryover information is stored in the snapshot, we have to re-store it after loading a linger state
+			gamestate_.snapshot = config();
+			store_recalls();
+			store_gold();
 			return VICTORY;
 		}
 	} // end catch
-	catch(replay::error&) {
-		gui::message_dialog(*gui_,"",_("The file you have tried to load is corrupt")).show();
-		return QUIT;
-	}
 	catch(network::error& e) {
 		bool disconnect = false;
 		if(e.socket) {
@@ -498,7 +492,9 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			disconnect = true;
 		}
 
-		menu_handler_.save_game(_("A network disconnection has occurred, and the game cannot continue. Do you want to save the game?"),gui::YES_NO);
+		savegame::game_savegame save(gamestate_, *gui_, to_config(), preferences::compress_saves());
+		/** @todo FIXME: remove the odd linebreak after the string freeze */
+		save.save_game_interactive(gui_->video(), _("A network disconnection has occurred, and the game\ncannot continue. Do you want to save the game?"), gui::YES_NO);
 		if(disconnect) {
 			throw network::error();
 		} else {
@@ -515,25 +511,22 @@ void playsingle_controller::play_turn(bool save)
 	gui_->invalidate_game_status();
 	events::raise_draw_event();
 
-	LOG_NG << "turn: " << status_.turn() << "\n";
+	LOG_NG << "turn: " << turn() << "\n";
 
 	if(non_interactive())
-		std::cout << "Turn " << status_.turn() << ":" << std::endl;
+		std::cout << "Turn " << turn() << ":" << std::endl;
 
 
-	for(player_number_ = first_player_; player_number_ <= teams_.size(); player_number_++) {
+	for (player_number_ = first_player_; player_number_ <= int(teams_.size()); ++player_number_)
+	{
 		// If a side is empty skip over it.
 		if (current_team().is_empty()) continue;
 		try {
-			if (skip_next_turn_) {
-				skip_next_turn_ = false;
-				throw end_turn_exception();
-			}
+			save_blocker blocker;
 			init_side(player_number_ - 1);
 		} catch (end_turn_exception) {
 			if (current_team().is_network() == false) {
-				turn_info turn_data(gamestate_, status_, *gui_, map_,
-						teams_, player_number_, units_, replay_sender_, undo_stack_);
+				turn_info turn_data(player_number_, replay_sender_, undo_stack_);
 				recorder.end_turn();
 				turn_data.sync_network();
 			}
@@ -542,25 +535,18 @@ void playsingle_controller::play_turn(bool save)
 
 		if (replaying_) {
 			LOG_NG << "doing replay " << player_number_ << "\n";
-			try {
-				replaying_ = ::do_replay(*gui_, map_, units_, teams_,
-						player_number_, status_, gamestate_);
-			} catch(replay::error&) {
-				gui::message_dialog(*gui_,"",_("The file you have tried to load is corrupt")).show();
-
-				replaying_ = false;
-			}
+			replaying_ = ::do_replay(player_number_);
 			LOG_NG << "result of replay: " << (replaying_?"true":"false") << "\n";
 		} else {
 			// If a side is dead end the turn.
-			if ((current_team().is_human() && team_units(units_, player_number_) == 0))
+			if (current_team().is_human() && side_units(units_, player_number_) == 0)
 			{
-				turn_info turn_data(gamestate_, status_, *gui_, map_,
-						teams_, player_number_, units_, replay_sender_, undo_stack_);
+				turn_info turn_data(player_number_, replay_sender_, undo_stack_);
 				recorder.end_turn();
 				turn_data.sync_network();
 				continue;
 			}
+			ai_testing::log_turn_start(player_number_);
 			play_side(player_number_, save);
 		}
 
@@ -570,20 +556,19 @@ void playsingle_controller::play_turn(bool save)
 			std::cout << " Player " << player_number_ << ": " <<
 				current_team().villages().size() << " Villages" <<
 				std::endl;
+			ai_testing::log_turn_end(player_number_);
 		}
 
-		check_victory(units_, teams_, *gui_);
+		check_victory();
+
+		//if loading a savegame, network turns might not have reset this yet
+		loading_game_ = false;
 	}
 
 	// Time has run out
 	check_time_over();
 
-	try {
-		finish_turn();
-	} catch (end_turn_exception) {
-		// If [end_turn] is encountered, skip the first existing side's turn.
-		skip_next_turn_ = true;
-	}
+	finish_turn();
 }
 
 void playsingle_controller::play_side(const unsigned int team_index, bool save)
@@ -598,7 +583,8 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 		// it has to be a class member since derived classes
 		// rely on it
 		player_type_changed_ = false;
-		end_turn_ = false;
+		if (!skip_next_turn_)
+			end_turn_ = false;
 
 
 		statistics::reset_turn_stats(teams_[team_index - 1].save_id());
@@ -638,15 +624,12 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 				//give control to human for the rest of this turn
 				player_type_changed_ = true;
 				temporary_human = true;
-			} catch(end_turn_exception& end_turn) {
-				if (end_turn.redo == team_index) {
-					player_type_changed_ = true;
-				}
 			}
 		}
 	} while (player_type_changed_);
 	// Keep looping if the type of a team (human/ai/networked)
 	// has changed mid-turn
+	skip_next_turn_ = false;
 }
 
 void playsingle_controller::before_human_turn(bool save)
@@ -660,8 +643,11 @@ void playsingle_controller::before_human_turn(bool save)
 	gui_->invalidate_all();
 	gui_->draw(true,true);
 
+	ai::manager::raise_turn_started();
+
 	if (save) {
-		menu_handler_.autosave(gamestate_.label, status_.turn(), gamestate_.starting_pos);
+		savegame::autosave_savegame save(gamestate_, *gui_, to_config(), preferences::compress_saves());
+		save.autosave(game_config::disable_autosave, preferences::autosavemax(), preferences::INFINITE_AUTO_SAVES);
 	}
 
 	if(preferences::turn_bell()) {
@@ -670,34 +656,17 @@ void playsingle_controller::before_human_turn(bool save)
 }
 
 void playsingle_controller::show_turn_dialog(){
-	if(preferences::turn_dialog()) {
+	if(preferences::turn_dialog() && (level_result_ == NONE) ) {
 		std::string message = _("It is now $name|'s turn");
 		utils::string_map symbols;
 		symbols["name"] = teams_[player_number_ - 1].current_player();
 		message = utils::interpolate_variables_into_string(message, &symbols);
-		gui::message_dialog(*gui_, "", message).show();
+		gui2::show_transient_message(gui_->video(), "", message);
 	}
 }
 
 void playsingle_controller::execute_gotos(){
-	// Execute goto-movements - first collect gotos in a list
-	std::vector<map_location> gotos;
-
-	for(unit_map::iterator ui = units_.begin(); ui != units_.end(); ++ui) {
-		if(ui->second.get_goto() == ui->first)
-			ui->second.set_goto(map_location());
-
-		if(ui->second.side() == player_number_ && map_.on_board(ui->second.get_goto()))
-			gotos.push_back(ui->first);
-	}
-
-	for(std::vector<map_location>::const_iterator g = gotos.begin(); g != gotos.end(); ++g) {
-		unit_map::const_iterator ui = units_.find(*g);
-		menu_handler_.move_unit_to_loc(ui,ui->second.get_goto(),false, player_number_, mouse_handler_);
-	}
-
-	// erase the footsteps after movement
-	gui_->set_route(NULL);
+	menu_handler_.execute_gotos(mouse_handler_, player_number_);
 }
 
 void playsingle_controller::play_human_turn() {
@@ -707,6 +676,7 @@ void playsingle_controller::play_human_turn() {
 	gui_->enable_menu("endturn", true);
 	while(!end_turn_) {
 		play_slice();
+		check_end_level();
 		gui_->draw();
 	}
 }
@@ -718,14 +688,14 @@ struct set_completion
 	}
 	~set_completion()
 	{
-		state_.completion = completion_;
+		state_.classification().completion = completion_;
 	}
 	private:
 	game_state& state_;
 	const std::string completion_;
 };
 
-void playsingle_controller::linger(upload_log& log)
+void playsingle_controller::linger()
 {
 	LOG_NG << "beginning end-of-scenario linger\n";
 	browse_ = true;
@@ -741,12 +711,12 @@ void playsingle_controller::linger(upload_log& log)
 	set_completion setter(gamestate_,"running");
 
 	// change the end-turn button text to its alternate label
-	gui_->get_theme().refresh_title2(std::string("button-endturn"), std::string("title2"));
+	gui_->get_theme().refresh_title2("button-endturn", "title2");
 	gui_->invalidate_theme();
 	gui_->redraw_everything();
 
 	// End all unit moves
-	for (unit_map::iterator u = units_.begin(); u != units_.end(); u++) {
+	for (unit_map::iterator u = units_.begin(); u != units_.end(); ++u) {
 		u->second.set_user_end_turn(true);
 	}
 	try {
@@ -757,17 +727,19 @@ void playsingle_controller::linger(upload_log& log)
 			// Reset the team number to make sure we're the right team.
 			player_number_ = first_player_;
 			play_slice();
-
 			gui_->draw();
 		}
-	} catch(game::load_game_exception&) {
+	} catch(game::load_game_exception& lge) {
 		// Loading a new game is effectively a quit.
-		log.quit(status_.turn());
-		throw;
+		if (lge.game != "") {
+			gamestate_ = game_state();
+		}
+
+		throw lge;
 	}
 
 	// revert the end-turn button text to its normal label
-	gui_->get_theme().refresh_title2(std::string("button-endturn"), std::string("title"));
+	gui_->get_theme().refresh_title2("button-endturn", "title");
 	gui_->invalidate_theme();
 	gui_->redraw_everything();
 	gui_->set_game_mode(game_display::RUNNING);
@@ -796,7 +768,7 @@ void playsingle_controller::after_human_turn(){
 
 	if(teams_[player_number_-1].uses_fog()) {
 		// needed because currently fog is only recalculated when a hex is /un/covered
-		recalculate_fog(map_,units_,teams_,player_number_-1);
+		recalculate_fog(player_number_);
 	}
 
 	gui_->set_route(NULL);
@@ -811,28 +783,17 @@ void playsingle_controller::play_ai_turn(){
 
 	const cursor::setter cursor_setter(cursor::WAIT);
 
-	turn_info turn_data(gamestate_,status_,*gui_,
-			map_, teams_, player_number_, units_, replay_sender_, undo_stack_);
+	turn_info turn_data(player_number_, replay_sender_, undo_stack_);
 
-	ai_interface::info ai_info(*gui_,map_,units_,teams_,player_number_,status_, turn_data, gamestate_);
-	std::string ai_algorithm = current_team().ai_algorithm();
-
-	boost::intrusive_ptr<ai_interface> ai_obj ;
-	ai_obj = ai_manager::get_ai( ai_algorithm, ai_info ) ;
-
-	ai_obj->user_interact().attach_handler(this);
-	ai_obj->unit_recruited().attach_handler(this);
-	ai_obj->unit_moved().attach_handler(this);
-	ai_obj->enemy_attacked().attach_handler(this);
 	try {
-		ai_obj->play_turn();
-	} catch (end_turn_exception) {
+		ai::manager::play_turn(player_number_);
+	} catch (end_turn_exception&) {
 	}
 	recorder.end_turn();
 	turn_data.sync_network();
 
 	gui_->recalculate_minimap();
-	::clear_shroud(*gui_,map_,units_,teams_,player_number_-1);
+	::clear_shroud(player_number_);
 	gui_->invalidate_unit();
 	gui_->invalidate_game_status();
 	gui_->invalidate_all();
@@ -842,30 +803,152 @@ void playsingle_controller::play_ai_turn(){
 
 void playsingle_controller::handle_generic_event(const std::string& name){
 	if (name == "ai_user_interact"){
-		play_slice();
-		gui_->draw();
+		play_slice(false);
+	}
+	if (end_turn_){
+		throw end_turn_exception();
 	}
 }
 
 void playsingle_controller::check_time_over(){
-	if(!status_.next_turn()) {
-
-		if(non_interactive()) {
-			std::cout << "time over (draw)\n";
-		}
+	bool b = next_turn();
+	if(!b) {
 
 		LOG_NG << "firing time over event...\n";
 		game_events::fire("time over");
 		LOG_NG << "done firing time over event...\n";
+		//if turns are added while handling 'time over' event
+		if (tod_manager_.is_time_left()) {
+			return;
+		}
 
+		if(non_interactive()) {
+			std::cout << "time over (draw)\n";
+			ai_testing::log_draw();
+		}
+
+		check_end_level();
 		throw end_level_exception(DEFEAT);
 	}
 }
 
+void playsingle_controller::store_recalls() {
+	std::set<std::string> side_ids;
+	std::vector<team>::iterator i;
+	for(i=teams_.begin(); i!=teams_.end(); ++i) {
+		side_ids.insert(i->save_id());
+		if (i->persistent()) {
+			config& new_side = gamestate_.snapshot.add_child("side");
+			new_side["save_id"] = i->save_id();
+			new_side["name"] = i->current_player();
+			std::stringstream can_recruit;
+			std::copy(i->recruits().begin(),i->recruits().end(),std::ostream_iterator<std::string>(can_recruit,","));
+			std::string can_recruit_str = can_recruit.str();
+			// Remove the trailing comma
+			if(can_recruit_str.empty() == false) {
+				can_recruit_str.resize(can_recruit_str.size()-1);
+			}
+			new_side["previous_recruits"] = can_recruit_str;
+			LOG_NG << "stored side in snapshot:\n" << new_side["save_id"] << std::endl;
+			//add the units of the recall list
+			foreach(const unit& u, i->recall_list()) {
+				config& new_unit = new_side.add_child("unit");
+				u.write(new_unit);
+			}
+		}
+	}
+	//add any players from starting_pos that do not have a team in the current scenario
+	foreach (const config* player_cfg, gamestate_.starting_pos.get_children("player")) {
+		if (side_ids.count((*player_cfg)["save_id"]) == 0) {
+			LOG_NG << "stored inactive side in snapshot:\n" << (*player_cfg)["save_id"] << std::endl;
+			gamestate_.snapshot.add_child("side", (*player_cfg));
+		}
+	}
+}
+
+void playsingle_controller::store_gold(bool obs)
+{
+	bool has_next_scenario = !gamestate_.classification().next_scenario.empty() &&
+		gamestate_.classification().next_scenario != "null";
+
+	std::ostringstream report;
+	std::string title;
+
+	if (obs) {
+		title = _("Scenario Report");
+	} else {
+		title = _("Victory");
+		report << "<b>" << _("You have emerged victorious!") << "</b>\n\n";
+	}
+
+	int persistent_teams = 0;
+	foreach (const team &t, teams_) {
+		if (t.persistent()) ++persistent_teams;
+	}
+
+	end_level_data &end_level = get_end_level_data();
+
+	if (persistent_teams > 0 && (has_next_scenario ||
+		gamestate_.classification().campaign_type == "test"))
+	{
+		int finishing_bonus_per_turn =
+			map_.villages().size() * game_config::village_income +
+			game_config::base_income;
+		int turns_left = std::max<int>(0, number_of_turns() - turn());
+		int finishing_bonus = (end_level.gold_bonus && turns_left > -1) ?
+			finishing_bonus_per_turn * turns_left : 0;
+		foreach (const team &t, teams_)
+		{
+			if (!t.persistent()) continue;
+			int carryover_gold = ((t.gold() + finishing_bonus) * end_level.carryover_percentage) / 100;
+			config::child_itors side_range = gamestate_.snapshot.child_range("side");
+			config::child_iterator side_it = side_range.first;
+
+			// Check if this side already exists in the snapshot.
+			while (side_it != side_range.second) {
+				if ((*side_it)["save_id"] == t.save_id()) {
+					(*side_it)["gold"] = str_cast<int>(carryover_gold);
+					(*side_it)["gold_add"] = end_level.carryover_add ? "yes" : "no";
+					(*side_it)["colour"] = t.colour();
+					(*side_it)["current_player"] = t.current_player();
+					(*side_it)["name"] = t.name();
+					break;
+				}
+				++side_it;
+			}
+
+			// If it doesn't, add a new child.
+			if (side_it == side_range.second) {
+				config &new_side = gamestate_.snapshot.add_child("side");
+				new_side["save_id"] = t.save_id();
+				new_side["gold"] = str_cast<int>(carryover_gold);
+				new_side["gold_add"] = end_level.carryover_add ? "yes" : "no";
+				new_side["colour"] = t.colour();
+				new_side["current_player"] = t.current_player();
+				new_side["name"] = t.name();
+			}
+
+			// Only show the report for ourselves.
+			if (!t.is_human()) continue;
+
+			if (persistent_teams > 1) {
+				report << "\n<b>" << t.current_player() << "</b>\n";
+			}
+
+			report_victory(report, carryover_gold, t.gold(), finishing_bonus_per_turn, turns_left, finishing_bonus);
+		}
+	}
+
+	if (end_level.carryover_report) {
+		gui2::show_transient_message(gui_->video(), title, report.str(), true);
+	}
+}
+
 bool playsingle_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int index) const
 {
 	bool res = true;
 	switch (command){
+		case hotkey::HOTKEY_ADD_WAYPOINT:
 		case hotkey::HOTKEY_UNIT_HOLD_POSITION:
 		case hotkey::HOTKEY_END_UNIT_TURN:
 		case hotkey::HOTKEY_RECRUIT:
@@ -876,14 +959,17 @@ bool playsingle_controller::can_execute_command(hotkey::HOTKEY_COMMAND command,
 			return (!browse_ || linger_) && !events::commands_disabled;
 
 		case hotkey::HOTKEY_DELAY_SHROUD:
-			return !linger_ && (current_team().uses_fog() || current_team().uses_shroud())
+			return !linger_ && (teams_[gui_->viewing_team()].uses_fog() || teams_[gui_->viewing_team()].uses_shroud())
 			&& !events::commands_disabled;
 		case hotkey::HOTKEY_UPDATE_SHROUD:
-			return !linger_ && !events::commands_disabled && current_team().auto_shroud_updates() == false;
+			return !linger_
+				&& player_number_ == gui_->viewing_side()
+				&& !events::commands_disabled
+				&& teams_[gui_->viewing_team()].auto_shroud_updates() == false;
 
 		// Commands we can only do if in debug mode
 		case hotkey::HOTKEY_CREATE_UNIT:
-		case hotkey::HOTKEY_CHANGE_UNIT_SIDE:
+		case hotkey::HOTKEY_CHANGE_SIDE:
 			return !events::commands_disabled && game_config::debug && map_.on_board(mouse_handler_.get_last_hex());
 
 		case hotkey::HOTKEY_LABEL_TEAM_TERRAIN:
