@@ -1,6 +1,6 @@
-/* $Id: playturn.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: playturn.cpp 49405 2011-05-07 08:08:15Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "map_label.hpp"
 #include "replay.hpp"
 #include "resources.hpp"
 #include "rng.hpp"
@@ -155,7 +156,6 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 			if (!player.empty() && leader.valid())
 				leader->second.rename(player);
 
-
 			if (controller == "human" && !tm.is_human()) {
 				if (!(*resources::teams)[resources::screen->get_playing_team()].is_human())
 				{
@@ -176,6 +176,8 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 				restart = false;
 			}
 
+			resources::screen->labels().recalculate_labels();
+
 			return restart ? PROCESS_RESTART_TURN : PROCESS_CONTINUE;
 		}
 	}
