@@ -1,6 +1,6 @@
-/* $Id: portrait.cpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: portrait.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2008 - 2009 by mark de wever <koraq@xs4all.nl>
+   Copyright (C) 2008 - 2011 by mark de wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,9 +16,12 @@
 
 #include "config.hpp"
 #include "log.hpp"
+#include "serialization/string_utils.hpp"
+#include "util.hpp"
 #include "wml_exception.hpp"
 
-#define WRN_CF LOG_STREAM(warn, config)
+static lg::log_domain log_config("config");
+#define WRN_CF LOG_STREAM(warn, log_config)
 
 static tportrait::tside get_side(const std::string& side)
 {
