@@ -1,6 +1,6 @@
-/* $Id: replay.hpp 33371 2009-03-06 22:40:59Z alink $ */
+/* $Id: replay.hpp 49342 2011-05-02 18:52:36Z crab $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -21,19 +21,28 @@
 #define REPLAY_H_INCLUDED
 
 #include "config.hpp"
-#include "gamestatus.hpp"
 #include "map_location.hpp"
-#include "random.hpp"
+#include "rng.hpp"
+
+#include <deque>
 
-class config_writer;
 class game_display;
 class terrain_label;
 class unit_map;
+class play_controller;
+struct time_of_day;
 
-struct verification_manager
-{
-	verification_manager(const unit_map& units);
-	~verification_manager();
+class chat_msg {
+public:
+	const std::string &text() const { return text_; }
+	const std::string &nick() const { return nick_; }
+	const std::string &color() const { return color_; }
+	chat_msg(const config &cfg);
+	virtual ~chat_msg();
+private:
+	std::string color_;
+	std::string nick_;
+	std::string text_;
 };
 
 class replay: public rand_rng::rng
@@ -42,35 +51,53 @@ public:
 	replay();
 	explicit replay(const config& cfg);
 
-	void set_save_info(const game_state& save);
-	void set_save_info(const game_state& save, const config::child_list& players);
-	void set_save_info_completion(const std::string &st);
+	void append(const config& cfg);
 
 	void set_skip(bool skip);
 	bool is_skipping() const;
 
-	void save_game(const std::string& label, const config& snapshot,
-	               const config& starting_pos, bool include_replay = true);
-
 	void add_start();
 	void add_recruit(int unit_index, const map_location& loc);
-	void add_recall(int unit_index, const map_location& loc);
-	void add_disband(int unit_index);
+	void add_recall(const std::string& unit_id, const map_location& loc);
+	void add_disband(const std::string& unit_id);
 	void add_countdown_update(int value,int team);
 	void add_movement(const std::vector<map_location>& steps);
 	void add_attack(const map_location& a, const map_location& b,
-	                int att_weapon, int def_weapon);
+		int att_weapon, int def_weapon, const std::string& attacker_type_id,
+		const std::string& defender_type_id, int attacker_lvl,
+		int defender_lvl, const size_t turn, const time_of_day &t);
+	void add_seed(const char* child_name, rand_rng::seed_t seed);
 	void choose_option(int index);
 	void text_input(std::string input);
 	void set_random_value(const std::string& choice);
 	void add_label(const terrain_label*);
 	void clear_labels(const std::string&);
 	void add_rename(const std::string& name, const map_location& loc);
+	void init_side();
 	void end_turn();
 	void add_event(const std::string& name,
 		const map_location& loc=map_location::null_location);
 	void add_unit_checksum(const map_location& loc,config* const cfg);
 	void add_checksum_check(const map_location& loc);
+	void add_log_data(const std::string &key, const std::string &var);
+	void add_log_data(const std::string &category, const std::string &key, const std::string &var);
+	void add_log_data(const std::string &category, const std::string &key, const config& c);
+
+	/**
+	 * Mark an expected advancement adding it to the queue
+	 */
+	void add_expected_advancement(const map_location& loc);
+
+	/**
+	 * Access to the expected advancements queue.
+	 */
+	const std::deque<map_location>& expected_advancements() const;
+
+	/**
+	 * Remove the front expected advancement from the queue
+	 */
+	void pop_expected_advancement();
+
 	/**
 	 * Adds an advancement to the replay, the following option command
 	 * determines which advancement option has been choosen
@@ -79,7 +106,7 @@ public:
 
 	void add_chat_message_location();
 	void speak(const config& cfg);
-	std::string build_chat_log();
+	const std::vector<chat_msg>& build_chat_log();
 
 	//get data range will get a range of moves from the replay system.
 	//if data_type is 'ALL_DATA' then it will return all data in this range
@@ -91,6 +118,7 @@ public:
 	enum DATA_TYPE { ALL_DATA, NON_UNDO_DATA };
 	config get_data_range(int cmd_start, int cmd_end, DATA_TYPE data_type=ALL_DATA);
 	config get_last_turn(int num_turns=1);
+	const config& get_replay_data() const { return cfg_; }
 
 	void undo();
 
@@ -110,14 +138,8 @@ public:
 
 	int ncommands();
 
-	static void throw_error(const std::string& msg);
+	static void process_error(const std::string& msg);
 
-	struct error {
-		error(const std::string& msg) : message(msg) {}
-		std::string message;
-	};
-
-	static std::string last_replay_error;
 private:
 	//generic for add_movement and add_attack
 	void add_pos(const std::string& type,
@@ -125,7 +147,7 @@ private:
 
 	void add_value(const std::string& type, int value);
 
-	void add_chat_log_entry(const config* speak, std::stringstream& str) const;
+	void add_chat_log_entry(const config &speak, std::back_insert_iterator< std::vector<chat_msg> > &i) const;
 
 	const config::child_list& commands() const;
 	void remove_command(int);
@@ -146,11 +168,15 @@ private:
 
 	config* current_;
 
-	game_state saveInfo_;
-
 	bool skip_;
 
 	std::vector<int> message_locations;
+
+	/**
+	 * A queue of units (locations) that are supposed to advance but the
+	 * relevant advance (choice) message has not yet been received
+	 */
+	std::deque<map_location> expected_advancements_;
 };
 
 replay& get_replay_source();
@@ -159,14 +185,9 @@ extern replay recorder;
 
 //replays up to one turn from the recorder object
 //returns true if it got to the end of the turn without data running out
-bool do_replay(game_display& disp, const gamemap& map,
-	unit_map& units, std::vector<team>& teams, int team_num,
-	const gamestatus& state, game_state& state_of_game, replay* obj=NULL);
+bool do_replay(int side_num, replay *obj = NULL);
 
-bool do_replay_handle(game_display& disp, const gamemap& map,
-					  unit_map& units, std::vector<team>& teams, int team_num,
-	   const gamestatus& state, game_state& state_of_game,
-	const std::string& do_untill);
+bool do_replay_handle(int side_num, const std::string &do_untill);
 
 //an object which can be made to undo a recorded move
 //unless the transaction is confirmed
