@@ -1,6 +1,6 @@
-/* $Id: replay_controller.cpp 36212 2009-06-14 21:59:53Z soliton $ */
+/* $Id: replay_controller.cpp 48834 2011-03-10 18:07:57Z dragonking $ */
 /*
-   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2011 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -15,17 +15,21 @@
 
 #include "global.hpp"
 
-#include "config_adapter.hpp"
 #include "game_end_exceptions.hpp"
+#include "game_events.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "replay.hpp"
 #include "replay_controller.hpp"
+#include "resources.hpp"
+#include "savegame.hpp"
 
+static lg::log_domain log_engine("engine");
+#define DBG_NG LOG_STREAM(debug, log_engine)
 
-#define DBG_NG LOG_STREAM(debug, engine)
-#define DBG_REPLAY LOG_STREAM(debug, replay)
-#define LOG_REPLAY LOG_STREAM(info, replay)
-#define ERR_REPLAY LOG_STREAM(err, replay)
+static lg::log_domain log_replay("replay");
+#define DBG_REPLAY LOG_STREAM(debug, log_replay)
+#define LOG_REPLAY LOG_STREAM(info, log_replay)
 
 LEVEL_RESULT play_replay_level(const config& game_config,
 		const config* level, CVideo& video, game_state& state_of_game)
@@ -45,7 +49,6 @@ LEVEL_RESULT play_replay_level(const config& game_config,
 	}
 	catch(end_level_exception&){
 		DBG_NG << "play_replay_level: end_level_exception\n";
-	} catch (replay::error&) {
 	}
 
 	return VICTORY;
@@ -57,29 +60,26 @@ replay_controller::replay_controller(const config& level,
 	play_controller(level, state_of_game, ticks, num_turns, game_config, video, false),
 	teams_start_(),
 	gamestate_start_(state_of_game),
-	status_start_(level, num_turns, &state_of_game),
 	units_start_(),
+	tod_manager_start_(level, num_turns, &state_of_game),
 	current_turn_(1),
 	delay_(0),
 	is_playing_(false),
 	show_everything_(false),
-	show_team_(state_of_game.campaign_type == "multiplayer" ? 0 : 1),
-	skip_next_turn_(false)
+	show_team_(state_of_game.classification().campaign_type == "multiplayer" ? 0 : 1),
+	buttons_()
 {
 	init();
 	gamestate_start_ = gamestate_;
 }
 
-replay_controller::~replay_controller(){
+replay_controller::~replay_controller()
+{
 	//YogiHH
 	//not absolutely sure if this is needed, but it makes me feel a lot better ;-)
 	//feel free to delete this if it is not necessary
-	gui_->get_theme().theme_reset().detach_handler(this);
-}
-
-bool replay_controller::continue_replay() {
-	return gui::dialog(*gui_,"",_("The file you have tried to load is corrupt."
-			" Continue playing?"),gui::OK_CANCEL).show();
+	gui_->get_theme().theme_reset_event().detach_handler(this);
+	gui_->complete_redraw_event().detach_handler(this);
 }
 
 void replay_controller::init(){
@@ -89,28 +89,15 @@ void replay_controller::init(){
 	const cursor::setter cursor_setter(cursor::NORMAL);
 	init_replay_display();
 
-	try {
-		fire_prestart(true);
-	} catch (replay::error&) {
-		if(!continue_replay()) {
-			throw;
-		}
-	}
+	fire_prestart(true);
 	init_gui();
 	statistics::fresh_stats();
-	victory_conditions::set_victory_when_enemies_defeated(
-		utils::string_bool(level_["victory_when_enemies_defeated"], true)
-	);
+	set_victory_when_enemies_defeated(
+		utils::string_bool(level_["victory_when_enemies_defeated"], true));
 
 	DBG_REPLAY << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
 
-	try {
-		fire_start(!loading_game_);
-	} catch (replay::error&) {
-		if(!continue_replay()) {
-			throw;
-		}
-	}
+	fire_start(!loading_game_);
 	update_gui();
 
 	units_start_ = units_;
@@ -131,40 +118,148 @@ void replay_controller::init_gui(){
 	for(std::vector<team>::iterator t = teams_.begin(); t != teams_.end(); ++t) {
 		t->reset_objectives_changed();
 	}
+	
+	buttons_.update(gui_);
 }
 
 void replay_controller::init_replay_display(){
 	DBG_REPLAY << "initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
 
 	rebuild_replay_theme();
-	gui_->get_theme().theme_reset().attach_handler(this);
+	gui_->get_theme().theme_reset_event().attach_handler(this);
+	gui_->complete_redraw_event().attach_handler(this);
 	DBG_REPLAY << "done initializing replay-display... " << (SDL_GetTicks() - ticks_) << "\n";
 }
 
-void replay_controller::rebuild_replay_theme(){
-	const config* theme_cfg = get_theme(game_config_, level_["theme"]);
-	if (theme_cfg) {
-		const config* replay_theme_cfg = theme_cfg->child("resolution")->child("replay");
-		if (NULL != replay_theme_cfg)
+void replay_controller::rebuild_replay_theme()
+{
+	const config &theme_cfg = get_theme(game_config_, level_["theme"]);
+	if (const config &res = theme_cfg.child("resolution"))
+	{
+		if (const config &replay_theme_cfg = res.child("replay"))
 			gui_->get_theme().modify(replay_theme_cfg);
+		gui_->get_theme().modify_label("time-icon", _ ("current local time"));
 		//Make sure we get notified if the theme is redrawn completely. That way we have
 		//a chance to restore the replay controls of the theme as well.
 		gui_->invalidate_theme();
 	}
 }
 
+
+void replay_controller::replay_buttons_wrapper::update(boost::scoped_ptr<game_display>& gui_)
+{
+ 	play_button_ = gui_->find_button("button-playreplay");
+	stop_button_ = gui_->find_button("button-stopreplay");
+	reset_button_ = gui_->find_button("button-resetreplay");
+	play_turn_button_ = gui_->find_button("button-nextturn");
+	play_side_button_ = gui_->find_button("button-nextside");
+	
+	//check if we have all buttons - if someone messed with theme then some buttons may be missing
+	//if any of the buttons is missing, we just disable everything
+	if( !play_button_ || !stop_button_ || !reset_button_ || !play_turn_button_ || !play_side_button_ ) {
+	 
+		is_valid_ = false;
+		enabled_buttons_ = 0;
+	} else {
+		is_valid_ = true;
+		if( enabled_buttons_ == 0)
+			enabled_buttons_ = PLAY_BUTTON_ENABLED | PLAY_TURN_BUTTON_ENABLED | PLAY_SIDE_BUTTON_ENABLED;
+	}
+	
+	update_buttons_states();
+}
+
+void replay_controller::replay_buttons_wrapper::update_buttons_states()
+{
+	if( enabled_buttons_ & PLAY_BUTTON_ENABLED ) {
+		play_button_->enable(true);
+	} else
+		play_button_->enable(false);
+	
+	if( enabled_buttons_ & STOP_BUTTON_ENABLED ) {
+		stop_button_->enable(true);
+	} else
+		stop_button_->enable(false);
+	
+	if( enabled_buttons_ & RESET_BUTTON_ENABLED ) {
+		reset_button_->enable(true);
+	} else
+		reset_button_->enable(false);		
+	
+	if( enabled_buttons_ & PLAY_TURN_BUTTON_ENABLED ) {
+		play_turn_button_->enable(true);
+	} else
+		play_turn_button_->enable(false);
+	
+	if( enabled_buttons_ & PLAY_SIDE_BUTTON_ENABLED ) {
+		play_side_button_->enable(true);
+	} else
+		play_side_button_->enable(false);	
+}
+
+void replay_controller::replay_buttons_wrapper::playback_should_start() 
+{
+	if( !is_valid_ )
+		return;
+	
+	play_button_->enable(false);
+	stop_button_->enable(true);
+	reset_button_->enable(false);
+	play_turn_button_->enable(false);
+	play_side_button_->enable(false);
+	
+	enabled_buttons_ = STOP_BUTTON_ENABLED;
+}
+
+void replay_controller::replay_buttons_wrapper::playback_should_stop(bool is_playing) 
+{
+	if( !is_valid_)
+		return;
+	
+	if( !recorder.at_end() ) {
+		
+		enabled_buttons_ = PLAY_BUTTON_ENABLED | RESET_BUTTON_ENABLED | PLAY_TURN_BUTTON_ENABLED | PLAY_SIDE_BUTTON_ENABLED;	
+		
+		update_buttons_states();
+	
+		play_button_->release();
+		play_turn_button_->release();
+		play_side_button_->release();
+	} else {
+		enabled_buttons_ = RESET_BUTTON_ENABLED;
+		
+		update_buttons_states();
+	}
+	
+	if( !is_playing ) {
+		//user interrupted
+		stop_button_->release();
+	}	
+}
+
+void replay_controller::replay_buttons_wrapper::reset_buttons() 
+{
+	if( !is_valid_ )
+		return;
+	
+	enabled_buttons_ = PLAY_BUTTON_ENABLED | PLAY_TURN_BUTTON_ENABLED | PLAY_SIDE_BUTTON_ENABLED;
+	
+	stop_button_->release();
+	reset_button_->release();
+	
+	update_buttons_states();
+}
+
 void replay_controller::reset_replay(){
-	gui::button* b = gui_->find_button("button-playreplay");
-	if (b != NULL) { b->release(); }
-	b = gui_->find_button("button-stopreplay");
-	if (b != NULL) { b->release(); }
 	gui_->clear_chat_messages();
 	is_playing_ = false;
 	player_number_ = 1;
 	current_turn_ = 1;
+	skip_replay_ = false;
+	tod_manager_= tod_manager_start_;
 	recorder.start_replay();
+	recorder.set_skip(false);
 	units_ = units_start_;
-	status_ = status_start_;
 	gamestate_ = gamestate_start_;
 	teams_ = teams_start_;
 	statistics::fresh_stats();
@@ -174,10 +269,7 @@ void replay_controller::reset_replay(){
 		// old manager is actually destroyed (triggering an assertion
 		// failure)
 		events_manager_.reset();
-		events_manager_.reset(new game_events::manager(level_,map_,
-								units_,teams_, gamestate_,status_));
-		events_manager_->set_gui(*gui_);
-		events_manager_->set_soundsource(*soundsources_manager_);
+		events_manager_.reset(new game_events::manager(level_));
 	}
 
 	gui_->new_turn();
@@ -186,43 +278,40 @@ void replay_controller::reset_replay(){
 	(*gui_).invalidate_all();
 	(*gui_).draw();
 
-	try {
-		fire_prestart(true);
-		fire_start(!loading_game_);
-	} catch (replay::error&) {
-		if(!continue_replay()) {
-			throw;
-		}
-	}
+	fire_prestart(true);
+	fire_start(!loading_game_);
 	gui_->new_turn();
 	gui_->invalidate_game_status();
 	events::raise_draw_event();
 	(*gui_).invalidate_all();
 	(*gui_).draw();
-	b = gui_->find_button("button-resetreplay");
-	if (b != NULL) { b->release(); }
+	gui_->set_team(player_number_-1, show_everything_);
+	//gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
+
+	buttons_.reset_buttons();
 }
 
 void replay_controller::stop_replay(){
 	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-playreplay");
-	if (b != NULL) { b->release(); }
 }
 
 void replay_controller::replay_next_turn(){
 	is_playing_ = true;
+	buttons_.playback_should_start();
+	
 	play_turn();
 
- 	if (!skip_replay_){
+ 	if (!skip_replay_ || !is_playing_){
 		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
-	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-nextturn");
-	if (b != NULL) { b->release(); }
+
+	buttons_.playback_should_stop(is_playing_);
 }
 
 void replay_controller::replay_next_side(){
 	is_playing_ = true;
+	buttons_.playback_should_start();
+	
 	play_side(player_number_ - 1, false);
 
 	if (static_cast<size_t>(player_number_) > teams_.size()) {
@@ -230,13 +319,24 @@ void replay_controller::replay_next_side(){
 		current_turn_++;
 	}
 
-	if (!skip_replay_) {
+	if (!skip_replay_ || !is_playing_) {
 		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
 
-	is_playing_ = false;
-	gui::button* b = gui_->find_button("button-nextside");
-	if (b != NULL) { b->release(); }
+	buttons_.playback_should_stop(is_playing_);
+}
+
+void replay_controller::process_oos(const std::string& msg) const
+{
+	if (game_config::ignore_replay_errors) return;
+
+	/** @todo FIXME: activate translation support after string freeze */
+	std::stringstream message;
+	message << "The replay is corrupt/out of sync. It might not make much sense to continue. Do you want to save the game?";
+	message << "\n\nError details:\n\n" << msg;
+
+	savegame::oos_savegame save(to_config());
+	save.save_game_interactive(resources::screen->video(), message.str(), gui::YES_NO); // can throw end_level_exception
 }
 
 void replay_controller::replay_show_everything(){
@@ -261,41 +361,37 @@ void replay_controller::replay_show_team1(){
 }
 
 void replay_controller::replay_skip_animation(){
-	recorder.set_skip(!recorder.is_skipping());
 	skip_replay_ = !skip_replay_;
+	recorder.set_skip(skip_replay_);
 }
 
 void replay_controller::play_replay(){
-	gui::button* b = gui_->find_button("button-stopreplay");
-	if (b != NULL) { b->release(); }
+
 	if (recorder.at_end()){
 		return;
 	}
 
 	try{
 		is_playing_ = true;
+		buttons_.playback_should_start();
 
 		DBG_REPLAY << "starting main loop\n" << (SDL_GetTicks() - ticks_) << "\n";
 		for(; !recorder.at_end() && is_playing_; first_player_ = 1) {
-			try{
-				play_turn();
-			}
-			catch (replay::error&) //when user due to error want stop playing
-			{
-				is_playing_ = false;
-			}
+			play_turn();
 		} //end for loop
-		is_playing_ = false;
+		
+		if (!is_playing_) {
+			gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
+		}
 	}
 	catch(end_level_exception& e){
 		if (e.result == QUIT) { throw e; }
 	}
+	
+	buttons_.playback_should_stop(is_playing_);
 }
 
 void replay_controller::play_turn(){
-	if (recorder.at_end()){
-		return;
-	}
 
 	LOG_REPLAY << "turn: " << current_turn_ << "\n";
 
@@ -313,11 +409,8 @@ void replay_controller::play_turn(){
 }
 
 void replay_controller::play_side(const unsigned int /*team_index*/, bool){
-	if (recorder.at_end()){
-		return;
-	}
 
-	DBG_REPLAY << "Status turn number: " << status_.turn() << "\n";
+	DBG_REPLAY << "Status turn number: " << turn() << "\n";
 	DBG_REPLAY << "Replay_Controller turn number: " << current_turn_ << "\n";
 	DBG_REPLAY << "Player number: " << player_number_ << "\n";
 
@@ -326,30 +419,16 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 		if (!current_team().is_empty()) {
 			statistics::reset_turn_stats(current_team().save_id());
 
-			try {
-				if (skip_next_turn_) {
-					skip_next_turn_ = false;
-					throw end_turn_exception();
-				}
-				play_controller::init_side(player_number_ - 1, true);
+			play_controller::init_side(player_number_ - 1, true);
 
-				DBG_REPLAY << "doing replay " << player_number_ << "\n";
-				try {
-					::do_replay(*gui_, map_, units_, teams_,
-							player_number_, status_, gamestate_);
-				} catch(replay::error&) {
-					if(!continue_replay()) {
-						throw;
-					}
-				}
-			} catch(end_turn_exception) {
-			}
+			DBG_REPLAY << "doing replay " << player_number_ << "\n";
+			do_replay(player_number_);
 
 			finish_side_turn();
 
 			// This is necessary for replays in order to show possible movements.
 			for (unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
-				if (uit->second.side() != static_cast<size_t>(player_number_)) {
+				if (uit->second.side() != player_number_) {
 					uit->second.new_turn();
 				}
 			}
@@ -358,23 +437,16 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 		player_number_++;
 
 		if (static_cast<size_t>(player_number_) > teams_.size()) {
-			status_.next_turn();
-			try {
-				finish_turn();
-			} catch (end_turn_exception) {
-				skip_next_turn_ = true;
-			}
+			next_turn();
+			finish_turn();
 			player_number_ = 1;
 			current_turn_++;
+			gui_->new_turn();
 		}
 
 		update_teams();
 		update_gui();
 	}
-	catch (replay::error&) //if replay throws an error, we don't want to get thrown out completely
-	{
-		is_playing_ = false;
-	}
 	catch(end_level_exception& e){
 		//VICTORY/DEFEAT end_level_exception shall not return to title screen
 		if ((e.result != VICTORY) && (e.result != DEFEAT)) { throw e; }
@@ -387,11 +459,14 @@ void replay_controller::update_teams(){
 		next_team = 1;
 	}
 
-	if (!show_team_)
-		gui_->set_team(next_team - 1, show_everything_);
+	if (!show_team_) {
+ 		gui_->set_team(next_team - 1, show_everything_);
+	} else {
+		gui_->set_team(show_team_ - 1, show_everything_);
+	}
 
-	::clear_shroud(*gui_, map_, units_, teams_,
-		next_team - 1);
+	::clear_shroud(next_team);
+	
 	gui_->set_playing_team(next_team - 1);
 	gui_->invalidate_all();
 }
@@ -406,7 +481,6 @@ void replay_controller::update_gui(){
 
 void replay_controller::preferences(){
 	play_controller::preferences();
-	init_replay_display();
 	update_gui();
 }
 
@@ -414,8 +488,17 @@ void replay_controller::show_statistics(){
 	menu_handler_.show_statistics(gui_->playing_team()+1);
 }
 
-void replay_controller::handle_generic_event(const std::string& /*name*/){
-	rebuild_replay_theme();
+void replay_controller::handle_generic_event(const std::string& name){
+	
+	if( name == "completely_redrawn" ) {
+		buttons_.update(gui_);
+		
+		gui::button* skip_animation_button = gui_->find_button("skip-animation");
+		
+		skip_animation_button->set_check(skip_replay_);
+	} else {
+		rebuild_replay_theme();
+	}
 }
 
 bool replay_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int index) const
