@@ -1,6 +1,6 @@
-/* $Id: reports.hpp 33240 2009-03-01 15:46:42Z soliton $ */
+/* $Id: reports.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,8 @@
 #include <vector>
 
 #include "image.hpp"
-#include "team.hpp"
 
-class gamestatus;
-class unit_map;
+class team;
 
 //this module is responsible for outputting textual reports of
 //various game and unit statistics
@@ -31,15 +29,15 @@ namespace reports {
 	enum TYPE { UNIT_NAME, UNIT_TYPE, UNIT_RACE, UNIT_LEVEL,
 		    UNIT_SIDE, UNIT_AMLA, UNIT_TRAITS, UNIT_STATUS,
 		    UNIT_ALIGNMENT, UNIT_ABILITIES, UNIT_HP, UNIT_XP,
-		    UNIT_ADVANCEMENT_OPTIONS, UNIT_MOVES, UNIT_WEAPONS,
+		    UNIT_ADVANCEMENT_OPTIONS, UNIT_DEFENSE, UNIT_MOVES, UNIT_WEAPONS,
 		    UNIT_IMAGE, UNIT_PROFILE, TIME_OF_DAY,
 		    TURN, GOLD, VILLAGES, NUM_UNITS, UPKEEP, EXPENSES,
 		    INCOME, TERRAIN, POSITION, SIDE_PLAYING, OBSERVERS,
 		    REPORT_COUNTDOWN, REPORT_CLOCK, SELECTED_TERRAIN,
-		    EDIT_LEFT_BUTTON_FUNCTION, EDITOR2_TOOL_HINT, NUM_REPORTS};
+		    EDIT_LEFT_BUTTON_FUNCTION, EDITOR_TOOL_HINT, NUM_REPORTS};
 
 	enum { UNIT_REPORTS_BEGIN=UNIT_NAME, UNIT_REPORTS_END=UNIT_PROFILE+1 };
-	enum { STATUS_REPORTS_BEGIN=TIME_OF_DAY, STATUS_REPORTS_END=EDITOR2_TOOL_HINT };
+	enum { STATUS_REPORTS_BEGIN=TIME_OF_DAY, STATUS_REPORTS_END=EDITOR_TOOL_HINT };
 
 	const std::string& report_name(TYPE type);
 
@@ -84,19 +82,16 @@ namespace reports {
 		}
 
 		// Convenience functions
-		void add_text(std::stringstream& text, std::stringstream& tooltip);
 		void add_text(const std::string& text, const std::string& tooltip);
 		void add_image(const std::string& image, const std::string& tooltip);
-		void add_image(std::stringstream& image, std::stringstream& tooltip);
 	};
 
 	report generate_report(TYPE type,
 			       std::map<reports::TYPE, std::string> report_contents,
-			       const gamemap& map, unit_map& units,
-			       const std::vector<team>& teams, const team& current_team,
-			       unsigned int current_side, int unsigned active_side,
+		const team &current_team,
+			       int current_side, int active_side,
 			       const map_location& loc, const map_location& mouseover, const map_location& displayed_unit_hex,
-			       const gamestatus& status, const std::set<std::string>& observers,
+		const std::set<std::string> &observers,
 			       const config& level, bool show_everything = false);
 }
 
