@@ -1,6 +1,6 @@
-/* $Id: sdl_utils.cpp 34753 2009-04-12 06:01:26Z boucman $ */
+/* $Id: sdl_utils.cpp 49411 2011-05-07 15:11:50Z soliton $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,10 +19,12 @@
 
 #include "global.hpp"
 
-#include "config.hpp"
 #include "video.hpp"
 
+#include <algorithm>
+#include <cassert>
 #include <cstring>
+#include <iostream>
 
 SDL_Color int_to_color(const Uint32 rgb) {
 	SDL_Color to_return = {
@@ -33,13 +35,6 @@ SDL_Color int_to_color(const Uint32 rgb) {
 	return to_return;
 }
 
-Uint32 color_to_int(const SDL_Color& color) {
-	Uint32 to_return = color.r;
-	to_return = (to_return << 8) & color.g;
-	to_return = (to_return << 8) & color.b;
-	return to_return;
-}
-
 SDLKey sdl_keysym_from_name(std::string const &keyname)
 {
 	static bool initialized = false;
@@ -83,8 +78,7 @@ SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
 	return res;
 }
 
-SDL_Rect get_rect_union(const SDL_Rect& rect1, const SDL_Rect& rect2)
-{
+SDL_Rect get_rect_union(SDL_Rect const &rect1, SDL_Rect const& rect2) {
 	const int left_side = std::max(rect1.x, rect2.x);
 	const int right_side = std::min(rect1.x + rect1.w, rect2.x + rect2.w);
 	if(left_side > right_side) {
@@ -105,7 +99,6 @@ SDL_Rect get_rect_union(const SDL_Rect& rect1, const SDL_Rect& rect2)
 
 	return result;
 }
-
 SDL_Rect create_rect(const int x, const int y, const int w, const int h)
 {
 	SDL_Rect rect = { x, y, w, h };
@@ -132,7 +125,7 @@ static SDL_PixelFormat& get_neutral_pixel_format()
 		return format;
 	}
 
-surface make_neutral_surface(surface const &surf)
+surface make_neutral_surface(const surface &surf)
 {
 	if(surf == NULL) {
 		std::cerr << "null neutral surface...\n";
@@ -165,7 +158,7 @@ surface create_neutral_surface(int w, int h)
 	return result;
 }
 
-surface create_optimized_surface(surface const &surf)
+surface create_optimized_surface(const surface &surf)
 {
 	if(surf == NULL)
 		return NULL;
@@ -271,7 +264,7 @@ surface stretch_surface_vertical(
 }
 
 // NOTE: Don't pass this function 0 scaling arguments.
-surface scale_surface(surface const &surf, int w, int h, bool optimize)
+surface scale_surface(const surface &surf, int w, int h, bool optimize)
 {
 	// Since SDL version 1.1.5 0 is transparent, before 255 was transparent.
 	assert(SDL_ALPHA_TRANSPARENT==0);
@@ -409,9 +402,9 @@ surface scale_surface(surface const &surf, int w, int h, bool optimize)
 				  g = pix[loc] >> 8;
 				  b = pix[loc] >> 0;
 				  if (a == 0) {
-				    r = avg_r;
-				    g = avg_g;
-				    b = avg_b;
+				    r = static_cast<Uint8>(avg_r);
+				    g = static_cast<Uint8>(avg_g);
+				    b = static_cast<Uint8>(avg_b);
 				  }
 				  rr += r * bilin[loc];
 				  gg += g * bilin[loc];
@@ -435,7 +428,7 @@ surface scale_surface(surface const &surf, int w, int h, bool optimize)
 // This is a copy-paste of the previous function scale_surface
 // We only removed the alpha channel and big comments
 // and change how we optimize the resulting surface
-surface scale_opaque_surface(surface const &surf, int w, int h, bool optimize_format)
+surface scale_opaque_surface(const surface &surf, int w, int h, bool optimize_format)
 {
 	if(surf == NULL)
 		return NULL;
@@ -524,7 +517,7 @@ surface scale_opaque_surface(surface const &surf, int w, int h, bool optimize_fo
 }
 
 
-surface scale_surface_blended(surface const &surf, int w, int h, bool optimize)
+surface scale_surface_blended(const surface &surf, int w, int h, bool optimize)
 {
 	if(surf== NULL)
 		return NULL;
@@ -608,7 +601,7 @@ surface scale_surface_blended(surface const &surf, int w, int h, bool optimize)
 	return optimize ? create_optimized_surface(dst) : dst;
 }
 
-surface adjust_surface_colour(surface const &surf, int red, int green, int blue, bool optimize)
+surface adjust_surface_colour(const surface &surf, int red, int green, int blue, bool optimize)
 {
 	if((red == 0 && green == 0 && blue == 0) || surf == NULL)
 		return create_optimized_surface(surf);
@@ -648,7 +641,7 @@ surface adjust_surface_colour(surface const &surf, int red, int green, int blue,
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface greyscale_image(surface const &surf, bool optimize)
+surface greyscale_image(const surface &surf, bool optimize)
 {
 	if(surf == NULL)
 		return NULL;
@@ -693,57 +686,7 @@ surface greyscale_image(surface const &surf, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface darken_image(surface const &surf, bool optimize)
-{
-	if(surf == NULL)
-		return NULL;
-
-	surface nsurf(make_neutral_surface(surf));
-	if(nsurf == NULL) {
-		std::cerr << "failed to make neutral surface\n";
-		return NULL;
-	}
-
-	{
-		surface_lock lock(nsurf);
-		Uint32* beg = lock.pixels();
-		Uint32* end = beg + nsurf->w*surf->h;
-
-		while(beg != end) {
-			Uint8 alpha = (*beg) >> 24;
-
-			if(alpha) {
-				Uint8 r, g, b;
-				r = (*beg) >> 16;
-				g = (*beg) >> 8;
-				b = (*beg);
-
-				//const Uint8 avg = (red+green+blue)/3;
-
-				// Use the correct formula for RGB to grayscale conversion.
-				// Ok, this is no big deal :)
-				// The correct formula being:
-				// gray=0.299red+0.587green+0.114blue
-				const Uint8 avg = static_cast<Uint8>((
-					77  * static_cast<Uint16>(r) +
-					150 * static_cast<Uint16>(g) +
-					29  * static_cast<Uint16>(b)  ) / 256);
-				// then tint 77%, 67%, 72%
-				r = ((avg * 196) >> 8);
-				g = ((avg * 171) >> 8);
-				b = ((avg * 184) >> 8);
-
-				*beg = (alpha << 24) | (r << 16) | (g << 8) | b;
-			}
-
-			++beg;
-		}
-	}
-
-	return optimize ? create_optimized_surface(nsurf) : nsurf;
-}
-
-surface shadow_image(surface const &surf, bool optimize)
+surface shadow_image(const surface &surf, bool optimize)
 {
 	if(surf == NULL)
 		return NULL;
@@ -796,8 +739,6 @@ surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb, boo
 		Uint32* beg = lock.pixels();
 		Uint32* end = beg + nsurf->w*surf->h;
 
-		std::map<Uint32, Uint32>::const_iterator map_rgb_end = map_rgb.end();
-
 		while(beg != end) {
 			Uint8 alpha = (*beg) >> 24;
 
@@ -817,7 +758,7 @@ surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb, boo
 	return surf;
 }
 
-surface brighten_image(surface const &surf, fixed_t amount, bool optimize)
+surface brighten_image(const surface &surf, fixed_t amount, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -859,7 +800,7 @@ surface brighten_image(surface const &surf, fixed_t amount, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize)
+surface adjust_surface_alpha(const surface &surf, fixed_t amount, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -898,7 +839,7 @@ surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
+surface adjust_surface_alpha_add(const surface &surf, int amount, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -936,7 +877,7 @@ surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface mask_surface(surface const &surf, surface const &mask)
+surface mask_surface(const surface &surf, const surface &mask)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -992,7 +933,7 @@ surface mask_surface(surface const &surf, surface const &mask)
 	//return create_optimized_surface(nsurf);
 }
 
-bool in_mask_surface(surface const &surf, surface const &mask)
+bool in_mask_surface(const surface &surf, const surface &mask)
 {
 	if(surf == NULL || mask == NULL) {
 		return false;
@@ -1035,8 +976,7 @@ bool in_mask_surface(surface const &surf, surface const &mask)
 	return true;
 }
 
-
-surface blur_surface(surface const &surf, int depth, bool optimize)
+surface blur_surface(const surface &surf, int depth, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1049,7 +989,22 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 		return NULL;
 	}
 
-	const int max_blur = 256;
+	SDL_Rect rect = { 0, 0, surf->w, surf->h };
+	blur_surface(res, rect, depth);
+
+	return optimize ? create_optimized_surface(res) : res;
+}
+
+void blur_surface(surface& surf, SDL_Rect rect, unsigned depth)
+{
+	if(surf == NULL) {
+		return;
+	}
+
+	assert((surf->flags & SDL_RLEACCEL) == 0);
+	assert(surf->format->BitsPerPixel == 32);
+
+	const unsigned max_blur = 256;
 	if(depth > max_blur) {
 		depth = max_blur;
 	}
@@ -1059,14 +1014,15 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 
 	const Uint32 ff = 0xff;
 
-	surface_lock lock(res);
-	int x, y;
-	for(y = 0; y < res->h; ++y) {
+	const unsigned pixel_offset = rect.y * surf->w + rect.x;
+
+	surface_lock lock(surf);
+	for(unsigned y = 0; y < rect.h; ++y) {
 		const Uint32* front = &queue[0];
 		Uint32* back = &queue[0];
 		Uint32 red = 0, green = 0, blue = 0, avg = 0;
-		Uint32* p = lock.pixels() + y*res->w;
-		for(x = 0; x <= depth && x < res->w; ++x, ++p) {
+		Uint32* p = lock.pixels() + pixel_offset + y * surf->w;
+		for(unsigned x = 0; x <= depth && x < rect.w; ++x, ++p) {
 			red += ((*p) >> 16)&0xFF;
 			green += ((*p) >> 8)&0xFF;
 			blue += (*p)&0xFF;
@@ -1077,9 +1033,13 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 			}
 		}
 
-		p = lock.pixels() + y*res->w;
-		for(x = 0; x < res->w; ++x, ++p) {
-			*p = 0xFF000000 | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
+		p = lock.pixels() + pixel_offset + y * surf->w;
+		for(unsigned x = 0; x < rect.w; ++x, ++p) {
+			*p = 0xFF000000
+					| (std::min(red/avg,ff) << 16)
+					| (std::min(green/avg,ff) << 8)
+					| std::min(blue/avg,ff);
+
 			if(x >= depth) {
 				red -= ((*front) >> 16)&0xFF;
 				green -= ((*front) >> 8)&0xFF;
@@ -1091,7 +1051,7 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 				}
 			}
 
-			if(x + depth+1 < res->w) {
+			if(x + depth+1 < rect.w) {
 				Uint32* q = p + depth+1;
 				red += ((*q) >> 16)&0xFF;
 				green += ((*q) >> 8)&0xFF;
@@ -1105,12 +1065,12 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 		}
 	}
 
-	for(x = 0; x < res->w; ++x) {
+	for(unsigned x = 0; x < rect.w; ++x) {
 		const Uint32* front = &queue[0];
 		Uint32* back = &queue[0];
 		Uint32 red = 0, green = 0, blue = 0, avg = 0;
-		Uint32* p = lock.pixels() + x;
-		for(y = 0; y <= depth && y < res->h; ++y, p += res->w) {
+		Uint32* p = lock.pixels() + pixel_offset + x;
+		for(unsigned y = 0; y <= depth && y < rect.h; ++y, p += surf->w) {
 			red += ((*p) >> 16)&0xFF;
 			green += ((*p) >> 8)&0xFF;
 			blue += *p&0xFF;
@@ -1121,9 +1081,13 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 			}
 		}
 
-		p = lock.pixels() + x;
-		for(y = 0; y < res->h; ++y, p += res->w) {
-			*p = 0xFF000000 | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
+		p = lock.pixels() + pixel_offset + x;
+		for(unsigned y = 0; y < rect.h; ++y, p += surf->w) {
+			*p = 0xFF000000
+					| (std::min(red/avg,ff) << 16)
+					| (std::min(green/avg,ff) << 8)
+					| std::min(blue/avg,ff);
+
 			if(y >= depth) {
 				red -= ((*front) >> 16)&0xFF;
 				green -= ((*front) >> 8)&0xFF;
@@ -1135,8 +1099,8 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 				}
 			}
 
-			if(y + depth+1 < res->h) {
-				Uint32* q = p + (depth+1)*res->w;
+			if(y + depth+1 < rect.h) {
+				Uint32* q = p + (depth+1)*surf->w;
 				red += ((*q) >> 16)&0xFF;
 				green += ((*q) >> 8)&0xFF;
 				blue += (*q)&0xFF;
@@ -1148,11 +1112,9 @@ surface blur_surface(surface const &surf, int depth, bool optimize)
 			}
 		}
 	}
-
-	return optimize ? create_optimized_surface(res) : res;
 }
 
-surface blur_alpha_surface(surface const &surf, int depth, bool optimize)
+surface blur_alpha_surface(const surface &surf, int depth, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1274,7 +1236,7 @@ surface blur_alpha_surface(surface const &surf, int depth, bool optimize)
 	return optimize ? create_optimized_surface(res) : res;
 }
 
-surface cut_surface(surface const &surf, SDL_Rect const &r)
+surface cut_surface(const surface &surf, SDL_Rect const &r)
 {
 	surface res = create_compatible_surface(surf, r.w, r.h);
 
@@ -1325,7 +1287,7 @@ surface cut_surface(surface const &surf, SDL_Rect const &r)
 	return res;
 }
 
-surface blend_surface(surface const &surf, double amount, Uint32 colour, bool optimize)
+surface blend_surface(const surface &surf, double amount, Uint32 colour, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -1372,7 +1334,7 @@ surface blend_surface(surface const &surf, double amount, Uint32 colour, bool op
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface flip_surface(surface const &surf, bool optimize)
+surface flip_surface(const surface &surf, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1401,7 +1363,7 @@ surface flip_surface(surface const &surf, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface flop_surface(surface const &surf, bool optimize)
+surface flop_surface(const surface &surf, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1431,7 +1393,7 @@ surface flop_surface(surface const &surf, bool optimize)
 }
 
 
-surface create_compatible_surface(surface const &surf, int width, int height)
+surface create_compatible_surface(const surface &surf, int width, int height)
 {
 	if(surf == NULL)
 		return NULL;
@@ -1442,8 +1404,12 @@ surface create_compatible_surface(surface const &surf, int width, int height)
 	if(height == -1)
 		height = surf->h;
 
-	return SDL_CreateRGBSurface(SDL_SWSURFACE,width,height,surf->format->BitsPerPixel,
-		                        surf->format->Rmask,surf->format->Gmask,surf->format->Bmask,surf->format->Amask);
+	surface s = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, surf->format->BitsPerPixel,
+		surf->format->Rmask, surf->format->Gmask, surf->format->Bmask, surf->format->Amask);
+	if (surf->format->palette) {
+		SDL_SetPalette(s, SDL_LOGPAL, surf->format->palette->colors, 0, surf->format->palette->ncolors);
+	}
+	return s;
 }
 
 void blit_surface(const surface& src,
@@ -1601,7 +1567,7 @@ void blit_surface(const surface& src,
 
 
 
-void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &target)
+void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, const surface &target)
 {
 	if(alpha == SDL_ALPHA_OPAQUE) {
 		SDL_FillRect(target,&rect,colour);
@@ -1621,7 +1587,7 @@ void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &
 	SDL_BlitSurface(tmp,NULL,target,&rect);
 }
 
-surface get_surface_portion(surface const &src, SDL_Rect &area, bool optimize_format)
+surface get_surface_portion(const surface &src, SDL_Rect &area, bool optimize_format)
 {
 	if (src == NULL) {
 		return NULL;
@@ -1667,7 +1633,7 @@ struct not_alpha
 
 }
 
-SDL_Rect get_non_transparent_portion(surface const &surf)
+SDL_Rect get_non_transparent_portion(const surface &surf)
 {
 	SDL_Rect res = {0,0,0,0};
 	const surface nsurf(make_neutral_surface(surf));
@@ -1763,27 +1729,6 @@ SDL_Color inverse(const SDL_Color& colour) {
 	return inverse;
 }
 
-void pixel_data::read(const config& cfg) {
-	const std::string& red = cfg["red"];
-	const std::string& green = cfg["green"];
-	const std::string& blue = cfg["blue"];
-
-	if (red.empty())
-		r = 0;
-	else
-		r = atoi(red.c_str());
-
-	if (green.empty())
-		g = 0;
-	else
-		g = atoi(green.c_str());
-
-	if (blue.empty())
-		b = 0;
-	else
-		b = atoi(blue.c_str());
-}
-
 surface_restorer::surface_restorer() : target_(NULL), rect_(empty_rect), surface_(NULL)
 {
 }
