@@ -1,8 +1,8 @@
-/* $Id: parser.cpp 35988 2009-06-01 11:41:05Z silene $ */
+/* $Id: parser.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2011 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include "wesconfig.h"
 #include "serialization/preprocessor.hpp"
 #include "serialization/tokenizer.hpp"
+#include "serialization/string_utils.hpp"
 #include "foreach.hpp"
 
 #include <stack>
@@ -37,9 +38,10 @@
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/algorithm/string/replace.hpp>
 
-#define ERR_CF LOG_STREAM(err, config)
-#define WRN_CF LOG_STREAM(warn, config)
-#define LOG_CF LOG_STREAM(info, config)
+static lg::log_domain log_config("config");
+#define ERR_CF LOG_STREAM(err, log_config)
+#define WRN_CF LOG_STREAM(warn, log_config)
+#define LOG_CF LOG_STREAM(info, log_config)
 
 static const size_t max_recursion_levels = 1000;
 
@@ -58,9 +60,8 @@ public:
 private:
 	void parse_element();
 	void parse_variable();
-	void parse_directive();
 	std::string lineno_string(utils::string_map &map, std::string const &lineno,
-			          std::string const &error_string);
+		const char *error_string);
 	void error(const std::string& message);
 
 	config& cfg_;
@@ -97,9 +98,7 @@ parser::parser(config &cfg, std::istream &in) :
 
 parser::~parser()
 {
-	if(tok_) {
-		delete tok_;
-	}
+	delete tok_;
 }
 
 void parser::operator()(std::string* error_log)
@@ -121,9 +120,9 @@ void parser::operator()(std::string* error_log)
 				parse_variable();
 				break;
 			default:
-				if ((unsigned char)tok_->current_token().value[0] == 0xEF &&
-				    (unsigned char)tok_->next_token().value[0]    == 0xBB &&
-				    (unsigned char)tok_->next_token().value[0]    == 0xBF)
+				if (static_cast<unsigned char>(tok_->current_token().value[0]) == 0xEF &&
+				    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBB &&
+				    static_cast<unsigned char>(tok_->next_token().value[0])    == 0xBF)
 				{
 					ERR_CF << "Skipping over a utf8 BOM\n";
 				} else {
@@ -325,11 +324,11 @@ void parser::parse_variable()
 /**
  * This function is crap. Don't use it on a string_map with prefixes.
  */
-std::string parser::lineno_string(utils::string_map &i18n_symbols, std::string const &lineno,
-			          std::string const &error_string)
+std::string parser::lineno_string(utils::string_map &i18n_symbols,
+	std::string const &lineno, const char *error_string)
 {
 	i18n_symbols["pos"] = ::lineno_string(lineno);
-	std::string result = _(error_string.c_str());
+	std::string result = _(error_string);
 	foreach(utils::string_map::value_type& var, i18n_symbols)
 		boost::algorithm::replace_all(result, std::string("$") + var.first, std::string(var.second));
 	return result;
@@ -412,6 +411,9 @@ void write_key_val(std::ostream &out, const std::string &key, const t_string &va
 {
 	bool first = true;
 	if (value.empty()) {
+		out << std::string(level, '\t') << key << AttributeEquals
+			<< AttributePrefix << AttributePostfix
+			<< AttributeEndPostfix;;
 		return;
 	}
 
@@ -476,18 +478,15 @@ static void write_internal(config const &cfg, std::ostream &out, std::string& te
 	if (tab > max_recursion_levels)
 		throw config::error("Too many recursion levels in config write");
 
-	for(string_map::const_iterator i = cfg.values.begin(), i_end = cfg.values.end(); i != i_end; ++i) {
-		write_key_val(out, i->first, i->second, tab, textdomain);
+	foreach (const config::attribute &i, cfg.attribute_range()) {
+		write_key_val(out, i.first, i.second, tab, textdomain);
 	}
 
-	for(config::all_children_iterator j = cfg.ordered_begin(), j_end = cfg.ordered_end(); j != j_end; ++j) {
-		const std::pair<const std::string*,const config*>& item = *j;
-		const std::string& name = *item.first;
-		const config& cfg = *item.second;
-
-		write_open_child(out, name, tab);
-		write_internal(cfg, out, textdomain, tab + 1);
-		write_close_child(out, name, tab);
+	foreach (const config::any_child &item, cfg.all_children_range())
+	{
+		write_open_child(out, item.key, tab);
+		write_internal(item.cfg, out, textdomain, tab + 1);
+		write_close_child(out, item.key, tab);
 	}
 }
 
