@@ -1,7 +1,7 @@
-/* $Id: preprocessor.hpp 34873 2009-04-13 15:17:37Z silene $ */
+/* $Id: preprocessor.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    See the COPYING file for more details.
 */
 
-/** @file serialization/preprocessor.hpp. */
+/** @file serialization/preprocessor.hpp */
 
 #ifndef SERIALIZATION_PREPROCESSOR_HPP_INCLUDED
 #define SERIALIZATION_PREPROCESSOR_HPP_INCLUDED
@@ -47,7 +47,7 @@ struct preproc_define
 	void write_argument(config_writer&, const std::string&) const;
 	void read(const config&);
 	void read_argument(const config &);
-	static preproc_map::value_type read_pair(const config*);
+	static preproc_map::value_type read_pair(const config &);
 	bool operator==(preproc_define const &) const;
 	bool operator<(preproc_define const &) const;
 	bool operator!=(preproc_define const &v) const { return !operator==(v); }
