@@ -1,6 +1,6 @@
-/* $Id: soundsource.hpp 31859 2009-01-01 10:28:26Z mordante $ */
+/* $Id: soundsource.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
+   Copyright (C) 2006 - 2011 by Karol Nowak <grzywacz@sul.uni.lodz.pl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #include "generic_event.hpp"
 #include "map_location.hpp"
+#include "savegame_config.hpp"
 
 class config;
 class display;
@@ -43,6 +44,7 @@ class positional_source {
 	unsigned int range_;
 	unsigned int faderange_;
 	bool check_fogged_;
+	bool check_shrouded_;
 	std::string files_;
 	std::vector<map_location> locations_;
 
@@ -51,7 +53,6 @@ class positional_source {
 	static unsigned int last_id;
 
 public:
-
 	// min_delay is a minimum time in seconds, which must pass before
 	// this sound source can be played again if it remains visible
 	//
@@ -61,13 +62,11 @@ public:
 	positional_source(const sourcespec &spec);
 	~positional_source();
 
+	bool is_global();
+
 	void update(unsigned int time, const display &disp);
 	void update_positions(unsigned int time, const display &disp);
 
-	void add_location(const map_location &loc);
-	void remove_location(const map_location &loc);
-	void replace_location(const map_location &oldloc, const map_location &newloc);
-
 	int calculate_volume(const map_location &loc, const display &disp);
 
 	/**
@@ -77,7 +76,7 @@ public:
 	void write_config(config& cfg) const;
 };
 
-class manager : public events::observer {
+class manager : public events::observer, public savegame::savegame_config {
 
 	typedef std::map<std::string, positional_source *> positional_source_map;
 	typedef positional_source_map::iterator            positional_source_iterator;
@@ -98,8 +97,6 @@ public:
 	void remove(const std::string &id);
 	void update();
 
-	void add_location(const std::string &id, const map_location &loc);
-
 	// checks which sound sources are visible
 	void update_positions();
 
@@ -108,6 +105,8 @@ public:
 	 * "sound_source", appendend to existing content.
 	 */
 	void write_sourcespecs(config& cfg) const;
+
+	config to_config() const;
 };
 
 /**
@@ -127,6 +126,7 @@ class sourcespec
 	int range_;
 	int faderange_;
 	bool check_fogged_;
+	bool check_shrouded_;
 
 	std::vector<map_location> locations_;
 
@@ -141,6 +141,7 @@ public:
 		range_(3),
 		faderange_(14),
 		check_fogged_(false),
+		check_shrouded_(false),
 		locations_()
 	{}
 
@@ -160,17 +161,18 @@ public:
 	}
 
 	bool check_fogged() const { return check_fogged_; }
+	bool check_shrouded() const { return check_shrouded_; }
 
 	void set_check_fogged(bool value) {
 		check_fogged_ = value;
 	}
 
-	const std::vector<map_location>& get_locations() const {
-		return locations_;
+	void set_check_shrouded(bool value) {
+		check_shrouded_ = value;
 	}
 
-	void add_location(const map_location& loc) {
-		locations_.push_back(loc);
+	const std::vector<map_location>& get_locations() const {
+		return locations_;
 	}
 
 	int full_range() const { return range_; }
