@@ -1,6 +1,6 @@
-/* $Id: statistics.cpp 33516 2009-03-10 08:37:11Z silene $ */
+/* $Id: statistics.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,13 +19,14 @@
 
 #include "global.hpp"
 #include "statistics.hpp"
+#include "foreach.hpp"
 #include "log.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "unit.hpp"
 #include "util.hpp"
 
-#define ERR_NG lg::err(lg::engine)
-#define DBG_NG LOG_STREAM(debug, engine)
+static lg::log_domain log_engine("engine");
+#define DBG_NG LOG_STREAM(debug, log_engine)
 
 namespace {
 
@@ -35,8 +36,6 @@ bool mid_scenario = false;
 
 typedef statistics::stats stats;
 
-int stats_disabled = 0;
-
 struct scenario_stats
 {
 	explicit scenario_stats(const std::string& name) :
@@ -57,9 +56,8 @@ scenario_stats::scenario_stats(const config& cfg) :
 	team_stats(),
 	scenario_name(cfg["scenario"])
 {
-	const config::child_list& teams = cfg.get_children("team");
-	for(config::child_list::const_iterator i = teams.begin(); i != teams.end(); ++i) {
-		team_stats[(**i)["save_id"]] = stats(**i);
+	foreach (const config &team, cfg.child_range("team")) {
+		team_stats[team["save_id"]] = stats(team);
 	}
 }
 
@@ -122,8 +120,8 @@ static void write_str_int_map(config_writer &out, const stats::str_int_map& m)
 static stats::str_int_map read_str_int_map(const config& cfg)
 {
 	stats::str_int_map m;
-	for(string_map::const_iterator i = cfg.values.begin(); i != cfg.values.end(); ++i) {
-		m[i->first] = atoi(i->second.c_str());
+	foreach (const config::attribute &i, cfg.attribute_range()) {
+		m[i.first] = atoi(i.second.c_str());
 	}
 
 	return m;
@@ -160,11 +158,11 @@ static void write_battle_result_map(config_writer &out, const stats::battle_resu
 static stats::battle_result_map read_battle_result_map(const config& cfg)
 {
 	stats::battle_result_map m;
-	const config::child_list c = cfg.get_children("sequence");
-	for(config::child_list::const_iterator i = c.begin(); i != c.end(); ++i) {
-		config item = **i;
+	foreach (const config &i, cfg.child_range("sequence"))
+	{
+		config item = i;
 		const int key = atoi(item["_num"].c_str());
-		item.values.erase("_num");
+		item.remove_attribute("_num");
 		m[key] = read_str_int_map(item);
 	}
 
@@ -368,29 +366,29 @@ void stats::write(config_writer &out) const
 
 void stats::read(const config& cfg)
 {
-	if(cfg.child("recruits")) {
-		recruits = read_str_int_map(*cfg.child("recruits"));
+	if (const config &c = cfg.child("recruits")) {
+		recruits = read_str_int_map(c);
 	}
-	if(cfg.child("recalls")) {
-		recalls = read_str_int_map(*cfg.child("recalls"));
+	if (const config &c = cfg.child("recalls")) {
+		recalls = read_str_int_map(c);
 	}
-	if(cfg.child("advances")) {
-		advanced_to = read_str_int_map(*cfg.child("advances"));
+	if (const config &c = cfg.child("advances")) {
+		advanced_to = read_str_int_map(c);
 	}
-	if(cfg.child("deaths")) {
-		deaths = read_str_int_map(*cfg.child("deaths"));
+	if (const config &c = cfg.child("deaths")) {
+		deaths = read_str_int_map(c);
 	}
-	if(cfg.child("killed")) {
-		killed = read_str_int_map(*cfg.child("killed"));
+	if (const config &c = cfg.child("killed")) {
+		killed = read_str_int_map(c);
 	}
-	if(cfg.child("recalls")) {
-		recalls = read_str_int_map(*cfg.child("recalls"));
+	if (const config &c = cfg.child("recalls")) {
+		recalls = read_str_int_map(c);
 	}
-	if(cfg.child("attacks")) {
-		attacks = read_battle_result_map(*cfg.child("attacks"));
+	if (const config &c = cfg.child("attacks")) {
+		attacks = read_battle_result_map(c);
 	}
-	if(cfg.child("defends")) {
-		defends = read_battle_result_map(*cfg.child("defends"));
+	if (const config &c = cfg.child("defends")) {
+		defends = read_battle_result_map(c);
 	}
 
 	recruit_cost = lexical_cast<long long>(cfg["recruit_cost"]);
@@ -409,9 +407,6 @@ void stats::read(const config& cfg)
 	save_id = cfg["save_id"];
 }
 
-disabler::disabler() { stats_disabled++; }
-disabler::~disabler() { stats_disabled--; }
-
 scenario_context::scenario_context(const std::string& name)
 {
 	if(!mid_scenario || master_stats.empty()) {
@@ -441,9 +436,6 @@ attack_context::attack_context(const unit& a,
 
 attack_context::~attack_context()
 {
-	if(stats_disabled > 0)
-		return;
-
 	std::string attacker_key = "s" + attacker_res;
 	std::string defender_key = "s" + defender_res;
 
@@ -479,9 +471,6 @@ void attack_context::attack_expected_damage(double attacker_inflict_, double def
 
 void attack_context::attack_result(hit_result res, int damage, int drain)
 {
-	if(stats_disabled > 0)
-		return;
-
 	attacker_res.push_back(res == MISSES ? '0' : '1');
 	stats &att_stats = attacker_stats(), &def_stats = defender_stats();
 
@@ -506,9 +495,6 @@ void attack_context::attack_result(hit_result res, int damage, int drain)
 
 void attack_context::defend_result(hit_result res, int damage, int drain)
 {
-	if(stats_disabled > 0)
-		return;
-
 	defender_res.push_back(res == MISSES ? '0' : '1');
 	stats &att_stats = attacker_stats(), &def_stats = defender_stats();
 
@@ -533,9 +519,6 @@ void attack_context::defend_result(hit_result res, int damage, int drain)
 
 void recruit_unit(const unit& u)
 {
-	if(stats_disabled > 0)
-		return;
-
 	stats& s = get_stats(u.side_id());
 	s.recruits[u.type_id()]++;
 	s.recruit_cost += u.cost();
@@ -543,9 +526,6 @@ void recruit_unit(const unit& u)
 
 void recall_unit(const unit& u)
 {
-	if(stats_disabled > 0)
-		return;
-
 	stats& s = get_stats(u.side_id());
 	s.recalls[u.type_id()]++;
 	s.recall_cost += u.cost();
@@ -553,9 +533,6 @@ void recall_unit(const unit& u)
 
 void un_recall_unit(const unit& u)
 {
-	if(stats_disabled > 0)
-		return;
-
 	stats& s = get_stats(u.side_id());
 	s.recalls[u.type_id()]--;
 	s.recall_cost -= u.cost();
@@ -563,9 +540,6 @@ void un_recall_unit(const unit& u)
 
 void un_recruit_unit(const unit& u)
 {
-	if(stats_disabled > 0)
-		return;
-
 	stats& s = get_stats(u.side_id());
 	s.recruits[u.type_id()]--;
 	s.recruit_cost -= u.cost();
@@ -574,9 +548,6 @@ void un_recruit_unit(const unit& u)
 
 void advance_unit(const unit& u)
 {
-	if(stats_disabled > 0)
-		return;
-
 	stats& s = get_stats(u.side_id());
 	s.advanced_to[u.type_id()]++;
 }
@@ -640,9 +611,8 @@ void read_stats(const config& cfg)
 	fresh_stats();
 	mid_scenario = (utils::string_bool(cfg["mid_scenario"]));
 
-	const config::child_list& scenarios = cfg.get_children("scenario");
-	for(config::child_list::const_iterator i = scenarios.begin(); i != scenarios.end(); ++i) {
-		master_stats.push_back(scenario_stats(**i));
+	foreach (const config &s, cfg.child_range("scenario")) {
+		master_stats.push_back(scenario_stats(s));
 	}
 }
 
@@ -670,5 +640,15 @@ int sum_str_int_map(const stats::str_int_map& m)
 	return res;
 }
 
+int sum_cost_str_int_map(const stats::str_int_map &m)
+{
+	int cost = 0;
+	for (stats::str_int_map::const_iterator i = m.begin(); i != m.end(); ++i) {
+		cost += i->second * unit_types.find(i->first)->cost();
+	}
+
+	return cost;
+}
+
 } // end namespace statistics
 
