@@ -1,6 +1,6 @@
-/* $Id: team.cpp 41650 2010-03-20 21:53:36Z ai0867 $ */
+/* $Id: team.cpp 49342 2011-05-02 18:52:36Z crab $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -752,6 +752,11 @@ std::string team::get_side_highlight(int side)
 	return rgb2highlight(get_side_color_range(side+1).mid());
 }
 
+std::string team::get_side_highlight_pango(int side)
+{
+	return rgb2highlight_pango(get_side_color_range(side+1).mid());
+}
+
 void team::log_recruitable(){
 	LOG_NG << "Adding recruitable units: \n";
 	for (std::set<std::string>::const_iterator it = info_.can_recruit.begin();
