@@ -1,6 +1,6 @@
-/* $Id: theme.cpp 40489 2010-01-01 13:16:49Z mordante $ */
+/* $Id: theme.cpp 48834 2011-03-10 18:07:57Z dragonking $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -522,7 +522,7 @@ theme::menu::menu(const config& cfg) : object(cfg),
 }
 
 theme::theme(const config& cfg, const SDL_Rect& screen) :
-	theme_reset_("theme_reset"),
+	theme_reset_event_("theme_reset"),
 	cur_theme(),
 	cfg_(),
 	panels_(),
@@ -594,7 +594,7 @@ bool theme::set_resolution(const SDL_Rect& screen)
 			m->set_title(title_stash[m->get_id()]);
 	}
 
-	theme_reset_.notify_observers();
+	theme_reset_event_.notify_observers();
 
 	return result;
 }
