@@ -1,6 +1,6 @@
-/* $Id: theme.hpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: theme.hpp 48834 2011-03-10 18:07:57Z dragonking $ */
 /*
-   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -113,6 +113,7 @@ public:
 		using object::location;
 
 		const std::string& text() const { return text_; }
+		void set_text(const std::string& text) { text_ = text; }
 		const std::string& icon() const { return icon_; }
 
 		bool empty() const { return text_.empty() && icon_.empty(); }
@@ -186,7 +187,7 @@ public:
 
 		const std::vector<std::string>& items() const { return items_; }
 
-		void set_title(const std::string new_title) { title_ = new_title; }
+		void set_title(const std::string& new_title) { title_ = new_title; }
 	private:
 		bool context_;
 		std::string title_, tooltip_, image_, type_;
@@ -195,7 +196,7 @@ public:
 
 	explicit theme(const config& cfg, const SDL_Rect& screen);
 	bool set_resolution(const SDL_Rect& screen);
-	void modify(const config* cfg);
+	void modify(const config &cfg);
 
 	const std::vector<panel>& panels() const { return panels_; }
 	const std::vector<label>& labels() const { return labels_; }
@@ -208,8 +209,10 @@ public:
 	//If no menu entry is found, an empty menu object is returned.
 	menu* refresh_title(const std::string& id, const std::string& new_title);
 	menu* refresh_title2(const std::string& id, const std::string& title_tag);
+	void modify_label(const std::string& id, const std::string& text);
 
 	const status_item* get_status_item(const std::string& item) const;
+	const menu *get_menu_item(const std::string &key) const;
 
 	const SDL_Rect& main_map_location(const SDL_Rect& screen) const
 		{ return main_map_.location(screen); }
@@ -223,7 +226,7 @@ public:
 
 	const tborder& border() const { return border_; }
 
-	events::generic_event& theme_reset() { return theme_reset_; }
+	events::generic_event& theme_reset_event() { return theme_reset_event_; }
 
 private:
 	theme::object& find_element(std::string id);
@@ -233,7 +236,7 @@ private:
 
 	//notify observers that the theme has been rebuilt completely
 	//atm this is used for replay_controller to add replay controls to the standard theme
-	events::generic_event theme_reset_;
+	events::generic_event theme_reset_event_;
 
 	static std::map<std::string, config> known_themes;
 	std::string cur_theme;
