@@ -1,6 +1,6 @@
-/* $Id: cutter.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: cutter.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2004 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2011 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,8 @@
 
 #include "SDL_image.h"
 
+#include <iostream>
+
 namespace {
 
 	void print_usage(std::string name)
@@ -94,7 +96,10 @@ int main(int argc, char* argv[])
 
 	} catch(exploder_failure err) {
 		std::cerr << "Failed: " << err.message << "\n";
+		return 1;
 	}
+
+	return 0;
 }
 
 
