@@ -1,7 +1,7 @@
-/* $Id: tstring.cpp 31858 2009-01-01 10:27:41Z mordante $ */
+/* $Id: tstring.cpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
-   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2011 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,18 @@
 #include "global.hpp"
 
 #include <map>
+#include <vector>
 
 #include "tstring.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include <boost/functional/hash.hpp>
 
-#define LOG_CF lg::info(lg::config)
-#define ERR_CF lg::err(lg::config)
+static lg::log_domain log_config("config");
+#define LOG_CF LOG_STREAM(info, log_config)
+#define ERR_CF LOG_STREAM(err, log_config)
+
+static unsigned language_counter = 0;
 
 namespace {
 	const char TRANSLATABLE_PART = 0x01;
@@ -40,7 +45,27 @@ namespace {
 	std::map<std::string, unsigned int> textdomain_to_id;
 }
 
-t_string::walker::walker(const t_string& string) :
+size_t t_string_base::hash_value() const {
+	size_t seed = 0;
+	boost::hash_combine(seed, value_);
+	boost::hash_combine(seed, translatable_);
+	boost::hash_combine(seed, last_untranslatable_);
+	return seed;
+}
+
+t_string_base::walker::walker(const t_string& string) :
+	string_(string.get().value_),
+	begin_(0),
+	end_(string_.size()),
+	textdomain_(),
+	translatable_(false)
+{
+	if(string.get().translatable_) {
+		update();
+	}
+}
+
+t_string_base::walker::walker(const t_string_base& string) :
 	string_(string.value_),
 	begin_(0),
 	end_(string_.size()),
@@ -52,7 +77,7 @@ t_string::walker::walker(const t_string& string) :
 	}
 }
 
-t_string::walker::walker(const std::string& string) :
+t_string_base::walker::walker(const std::string& string) :
 	string_(string),
 	begin_(0),
 	end_(string_.size()),
@@ -62,7 +87,7 @@ t_string::walker::walker(const std::string& string) :
 	update();
 }
 
-void t_string::walker::update()
+void t_string_base::walker::update()
 {
 	unsigned int id;
 
@@ -139,36 +164,46 @@ void t_string::walker::update()
 	}
 }
 
-t_string::t_string() :
+t_string_base::t_string_base() :
 	value_(),
 	translated_value_(),
+	translation_timestamp_(0),
 	translatable_(false),
 	last_untranslatable_(false)
 {
 }
 
-t_string::t_string(const t_string& string) :
+t_string_base::t_string_base(const t_string_base& string) :
 	value_(string.value_),
 	translated_value_(string.translated_value_),
+	translation_timestamp_(string.translation_timestamp_),
 	translatable_(string.translatable_),
 	last_untranslatable_(string.last_untranslatable_)
 {
 }
 
-t_string::t_string(const std::string& string) :
+t_string_base::t_string_base(const std::string& string) :
 	value_(string),
 	translated_value_(),
+	translation_timestamp_(0),
 	translatable_(false),
 	last_untranslatable_(false)
 {
 }
 
-t_string::t_string(const std::string& string, const std::string& textdomain) :
+t_string_base::t_string_base(const std::string& string, const std::string& textdomain) :
 	value_(1, ID_TRANSLATABLE_PART),
 	translated_value_(),
+	translation_timestamp_(0),
 	translatable_(true),
 	last_untranslatable_(false)
 {
+	if (string.empty()) {
+		value_.clear();
+		translatable_ = false;
+		return;
+	}
+
 	std::map<std::string, unsigned int>::const_iterator idi = textdomain_to_id.find(textdomain);
 	unsigned int id;
 
@@ -185,17 +220,18 @@ t_string::t_string(const std::string& string, const std::string& textdomain) :
 	value_ += string;
 }
 
-t_string::t_string(const char* string) :
+t_string_base::t_string_base(const char* string) :
 	value_(string),
 	translated_value_(),
+	translation_timestamp_(0),
 	translatable_(false),
 	last_untranslatable_(false)
 {
 }
 
-t_string t_string::from_serialized(const std::string& string)
+t_string_base t_string_base::from_serialized(const std::string& string)
 {
-	t_string orig(string);
+	t_string_base orig(string);
 
 	if(!string.empty() && (string[0] == TRANSLATABLE_PART || string[0] == UNTRANSLATABLE_PART)) {
 		orig.translatable_ = true;
@@ -203,13 +239,13 @@ t_string t_string::from_serialized(const std::string& string)
 		orig.translatable_ = false;
 	}
 
-	t_string res;
+	t_string_base res;
 
 	for(walker w(orig); !w.eos(); w.next()) {
 		std::string substr(w.begin(), w.end());
 
 		if(w.translatable()) {
-			res += t_string(substr, w.textdomain());
+			res += t_string_base(substr, w.textdomain());
 		} else {
 			res += substr;
 		}
@@ -218,7 +254,7 @@ t_string t_string::from_serialized(const std::string& string)
 	return res;
 }
 
-std::string t_string::base_str() const
+std::string t_string_base::base_str() const
 {
 	std::string res;
 	for(walker w(*this); !w.eos(); w.next()) {
@@ -227,12 +263,12 @@ std::string t_string::base_str() const
 	return res;
 }
 
-std::string t_string::to_serialized() const
+std::string t_string_base::to_serialized() const
 {
-	t_string res;
+	t_string_base res;
 
 	for(walker w(*this); !w.eos(); w.next()) {
-		t_string chunk;
+		t_string_base chunk;
 
 		std::string substr(w.begin(), w.end());
 		if(w.translatable()) {
@@ -251,56 +287,61 @@ std::string t_string::to_serialized() const
 	return res.value();
 }
 
-t_string& t_string::operator=(const t_string& string)
+t_string_base& t_string_base::operator=(const t_string_base& string)
 {
 	value_ = string.value_;
+	translated_value_ = string.translated_value_;
+	translation_timestamp_ = string.translation_timestamp_;
 	translatable_ = string.translatable_;
 	last_untranslatable_ = string.last_untranslatable_;
-	translated_value_ = string.translated_value_;
 
 	return *this;
 }
 
-t_string& t_string::operator=(const std::string& string)
+t_string_base& t_string_base::operator=(const std::string& string)
 {
-	translatable_ = false;
-	value_ =  string;
-	translated_value_ = "";
-
-	return *this;
-}
-
-t_string& t_string::operator=(const char* string)
-{
-	translatable_ = false;
 	value_ = string;
 	translated_value_ = "";
+	translation_timestamp_ = 0;
+	translatable_ = false;
+	last_untranslatable_ = false;
 
 	return *this;
 }
 
-t_string t_string::operator+(const t_string& string) const
+t_string_base& t_string_base::operator=(const char* string)
 {
-	t_string res(*this);
+	value_ = string;
+	translated_value_ = "";
+	translation_timestamp_ = 0;
+	translatable_ = false;
+	last_untranslatable_ = false;
+
+	return *this;
+}
+
+t_string_base t_string_base::operator+(const t_string_base& string) const
+{
+	t_string_base res(*this);
 	res += string;
 	return res;
 }
 
-t_string t_string::operator+(const std::string& string) const
+t_string_base t_string_base::operator+(const std::string& string) const
 {
-	t_string res(*this);
+	t_string_base res(*this);
 	res += string;
 	return res;
 }
 
-t_string t_string::operator+(const char* string) const
+t_string_base t_string_base::operator+(const char* string) const
 {
-	t_string res(*this);
+	t_string_base res(*this);
 	res += string;
 	return res;
 }
 
-t_string& t_string::operator+=(const t_string& string)
+t_string_base& t_string_base::operator+=(const t_string_base& string)
 {
 	if (string.value_.empty())
 		return *this;
@@ -336,7 +377,7 @@ t_string& t_string::operator+=(const t_string& string)
 	return *this;
 }
 
-t_string& t_string::operator+=(const std::string& string)
+t_string_base& t_string_base::operator+=(const std::string& string)
 {
 	if (string.empty())
 		return *this;
@@ -359,7 +400,7 @@ t_string& t_string::operator+=(const std::string& string)
 	return *this;
 }
 
-t_string& t_string::operator+=(const char* string)
+t_string_base& t_string_base::operator+=(const char* string)
 {
 	if (string[0] == 0)
 		return *this;
@@ -383,14 +424,16 @@ t_string& t_string::operator+=(const char* string)
 }
 
 
-const std::string& t_string::str() const
+const std::string& t_string_base::str() const
 {
 	if(!translatable_)
 		return value_;
 
-	if(translatable_ && !translated_value_.empty())
+	if (translatable_ && !translated_value_.empty() && translation_timestamp_ == language_counter)
 		return translated_value_;
 
+	translated_value_.clear();
+
 	for(walker w(*this); !w.eos(); w.next()) {
 		std::string part(w.begin(), w.end());
 
@@ -401,11 +444,12 @@ const std::string& t_string::str() const
 		}
 	}
 
+	translation_timestamp_ = language_counter;
 	return translated_value_;
 }
 
 
-void t_string::add_textdomain(const std::string& name, const std::string& path)
+void t_string::add_textdomain(const std::string &name, const std::string &path)
 {
 	LOG_CF << "Binding textdomain " << name << " to path " << path << "\n";
 
@@ -414,7 +458,12 @@ void t_string::add_textdomain(const std::string& name, const std::string& path)
 	bind_textdomain_codeset(name.c_str(), "UTF-8");
 }
 
-std::ostream& operator<<(std::ostream& stream, const t_string& string)
+void t_string::reset_translations()
+{
+	++language_counter;
+}
+
+std::ostream& operator<<(std::ostream& stream, const t_string_base& string)
 {
 	stream << string.str();
 	return stream;
