@@ -1,6 +1,6 @@
-/* $Id: unit_animation.hpp 32290 2009-01-25 09:17:40Z boucman $ */
+/* $Id: unit_animation.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2006 - 2009 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
+   Copyright (C) 2006 - 2011 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -22,29 +22,32 @@
 #include <climits>
 #include <string>
 #include <vector>
+#include <set>
 
-class game_display;
 class attack_type;
+class game_display;
+class unit;
 
 class unit_animation
 {
 		/** Shouldn't be used so only declared. */
 		unit_animation();
 	public:
-		typedef enum { MATCH_FAIL=-2 , DEFAULT_ANIM=-1} variation_type;
+		typedef enum { MATCH_FAIL=-10 , DEFAULT_ANIM=-9} variation_type;
 		typedef enum { HIT, MISS, KILL, INVALID} hit_type;
 
 		static const std::vector<std::string>& all_tag_names();
 		static void fill_initial_animations( std::vector<unit_animation> & animations, const config & cfg);
 		static void add_anims( std::vector<unit_animation> & animations, const config & cfg);
 
-		int matches(const game_display &disp,const map_location& loc,const unit* my_unit,const std::string & event="",const int value=0,hit_type hit=INVALID,const attack_type* attack=NULL,const attack_type* second_attack = NULL, int swing_num =0) const;
+		int matches(const game_display &disp,const map_location& loc,const map_location& second_loc,const unit* my_unit,const std::string & event="",const int value=0,hit_type hit=INVALID,const attack_type* attack=NULL,const attack_type* second_attack = NULL, int value2 =0) const;
 
 
 		const unit_frame& get_last_frame() const{ return unit_anim_.get_last_frame() ; };
 		void add_frame(int duration, const unit_frame& value,bool force_change =false){ unit_anim_.add_frame(duration,value,force_change) ; };
 
 		bool need_update() const;
+		bool need_minimal_update() const;
 		bool animation_finished() const;
 		bool animation_finished_potential() const;
 		void update_last_draw_time();
@@ -53,7 +56,13 @@ class unit_animation
                 int time_to_tick(int animation_time) const { return unit_anim_.time_to_tick(animation_time); };
 		int get_animation_time() const{ return unit_anim_.get_animation_time() ; };
 		int get_animation_time_potential() const{ return unit_anim_.get_animation_time_potential() ; };
-		void start_animation(int start_time,const map_location &src = map_location::null_location, const map_location &dst = map_location::null_location , bool cycles=false, const std::string text="", const Uint32 text_color=0,const bool accelerate = true);
+		void start_animation(int start_time
+				, const map_location &src = map_location::null_location
+				, const map_location &dst = map_location::null_location
+				, bool cycles = false
+				, const std::string& text = ""
+				, const Uint32 text_color = 0
+				, const bool accelerate = true);
 		void update_parameters(const map_location &src, const map_location &dst);
                 void pause_animation();
                 void restart_animation();
@@ -66,9 +75,17 @@ class unit_animation
 	// reserved to class unit, for the special case of redrawing the unit base frame
 	const frame_parameters get_current_params(const frame_parameters & default_val = frame_parameters(),bool primary = true) const { return unit_anim_.parameters(default_val,primary); };
 	private:
-		static config prepare_animation(const config &cfg,const std::string animation_tag);
-		explicit unit_animation(const config& cfg,const std::string frame_string ="");
-		explicit unit_animation(int start_time,const unit_frame &frame,const std::string& event="",const int variation=DEFAULT_ANIM);
+		static config prepare_animation(const config &cfg
+				, const std::string& animation_tag);
+
+		explicit unit_animation(const config& cfg
+				,const std::string& frame_string = "");
+
+		explicit unit_animation(int start_time
+				, const unit_frame &frame
+				, const std::string& event = ""
+				, const int variation=DEFAULT_ANIM);
+
 		class particule:public animated<unit_frame>
 	{
 		public:
@@ -77,26 +94,32 @@ class unit_animation
 				accelerate(true),
 				parameters_(),
 				halo_id_(0),
-				last_frame_begin_time_(0),
-				invalidated_(false)
+				last_frame_begin_time_(0)
 				{};
-			explicit particule(const config& cfg,const std::string frame_string ="frame");
+			explicit particule(const config& cfg
+					, const std::string& frame_string ="frame");
+
 			virtual ~particule();
 			bool need_update() const;
-			void override(int start_time,int duration, const std::string highlight="", const std::string blend_ratio ="",Uint32 blend_color = 0,const std::string offset="",const std::string layer="");
+			bool need_minimal_update() const;
+			void override(int start_time
+					, int duration
+					, const std::string& highlight = ""
+					, const std::string& blend_ratio =""
+					, Uint32 blend_color = 0
+					, const std::string& offset = ""
+					, const std::string& layer = "");
 			void redraw( const frame_parameters& value,const map_location &src, const map_location &dst, const bool primary=false);
-			bool invalidate(const frame_parameters& value,const map_location &src, const map_location &dst,  const bool primary = false);
+			std::set<map_location> get_overlaped_hex(const frame_parameters& value,const map_location &src, const map_location &dst, const bool primary = false);
 			void start_animation(int start_time, bool cycles=false);
 			const frame_parameters parameters(const frame_parameters & default_val,bool primary) const { return get_current_frame().merge_parameters(get_current_frame_time(),parameters_.parameters(get_animation_time()-get_begin_time()),default_val,primary); };
 			bool accelerate;
 		private:
 
 			//animation params that can be locally overridden by frames
-			frame_builder parameters_;
+			frame_parsed_parameters parameters_;
 			int halo_id_;
 			int last_frame_begin_time_;
-			// optimisation
-			bool invalidated_;
 
 	};
 		t_translation::t_list terrain_types_;
@@ -110,12 +133,16 @@ class unit_animation
 		std::vector<config> primary_attack_filter_;
 		std::vector<config> secondary_attack_filter_;
 		std::vector<hit_type> hits_;
-		std::vector<int> swing_num_;
+		std::vector<int> value2_;
 		std::map<std::string,particule> sub_anims_;
 		particule unit_anim_;
 		/* these are drawing parameters, but for efficiancy reason they are in the anim and not in the particle */
 		map_location src_;
 		map_location dst_;
+		// optimisation
+		bool invalidated_;
+		bool play_offscreen_;
+		std::set<map_location> overlaped_hex_;
 };
 
 class unit_animator
@@ -128,24 +155,45 @@ class unit_animator
 		}
 
 
-		void add_animation(unit* animated_unit,const std::string& event,
-				const map_location &src = map_location::null_location,
-				const int value=0,bool with_bars = false,bool cycles = false,
-				const std::string text="",const Uint32 text_color=0,
-				const unit_animation::hit_type hit_type = unit_animation::INVALID,
-				const attack_type* attack=NULL, const attack_type* second_attack = NULL,
-				int swing_num =0);
-		void replace_anim_if_invalid(unit* animated_unit,const std::string& event,
-				const map_location &src = map_location::null_location,
-				const int value=0,bool with_bars = false,bool cycles = false,
-				const std::string text="",const Uint32 text_color=0,
-				const unit_animation::hit_type hit_type = unit_animation::INVALID,
-				const attack_type* attack=NULL, const attack_type* second_attack = NULL,
-				int swing_num =0);
+		void add_animation(unit* animated_unit
+				, const unit_animation * animation
+				, const map_location &src = map_location::null_location
+				, bool with_bars = false
+				, bool cycles = false
+				, const std::string& text = ""
+				, const Uint32 text_color = 0);
+		void add_animation(unit* animated_unit
+				, const std::string& event
+				, const map_location &src = map_location::null_location
+				, const map_location &dst = map_location::null_location
+				, const int value = 0
+				, bool with_bars = false
+				, bool cycles = false
+				, const std::string& text = ""
+				, const Uint32 text_color = 0
+				, const unit_animation::hit_type hit_type =
+					unit_animation::INVALID
+				, const attack_type* attack = NULL
+				, const attack_type* second_attack = NULL
+				, int value2 = 0);
+		void replace_anim_if_invalid(unit* animated_unit
+				, const std::string& event
+				, const map_location &src = map_location::null_location
+				, const map_location &dst = map_location::null_location
+				, const int value = 0
+				, bool with_bars = false
+				, bool cycles = false
+				, const std::string& text = ""
+				, const Uint32 text_color = 0
+				, const unit_animation::hit_type hit_type =
+					unit_animation::INVALID
+				, const attack_type* attack = NULL
+				, const attack_type* second_attack = NULL
+				, int value2 = 0);
 		void start_animations();
                 void pause_animation();
                 void restart_animation();
-		void empty(){start_time_ = INT_MIN ; animated_units_.clear();};
+		void clear(){start_time_ = INT_MIN ; animated_units_.clear();};
 		void set_all_standing();
 
 
