@@ -1,6 +1,6 @@
-/* $Id: unit_types.cpp 44190 2010-07-15 16:23:56Z silene $ */
+/* $Id: unit_types.cpp 49411 2011-05-07 15:11:50Z soliton $ */
 /*
-   Copyright (C) 2003 - 2010 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -1189,9 +1189,8 @@ void unit_type_data::set_config(config &cfg)
             */
 		}
         // we insert an empty unit_type and build it after the copy (for performance)
-        std::pair<unit_type_map::iterator,bool> insertion =
+        //std::pair<unit_type_map::iterator,bool> insertion =
             insert(std::pair<const std::string,unit_type>(id,unit_type()));
-        unit_type_map::iterator itor = types_.find(id);
         //	if (!insertion.second)
         // TODO: else { warning for multiple units with same id}
         LOG_CONFIG << "added " << id << " to unit_type list (unit_type_data.unit_types)\n";
