@@ -1,7 +1,7 @@
-/* $Id: variable.hpp 33100 2009-02-26 01:34:22Z sapient $ */
+/* $Id: variable.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2011 by Philippe Plantier <ayin@anathas.org>
 
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -44,11 +44,11 @@ public:
  */
 	vconfig();
 	vconfig(const vconfig& v);
-	vconfig(const config* cfg, bool is_volatile=false);
+	explicit vconfig(const config &cfg, bool is_volatile=false);
 	~vconfig();
 
 	vconfig& operator=(const vconfig& cfg);
-	vconfig& operator=(const config* cfg);
+	vconfig& operator=(const config &cfg);
 
 	bool null() const { return cfg_ == NULL; }
 	bool is_volatile() const { return cache_key_ != NULL; }
@@ -60,13 +60,20 @@ public:
 	vconfig child(const std::string& key) const;
 	bool has_child(const std::string& key) const;
 
-	/* Note: vconfig::operator[] returns const, and this should not be changed
-	   because vconfig is often used as a drop-in replacement for config, and
-	   this const will properly warn you if you try to assign vcfg["key"]=val;
-	*/
+	/**
+	 * Note: vconfig::operator[] returns const, and this should not be changed
+	 * because vconfig is often used as a drop-in replacement for config, and
+	 * this const will properly warn you if you try to assign vcfg["key"]=val;
+	 *
+	 * Note: The following construction is unsave:
+	 * const std::string& temp = vcfg["foo"];
+	 * This bind temp to a member of a temporary t_string. The lifetime of the
+	 * temporary is not extended by this reference binding and the temporary's
+	 * lifetime ends which causes UB. Instead use:
+	 * const std::string temp = vcfg["foo"];
+	 */
 	const t_string operator[](const std::string& key) const { return expand(key); }
 	const t_string expand(const std::string&) const; /** < Synonym for operator[] */
-	const t_string& get_attribute(const std::string& key) const { return (*cfg_)[key]; }
 	bool has_attribute(const std::string& key) const { return cfg_->has_attribute(key); }
 	bool empty() const { return (null() || cfg_->empty()); }
 
@@ -77,7 +84,7 @@ public:
 		typedef std::auto_ptr<value_type> pointer;
 		typedef value_type& reference;
 		typedef config::all_children_iterator Itor;
-		explicit all_children_iterator(Itor i=Itor(), const config* cache_key=NULL);
+		explicit all_children_iterator(const Itor &i, const config *cache_key = NULL);
 
 		all_children_iterator& operator++();
 		all_children_iterator  operator++(int);
@@ -116,13 +123,11 @@ namespace variable
 {
 
 /**
- * Used to for the functions in variable.cpp to locate the current global
- * variable repository
+ * Used to clear the cache for variables.
  */
 class manager
 {
 public:
-	manager(game_state* repository);
 	~manager();
 };
 
@@ -148,11 +153,11 @@ private:
 class scoped_weapon_info : public scoped_wml_variable
 {
 public:
-	scoped_weapon_info(const std::string& var_name, const config* data)
+	scoped_weapon_info(const std::string& var_name, const config &data)
 		: scoped_wml_variable(var_name), data_(data) {}
 	void activate();
 private:
-	config const* const data_;
+	config const &data_;
 };
 
 class scoped_xy_unit : public scoped_wml_variable
