@@ -1,6 +1,6 @@
-/* $Id: textbox.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: textbox.hpp 48450 2011-02-08 20:55:18Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2011 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,7 @@
 
 #include "../serialization/string_utils.hpp"
 #include "../sdl_utils.hpp"
+#include "font.hpp"
 
 #include "scrollarea.hpp"
 
@@ -31,10 +32,9 @@ public:
 	virtual ~textbox();
 
 	const std::string text() const;
-	void set_text(const std::string& text);
-	void append_text(const std::string& text,bool auto_scroll = false);
+	void set_text(const std::string& text, const SDL_Color& color =font::NORMAL_COLOUR);
+	void append_text(const std::string& text,bool auto_scroll = false, const SDL_Color& color =font::NORMAL_COLOUR);
 	void clear();
-	void process();
 
 	void set_editable(bool value);
 	bool editable() const;
@@ -43,12 +43,6 @@ public:
 
 	void set_wrap(bool val);
 
-	void set_location(const SDL_Rect& rect);
-
-	//VC++ doesn't like a 'using scrollarea::set_location' directive here, so we declare
-	//an inline forwarding function instead
-	void set_location(int x, int y) { widget::set_location(x,y); }
-
 protected:
 	virtual void draw_contents();
 	virtual void set_inner_location(SDL_Rect const &);
@@ -91,8 +85,8 @@ private:
 	void handle_event(const SDL_Event& event);
 
 	void draw_cursor(int pos, CVideo &video) const;
-	void update_text_cache(bool reset = false);
-	surface add_text_line(const wide_string& text);
+	void update_text_cache(bool reset = false, const SDL_Color& color =font::NORMAL_COLOUR);
+	surface add_text_line(const wide_string& text, const SDL_Color& color =font::NORMAL_COLOUR);
 	bool is_selection();
 	void erase_selection();
 
